{ $Id: CellType.pas,v 1.14 2001/06/18 05:14:19 laa Exp $}

{
    This file is part of the TTranslator 
    Copyright (C) 2001 Polycon Ab

    TTranslator is free software; you can redistribute it and/or modify
    it under the terms of the version 2 of the GNU General Public License
    as published by the Free Software Foundation. For more information
    about licensing, please refer to
    http://www.polycon.fi/translator/licensing.html

    TTranslator is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Foobar; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
}

unit CellType;

interface
{$ifndef WEBAPPLICATION}

uses
  Windows, Graphics, Classes, Controls,
  PGInterfaces, PGTypes, DataType, DataTypes, DerivedDataType;

type

  TCellType = class
  protected
    function GetReadOnly : Boolean; virtual;
    function GetHasIcon : Boolean; virtual;
    function GetHasButton : Boolean; virtual;
  public
    constructor Create;
    destructor Destroy; override;

    procedure RenderEditor(Canvas : TCanvas; EditorInterface : IInplaceEdit;
              InfoSupplier : ICellTypeInfoSupplier); virtual;
    class procedure RenderValue(AValue : TValue; Canvas : TCanvas; var Rect : TRect); virtual; abstract;

    procedure Activate(EditorInterface : IInplaceEdit; InfoSupplier : ICellTypeInfoSupplier); virtual;
    procedure RenderIcon(IconInfoSupplier : IIconInfoSupplier; Canvas : TCanvas; var Rect : TRect); virtual;
    procedure KeyDown(var Key: Word; Shift: TShiftState; EditorInterface : IInplaceEdit;
              InfoSupplier : ICellTypeInfoSupplier; var ExecInherited : Boolean); virtual;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer; EditorInterface : IInplaceEdit;
              InfoSupplier : ICellTypeInfoSupplier; var ExecInherited : Boolean); virtual;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer; EditorInterface : IInplaceEdit;
              InfoSupplier : ICellTypeInfoSupplier; var ExecInherited : Boolean); virtual;
    procedure Click;{(Formatter : TPGDataFormatter; PolyGrid : TPolyGrid);} virtual; abstract;
    function MinHeight : Integer; virtual;
    function MinWidth : Integer; virtual;

    property ReadOnly : Boolean read GetReadOnly;
    property HasIcon : Boolean read GetHasIcon;
    property HasButton : Boolean read GetHasButton;
  end;

  TEditableCellType = class(TCellType)
  protected
    function GetReadOnly : Boolean; override;
    constructor Create;
  public
    class procedure RenderValue(AValue : TValue; Canvas : TCanvas; var Rect : TRect); override;
    procedure Click{(Formatter : TPGDataFormatter; PolyGrid : TPolyGrid)}; override;
  end;

  TPictureCellType = class(TCellType)
  protected
    constructor Create;

    function GetReadOnly : Boolean; override;
  public
    class procedure RenderValue(AValue : TValue; Canvas : TCanvas; var Rect : TRect); override;
    procedure Click{(Formatter : TPGDataFormatter; PolyGrid : TPolyGrid)}; override;
  end;

  TIconCellType = class(TCellType)
  private
    FActiveIcon : TPicture;
    FDefaultIconAlign : TCellAlign;
  protected
    constructor Create;

    function GetReadOnly : Boolean; override;
  public
    destructor Destroy; override;
    class procedure RenderValue(AValue : TValue; Canvas : TCanvas; var Rect : TRect); override;
    procedure RenderIcon(IconInfoSupplier : IIconInfoSupplier; Canvas : TCanvas; var Rect : TRect); override;

    function MinHeight : Integer; override;
    function MinWidth : Integer; override;
  end;

  TNormalCellType = class(TEditableCellType)
  protected
    constructor Create;
  public
    procedure Click{(Formatter : TPGDataFormatter; PolyGrid : TPolyGrid)}; override;
  end;

  THeaderCellType = class(TIconCellType)
  protected
    constructor Create;
  public
    destructor Destroy; override;
    procedure RenderIcon(IconInfoSupplier : IIconInfoSupplier; Canvas : TCanvas; var Rect : TRect); override;
    procedure Click{(Formatter : TPGDataFormatter; PolyGrid : TPolyGrid)}; override;
  end;

  TCheckCellType = class(TIconCellType)
  private
    FCheckedBitmap : TPicture;
    FUncheckedBitmap : TPicture;
  protected
    constructor Create;
  public
    destructor Destroy; override;
    procedure RenderIcon(IconInfoSupplier : IIconInfoSupplier; Canvas : TCanvas; var Rect : TRect); override;
    procedure Click{(Formatter : TPGDataFormatter; PolyGrid : TPolyGrid)}; override;
  end;

  TButtonEditCellType = class(TNormalCellType)
  private
    FDefaultFace : TPicture;
    FButtonWidth : Integer;
  protected
    constructor Create;
    function CreateDefaultFace : TPicture; virtual; abstract;
    property DefaultFace : TPicture read FDefaultFace;
    property ButtonWidth : Integer read FButtonWidth;
  public
    destructor Destroy; override;
    procedure RenderEditor(Canvas : TCanvas; EditorInterface : IInplaceEdit;
              InfoSupplier : ICellTypeInfoSupplier); override;
    procedure RenderButton(Canvas : TCanvas; ARect : TRect; EditorInterface : IInplaceEdit;
              InfoSupplier : ICellTypeInfoSupplier; AFace : TPicture); virtual; abstract;
    procedure Click{(Formatter : TPGDataFormatter; PolyGrid : TPolyGrid)}; override;
    procedure ButtonClick(EditorInterface : IInplaceEdit;
              InfoSupplier : ICellTypeInfoSupplier); virtual;
  end;

  TComboCellType = class(TButtonEditCellType)
  protected
    constructor Create;
    function CreateDefaultFace : TPicture; override;
  public
    procedure RenderButton(Canvas : TCanvas; ARect : TRect; EditorInterface : IInplaceEdit;
              InfoSupplier : ICellTypeInfoSupplier; AFace : TPicture); override;
    procedure Activate(EditorInterface : IInplaceEdit; InfoSupplier : ICellTypeInfoSupplier); override;
//    procedure KeyDown(var Key: Word; Shift: TShiftState; EditorInterface : IInplaceEdit;
//              InfoSupplier : ICellTypeInfoSupplier; var ExecInherited : Boolean); override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer; EditorInterface : IInplaceEdit;
              InfoSupplier : ICellTypeInfoSupplier; var ExecInherited : Boolean); override;
    procedure Click{(Formatter : TPGDataFormatter; PolyGrid : TPolyGrid)}; override;
  end;

  TButtonCellType = class(TButtonEditCellType)
  protected
    constructor Create;
    function CreateDefaultFace : TPicture; override;
  public
    procedure RenderButton(Canvas : TCanvas; ARect : TRect; EditorInterface : IInplaceEdit;
              InfoSupplier : ICellTypeInfoSupplier; AFace : TPicture); override;
    procedure Activate(EditorInterface : IInplaceEdit; InfoSupplier : ICellTypeInfoSupplier); override;
    procedure KeyDown(var Key: Word; Shift: TShiftState; EditorInterface : IInplaceEdit;
              InfoSupplier : ICellTypeInfoSupplier; var ExecInherited : Boolean); override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer; EditorInterface : IInplaceEdit;
              InfoSupplier : ICellTypeInfoSupplier; var ExecInherited : Boolean); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer; EditorInterface : IInplaceEdit;
              InfoSupplier : ICellTypeInfoSupplier; var ExecInherited : Boolean); override;
    procedure ButtonClick(EditorInterface : IInplaceEdit;
              InfoSupplier : ICellTypeInfoSupplier); override;
  end;

var
  PictureCellType : TPictureCellType;
  NormalCellType : TNormalCellType;
  HeaderCellType : THeaderCellType;
  CheckCellType : TCheckCellType;
  ComboCellType : TComboCellType;
  ButtonCellType : TButtonCellType;

{$endif WEBAPPLICATION}
implementation
{$ifndef WEBAPPLICATION}

uses
  Math, PGDataFormatter, Messages, Buttons;

const
  ICON_MARGIN = 2;

{ ------------------------- TCellType ----------------------------- }

constructor TCellType.Create;
begin
  Inherited Create;
end;

destructor TCellType.Destroy;
begin
  Inherited Destroy;
end;

function TCellType.GetReadOnly : Boolean;
begin
  Result := False;
end;

function TCellType.GetHasIcon : Boolean;
begin
  Result := False;
end;

function TCellType.GetHasButton : Boolean;
begin
  Result := False;
end;

procedure TCellType.RenderEditor(Canvas : TCanvas; EditorInterface : IInplaceEdit;
    InfoSupplier : ICellTypeInfoSupplier);
begin
  // Nothing
end;

procedure TCellType.Activate(EditorInterface : IInplaceEdit; InfoSupplier : ICellTypeInfoSupplier);
begin
  // Nothing
end;

procedure TCellType.RenderIcon(IconInfoSupplier : IIconInfoSupplier; Canvas : TCanvas; var Rect : TRect);
begin
  // Nothing
end;

function TCellType.MinHeight : Integer;
begin
  Result := 0;
end;

function TCellType.MinWidth : Integer;
begin
  Result := 0;
end;

procedure TCellType.KeyDown(var Key: Word; Shift: TShiftState; EditorInterface : IInplaceEdit;
  InfoSupplier : ICellTypeInfoSupplier; var ExecInherited : Boolean);
begin
  ExecInherited := True;
end;

procedure TCellType.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer;
      EditorInterface : IInplaceEdit; InfoSupplier : ICellTypeInfoSupplier; var ExecInherited : Boolean);
begin
  // Nothing
end;

procedure TCellType.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer; EditorInterface : IInplaceEdit;
      InfoSupplier : ICellTypeInfoSupplier; var ExecInherited : Boolean);
begin
  EditorInterface.StopTracking;
end;

{ ------------------------- TEditableCellType ----------------------------- }

constructor TEditableCellType.Create;
begin
  inherited Create;
end;

function TEditableCellType.GetReadOnly : Boolean;
begin
  Result := False;
end;

class procedure TEditableCellType.RenderValue(AValue : TValue; Canvas : TCanvas; var Rect : TRect);
begin
  Canvas.TextOut(Rect.Left, Rect.Top, AsString(AValue) );
end;

procedure TEditableCellType.Click;
begin
end;


{ ------------------------- TPictureCellType ----------------------------- }

constructor TPictureCellType.Create;
begin
  inherited Create;
end;

function TPictureCellType.GetReadOnly : Boolean;
begin
  Result := True;
end;

class procedure TPictureCellType.RenderValue(AValue : TValue; Canvas : TCanvas; var Rect : TRect);
var
  APicture : TPicture;
begin
  APicture := AsPicture(AValue);
  if APicture <> nil then
    Canvas.Draw( Rect.Left, Rect.Top, APicture.Graphic);
end;

procedure TPictureCellType.Click{(Formatter : TPGDataFormatter; PolyGrid : TPolyGrid)};
begin
end;

{ ------------------------- TIconCellType ----------------------------- }

constructor TIconCellType.Create;
begin
  inherited Create;

  FActiveIcon := TPicture.Create;
  FDefaultIconAlign := caNone;
end;

destructor TIconCellType.Destroy;
begin
  FActiveIcon.Free; // LAA:tillsatt
  inherited Destroy;
end;

function TIconCellType.GetReadOnly : Boolean;
begin
  Result := True;
end;

class procedure TIconCellType.RenderValue(AValue : TValue; Canvas : TCanvas; var Rect : TRect);
begin
  if AValue.DataType is TPictureType then
    TPictureCellType.RenderValue(AValue, Canvas, Rect)
  else
    TNormalCellType.RenderValue(AValue, Canvas, Rect);
end;

procedure TIconCellType.RenderIcon(IconInfoSupplier : IIconInfoSupplier; Canvas : TCanvas; var Rect : TRect);
var
  ClipRect : TRect;
  Left, RectLeft, RectRight : Integer;
begin
  if FActiveIcon = nil then
    Exit;

  ClipRect := Rect;

  { Special case: checkbox. Note that combo dropdown button & ...-buttons
    are drawn by the editor (and thus only for the current cell!), whereas
    checkboxes are always drawn }

  IntersectClipRect( Canvas.handle, ClipRect.Left, ClipRect.top,
                     ClipRect.right, ClipRect.bottom );

  RectLeft := Rect.Left;
  RectRight := Rect.Right;

  case IconInfoSupplier.GetIconAlign(Self, FDefaultIconAlign) of
    caCenter:
    begin
      Left := MaxIntValue([(RectLeft + RectRight - FActiveIcon.Width) div 2, 0]);
      RectLeft := Left - ICON_MARGIN;
    end;
    caLeft:
    begin
      Left := RectLeft;
      RectLeft := Left + FActiveIcon.Width + ICON_MARGIN;
    end;
    else
    begin
      Left := MaxIntValue([RectRight - FActiveIcon.Width, 0]);
      RectRight := Left - ICON_MARGIN;
    end;
  end;

  Canvas.Draw( Left, (Rect.Top + Rect.Bottom - FActiveIcon.Height) div 2, FActiveIcon.Graphic);
  Rect.Left := RectLeft;
  Rect.Right := RectRight;
end;

function TIconCellType.MinHeight : Integer;
begin
  Result := 0;
  if FActiveIcon <> nil then
    Result := FActiveIcon.Height;
end;

function TIconCellType.MinWidth : Integer;
begin
  Result := 0;
  if FActiveIcon <> nil then
    Result := FActiveIcon.Width;
end;

{ ------------------------- TNormalCellType ----------------------------- }

constructor TNormalCellType.Create;
begin
  inherited Create;
end;

procedure TNormalCellType.Click{(Formatter : TPGDataFormatter; PolyGrid : TPolyGrid)};
begin
end;


{ ------------------------- THeaderCellType ----------------------------- }

constructor THeaderCellType.Create;
begin
  inherited Create;

{  FUpArrowBitmap := TPicture.Create;
  FDownArrowBitmap := TPicture.Create;
  ABitmap := TPicture.Create;

  ABitmap.Handle := LoadBitmap(0, PChar(OBM_UPARROWI));
  FUpArrowBitmap.Height := 10;
  FUpArrowBitmap.Width := 10;
  FUpArrowBitmap.Canvas.CopyRect(Rect(0,0,10,10), ABitmap.Canvas, Rect(4,2,14,12));
  ABitmap.Handle := LoadBitmap(0, PChar(OBM_DNARROWI));
  FDownArrowBitmap.Height := 10;
  FDownArrowBitmap.Width := 10;
  FDownArrowBitmap.Canvas.CopyRect(Rect(0,0,10,10), ABitmap.Canvas, Rect(4,2,14,12));
  ABitmap.Free;
                               }
  FDefaultIconAlign := caRight;
end;

destructor THeaderCellType.Destroy;
begin
  inherited Destroy;
end;

procedure THeaderCellType.RenderIcon(IconInfoSupplier : IIconInfoSupplier; Canvas : TCanvas; var Rect : TRect);
begin
  if IconInfoSupplier.GetIcon(Self, FActiveIcon) then
    Inherited RenderIcon(IconInfoSupplier, Canvas, Rect);
end;

procedure THeaderCellType.Click{(Formatter : TPGDataFormatter; PolyGrid : TPolyGrid)};
begin
end;

{ ------------------------- TCheckCellType ----------------------------- }

constructor TCheckCellType.Create;
var
  APicture : TPicture;
begin
  inherited Create;

  FCheckedBitmap := TPicture.Create;
  FUncheckedBitmap := TPicture.Create;
  APicture := TPicture.Create;
  { checkBitmap = bitmap containing windows radio & checkboxes in various states of checking}
  APicture.Bitmap.Handle := LoadBitmap(0, PChar(OBM_CHECKBOXES)); {DELPHI2 32759}
  FUncheckedBitmap.Bitmap.Height := 12;
  FUncheckedBitmap.Bitmap.Width := 12;
  FUncheckedBitmap.Bitmap.Canvas.CopyRect(Rect(0,0,12,12), APicture.Bitmap.Canvas, Rect(0,0,12,12));
  FCheckedBitmap.Bitmap.Height := 12;
  FCheckedBitmap.Bitmap.Width := 12;
  FCheckedBitmap.Bitmap.Canvas.CopyRect(Rect(0,0,12,12), APicture.Bitmap.Canvas, Rect(13,0,25,12));
  APicture.Free;

  FDefaultIconAlign := caLeft;
end;

destructor TCheckCellType.Destroy;
begin
  inherited Destroy;

  FCheckedBitmap.Free;
  FUncheckedBitmap.Free;
end;

procedure TCheckCellType.RenderIcon(IconInfoSupplier : IIconInfoSupplier; Canvas : TCanvas; var Rect : TRect);
begin
  if IconInfoSupplier.GetChecked then
    FActiveIcon.Assign(FCheckedBitmap)
  else
    FActiveIcon.Assign(FUncheckedBitmap);

  Inherited RenderIcon(IconInfoSupplier, Canvas, Rect);
end;

procedure TCheckCellType.Click{(Formatter : TPGDataFormatter; PolyGrid : TPolyGrid)};
begin
end;

{ ------------------------- TButtonEditCellType ----------------------------- }

constructor TButtonEditCellType.Create;
begin
  inherited Create;
  
  FButtonWidth := GetSystemMetrics(SM_CXVSCROLL);
  FDefaultFace := CreateDefaultFace;
end;

destructor TButtonEditCellType.Destroy;
begin
  Inherited Destroy;
  FDefaultFace.Free;
end;

procedure TButtonEditCellType.RenderEditor(Canvas : TCanvas; EditorInterface : IInplaceEdit;
    InfoSupplier : ICellTypeInfoSupplier);
var
  ARect: TRect;
  Width, Height : Integer;
  AFace, Pic : TPicture;
begin
  EditorInterface.GetSize(Width, Height);
  SetRect(ARect, Width - ButtonWidth, 0, Width, Height);
  EditorInterface.MapEditorToParent(ARect);

  AFace := TPicture.Create;
  if not InfoSupplier.GetEditorFace(Self, AFace) then
    Pic := DefaultFace
  else
    Pic := AFace;

  RenderButton(Canvas, ARect, EditorInterface, InfoSupplier, Pic);
  AFace.Free;

  SetRect(ARect, 2 , 2, Width - 2 - ButtonWidth, Height);
  EditorInterface.AfterRenderEditor(ARect);
end;

procedure TButtonEditCellType.Click{(Formatter : TPGDataFormatter; PolyGrid : TPolyGrid)};
begin
end;

procedure TButtonEditCellType.ButtonClick(EditorInterface : IInplaceEdit;
              InfoSupplier : ICellTypeInfoSupplier);
begin
  // Nothing
end;

{ ------------------------- TComboCellType ----------------------------- }

constructor TComboCellType.Create;
begin
  inherited Create;
end;

function TComboCellType.CreateDefaultFace : TPicture;
begin
  Result := TPicture.Create;
  Result.Bitmap.Handle := LoadBitmap(0, PChar(OBM_COMBO)); {DELPHI2: 32738}
end;

procedure TComboCellType.RenderButton(Canvas : TCanvas; ARect : TRect;
    EditorInterface : IInplaceEdit; InfoSupplier : ICellTypeInfoSupplier; AFace : TPicture);
var
  BLeft,BTop: Integer;
begin
  DrawButtonFace(Canvas, ARect, 0, bsAutoDetect, False, EditorInterface.EditorPressed, False);

  BLeft := ARect.Left + (ARect.Right - ARect.Left - AFace.Width - 3);
  BTop := ARect.Top + (ARect.Bottom - ARect.Top - AFace.Height) div 2;
  Canvas.Draw(BLeft, BTop, AFace.Graphic);
end;

procedure TComboCellType.Activate(EditorInterface : IInplaceEdit; InfoSupplier : ICellTypeInfoSupplier);
begin
  EditorInterface.InitListBox(InfoSupplier.GetComboStrings);
end;

procedure TComboCellType.Click{(Formatter : TPGDataFormatter; PolyGrid : TPolyGrid)};
begin
end;

{procedure TComboCellType.KeyDown(var Key: Word; Shift: TShiftState; EditorInterface : IInplaceEdit;
  InfoSupplier : ICellTypeInfoSupplier; var ExecInherited : Boolean);
begin
end;
}
procedure TComboCellType.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer;
    EditorInterface : IInplaceEdit; InfoSupplier : ICellTypeInfoSupplier; var ExecInherited : Boolean);
begin
  EditorInterface.ComboMouseDown(Button, Shift, X, Y);
  ExecInherited := True;
end;

{ ------------------------- TButtonCellType ----------------------------- }

constructor TButtonCellType.Create;
begin
  inherited Create;
end;

function TButtonCellType.CreateDefaultFace : TPicture;
var
  bmpTemp : TBitmap;
begin
  Result := TPicture.Create;

  bmpTemp := TBitmap.Create;
  bmpTemp.Handle := LoadBitmap(0, PChar(OBM_REDUCE)); {DELPHI2 32759}

  Result.Bitmap.Height := 12;
  Result.Bitmap.Width := 12;
  Result.Bitmap.Canvas.CopyRect(Rect(1,7,3,9), bmpTemp.Canvas, Rect(5,11,7,13));
  Result.Bitmap.Canvas.CopyRect(Rect(5,7,7,9), bmpTemp.Canvas, Rect(5,11,7,13));
  Result.Bitmap.Canvas.CopyRect(Rect(9,7,11,9), bmpTemp.Canvas, Rect(5,11,7,13));

{  DeleteObject( bmpTemp.Handle );
  bmpTemp.Handle := 0; }

  bmpTemp.Free;
end;

procedure TButtonCellType.RenderButton(Canvas : TCanvas; ARect : TRect;
    EditorInterface : IInplaceEdit; InfoSupplier : ICellTypeInfoSupplier; AFace : TPicture);
var
  BLeft,BTop: Integer;
  OldMode : TCopyMode;
begin
  OldMode := Canvas.CopyMode;
  Canvas.CopyMode := cmSrcAnd;

  DrawButtonFace(Canvas, ARect, 0, bsAutoDetect, False, EditorInterface.EditorPressed, False);
  BLeft := ARect.Left + (ARect.Right - ARect.Left - AFace.Width - 3);
  BTop := ARect.Top + (ARect.Bottom - ARect.Top - AFace.Height) div 2;
  Canvas.Draw(BLeft, BTop, AFace.Graphic);

  Canvas.CopyMode := OldMode;
end;

procedure TButtonCellType.Activate(EditorInterface : IInplaceEdit; InfoSupplier : ICellTypeInfoSupplier);
begin
end;

procedure TButtonCellType.ButtonClick(EditorInterface : IInplaceEdit;
    InfoSupplier : ICellTypeInfoSupplier);
var
  AValue : TValue;
begin
  AValue := ValueFromString(EditorInterface.GetText);
  InfoSupplier.EditButtonClick(AValue);
  EditorInterface.SetText(AsString(AValue));
end;

procedure TButtonCellType.KeyDown(var Key: Word; Shift: TShiftState;
  EditorInterface : IInplaceEdit; InfoSupplier : ICellTypeInfoSupplier; var ExecInherited : Boolean);
begin
  if ( (Key = VK_RETURN) and
       (Shift = [ssCtrl]) ) or
     ( (Key = VK_DOWN) and
       (Shift = [ssAlt]) ) then
  begin
    ButtonClick(EditorInterface, InfoSupplier);
    EditorInterface.AfterButtonKeyDown;
    ExecInherited := False;
  end;
end;

procedure TButtonCellType.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer;
    EditorInterface : IInplaceEdit; InfoSupplier : ICellTypeInfoSupplier; var ExecInherited : Boolean);
begin
  EditorInterface.ButtonMouseDown(Button, Shift, X, Y);
  ExecInherited := True;
end;

procedure TButtonCellType.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer;
    EditorInterface : IInplaceEdit; InfoSupplier : ICellTypeInfoSupplier; var ExecInherited : Boolean);
var
  WasPressed : Boolean;
begin
  WasPressed := EditorInterface.EditorPressed;
  EditorInterface.StopTracking;
  if (Button = mbLeft) and WasPressed then
    ButtonClick(EditorInterface, InfoSupplier);
  ExecInherited := True;
end;

initialization

  PictureCellType := TPictureCellType.Create;
  NormalCellType := TNormalCellType.Create;
  ButtonCellType := TButtonCellType.Create;
  HeaderCellType := THeaderCellType.Create;
  CheckCellType := TCheckCellType.Create;
  ComboCellType := TComboCellType.Create;

finalization

  PictureCellType.Free;
  NormalCellType.Free;
  ButtonCellType.Free;
  HeaderCellType.Free;
  CheckCellType.Free;
  ComboCellType.Free;

{$endif WEBAPPLICATION}
end.

