unit Main;

interface

uses
  WinProcs, WinTypes, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  StdCtrls, ExtCtrls, Preview, Menus;

type
  TForm1 = class(TForm)
    Panel1: TPanel;
    ComboBox1: TComboBox;
    Button2: TButton;
    Label1: TLabel;
    Label3: TLabel;
    Image1: TImage;
    Button3: TButton;
    Button4: TButton;
    PrintPreview: TPrintPreview;
    ComboBox2: TComboBox;
    Label2: TLabel;
    CheckBox1: TCheckBox;
    Image2: TImage;
    CheckBox2: TCheckBox;
    procedure FormCreate(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure ComboBox1Change(Sender: TObject);
    procedure ComboBox2Change(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure CheckBox1Click(Sender: TObject);
    procedure CheckBox2Click(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure Button4Click(Sender: TObject);
    procedure PrintPreviewChange(Sender: TObject);
    procedure PrintPreviewNewPage(Sender: TObject);
    procedure PrintPreviewPrintProgress(Sender: TObject; PageNum,
      Progress: Integer; var AbortIt: Boolean);
  private
    FirstActivation: Boolean;
    procedure CreatePages;
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

procedure TForm1.FormCreate(Sender: TObject);
begin
  ComboBox1.ItemIndex := 6;
  ComboBox2.ItemIndex := Ord(PrintPreview.Units);
  CheckBox1.Checked := PrintPreview.FastPrint;
  CheckBox2.Checked := PrintPreview.AutoScroll;
  FirstActivation := True;
end;

procedure TForm1.FormActivate(Sender: TObject);
begin
  if FirstActivation then
  begin
    FirstActivation := False;
    Update;
    CreatePages;
  end;
end;

procedure TForm1.ComboBox1Change(Sender: TObject);
begin
  case ComboBox1.ItemIndex of
    0: PrintPreview.Zoom := 50;
    1: PrintPreview.Zoom := 100;
    2: PrintPreview.Zoom := 150;
    3: PrintPreview.Zoom := 200;
    4: PrintPreview.ZoomState := zsZoomToWidth;
    5: PrintPreview.ZoomState := zsZoomToHeight;
    6: PrintPreview.ZoomState := zsZoomToFit;
  end;
  ActiveControl := PrintPreview;
end;

procedure TForm1.ComboBox2Change(Sender: TObject);
begin
  CreatePages;
end;

procedure TForm1.Button2Click(Sender: TObject);
begin
  if PrintPreview.State = psReady then
  begin
    Screen.Cursor := crHourglass;
    try
      PrintPreview.Print;
    finally
      Screen.Cursor := crDefault;
    end;
  end;
  ActiveControl := PrintPreview;
end;

procedure TForm1.CheckBox1Click(Sender: TObject);
begin
  PrintPreview.FastPrint := CheckBox1.Checked;
end;

procedure TForm1.CheckBox2Click(Sender: TObject);
begin
  PrintPreview.AutoScroll := CheckBox2.Checked;
end;

procedure TForm1.Button3Click(Sender: TObject);
begin
  PrintPreview.CurrentPage := PrintPreview.CurrentPage - 1;
  ActiveControl := PrintPreview;
end;

procedure TForm1.Button4Click(Sender: TObject);
begin
  PrintPreview.CurrentPage := PrintPreview.CurrentPage + 1;
  ActiveControl := PrintPreview;
end;

procedure TForm1.PrintPreviewChange(Sender: TObject);
begin
  if PrintPreview.CurrentPage <> 0 then
  begin
    with PrintPreview do
      Label3.Caption := Format('Page %d of %d', [CurrentPage, TotalPages]);
    Label3.Update;
    Button3.Enabled := (PrintPreview.CurrentPage > 1);
    Button4.Enabled := (PrintPreview.CurrentPage < PrintPreview.TotalPages);
  end
  else
    Label3.Caption := 'No Page';
end;

procedure TForm1.PrintPreviewNewPage(Sender: TObject);
begin
  with PrintPreview do
  begin
    { The following line ensures one pixel pen width in any mapping mode. }
    Canvas.Pen.Width := 0;
    Canvas.Brush.Style := bsCLear;
    with PageRect do Canvas.Rectangle(Left, Top, Right, Bottom);
  end;
end;

procedure TForm1.PrintPreviewPrintProgress(Sender: TObject; PageNum,
  Progress: Integer; var AbortIt: Boolean);
begin
  if Progress = 100 then
    Caption := EmptyStr
  else
    Caption := Format('%%%d - Printing page %d of %d...',
      [Progress, PageNum, PrintPreview.TotalPages]);
end;

{ In this example, the code is independent of Units property of PrintPreview.
  If you use only one measuremnt unit for PrintPreview, you can
  easily use constant values instead of passing them to ConvertUnit method.
  Something else, precision of mmLoEnglish unit is not enough for choosing small
  font size (the smallest font can be 1/10") and then in this example I have used
  different code for mmLoEnglish to increase font size. }
procedure TForm1.CreatePages;
var
  R: TRect;
begin
  Caption := 'Creating pages...';
  Randomize;
  with PrintPreview do
  begin
    Units := TUnits(ComboBox2.ItemIndex);
    BeginDoc;
    { ---------- Creates first page ---------- }
    { To draw the image first we should calculate the image rectangle. For this
      purpose we must convert image dimensions from screen's pixels/resolution
      to printer's unit/resolution. Here we also shrink the image dimension
      two times. }
    SetRect(R, 0, 0, Screen2PrinterUnit(Image1.Width) div 2,
                     Screen2PrinterUnit(Image1.Height) div 2);
    { We place the rectangle horizontally in center of page and vertically 4mm
      below of the page's top margin. }
    OffsetRect(R, PageRect.Left + (PageWidth - R.Right) div 2,
                  PageRect.Top + ConvertUnit(1000, mmHiMetric, Units));
    { Finally, we draw the image here. }
    Canvas.StretchDraw(R, Image1.Picture.Graphic);
    { We want to draw a frame and write sample text inside it. For horizontal
      sides 1cm margin, for top 1cm below of the image, and for bottom 1cm
      margin. }
    SetRect(R, PageRect.Left + ConvertUnit(1000, mmHiMetric, Units),
               R.Bottom + ConvertUnit(1000, mmHiMetric, Units),
               PageRect.Right - ConvertUnit(1000, mmHiMetric, Units),
               PageRect.Bottom - ConvertUnit(1000, mmHiMetric, Units));
    { Here, we draw the frame. }
    Canvas.Rectangle(R.Left, R.Top, R.Right, R.Bottom);
    { To write frame dimension below the frame we use font with 5mm height. When
      used unit is mmLoEnglish we set the font height 0.2". }
    if Units <> mmLoEnglish then
      Canvas.Font.Height := -ConvertUnit(500, mmHiMetric, Units)
    else
      Canvas.Font.Height := -2;
    Canvas.TextOut(R.Left, R.Bottom, Format('%d (%s) x %d (%s)',
      [R.Right - R.Left, ComboBox2.Items[ComboBox2.ItemIndex],
       R.Bottom - R.Top, ComboBox2.Items[ComboBox2.ItemIndex]]));
    { To begin drawing of the sample texts, we set font height to 0.8mm. Of
      course when used unit is mmLoEnglish, the smallest font height ca be 0.2". }
    if Units <> mmLoEnglish then
      Canvas.Font.Height := -ConvertUnit(80, mmHiMetric, Units)
    else
      Canvas.Font.Height := -2;
    { While we do not reach to bottom of the frame... }
    while R.Top - Canvas.Font.Height < R.Bottom do
    begin
      { Selects the font color randomly. }
      Canvas.Font.Color := RGB(Random(256), Random(256), Random(256));
      { Draws the text. }
      Canvas.TextRect(R, R.Left, R.Top, 'Powered by Borland Delphi.');
      { Increases frame's top for next line. }
      Inc(R.Top, -Canvas.Font.Height);
      { We increase the font height by 0.5mm. if used unit is mmLoEnglish, we
        increase the font height by 0.1". }
      if Units <> mmLoEnglish then
        Canvas.Font.Height := Canvas.Font.Height - ConvertUnit(50, mmHiMetric, Units)
      else
        Canvas.Font.Height := Canvas.Font.Height - 1;
    end;
    { ---------- Creates second page ---------- }
    NewPage;
    { To draw the image first we should calculate the image rectangle. For this
      purpose we must convert image dimensions from screen's pixels/resolution
      to printer's unit/resolution. We also stretch the image two times. }
    SetRect(R, 0, 0, Screen2PrinterUnit(Image2.Width) * 2,
                     Screen2PrinterUnit(Image2.Height) * 2);
    { We place the rectangle in center of the page }
    OffsetRect(R, PageRect.Left + (PageWidth - R.Right) div 2,
                  PageRect.Top + (PageHeight - R.Bottom) div 2);
    { Finally, we draw the image here. }
    Canvas.StretchDraw(R, Image2.Picture.Graphic);
    EndDoc;
  end;
  { If there's any installed printer on the system, we permit printing. }
  Button2.Enabled := PrintPreview.PrinterInstalled;
  ActiveControl := PrintPreview;
  Caption := PrintPreview.ClassName;
end;

end.
