unit Main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, ExtDlgs, ToolWin, ImgList, SysImg;

type
  TMainForm = class(TForm)
    SysImageListLarge: TSysImageList;
    SysImageListSmall: TSysImageList;
    ListView: TListView;
    ToolBar: TToolBar;
    SaveDialog: TSaveDialog;
    btnLargeIcons: TToolButton;
    btnSmallIcons: TToolButton;
    btnSaveIcon: TToolButton;
    procedure FormCreate(Sender: TObject);
    procedure ListViewData(Sender: TObject; Item: TListItem);
    procedure ListViewDataHint(Sender: TObject; StartIndex, EndIndex: Integer);
    procedure ListViewSelectItem(Sender: TObject; Item: TListItem; Selected: Boolean);
    procedure btnLargeIconsClick(Sender: TObject);
    procedure btnSmallIconsClick(Sender: TObject);
    procedure btnSaveIconClick(Sender: TObject);
  private
    procedure UpdateButtonStates;
  end;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}

procedure TMainForm.UpdateButtonStates;
begin
  btnLargeIcons.Enabled := (ListView.ViewStyle <> vsIcon);
  btnSmallIcons.Enabled := (ListView.ViewStyle <> vsSmallIcon);
  btnSaveIcon.Enabled := (ListView.SelCount > 0);
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  ListView.Items.Count := SysImageListLarge.Count;
  UpdateButtonStates;
end;

procedure TMainForm.ListViewData(Sender: TObject; Item: TListItem);
begin
  with Item do
  begin
    Caption := IntToStr(Index);
    ImageIndex := Index;
  end;
end;

procedure TMainForm.ListViewDataHint(Sender: TObject; StartIndex, EndIndex: Integer);
var
  Index: Integer;
begin
  for Index := StartIndex to EndIndex do
    with ListView.Items[Index] do
    begin
      Caption := IntToStr(Index);
      ImageIndex := Index;
    end;
end;

procedure TMainForm.ListViewSelectItem(Sender: TObject; Item: TListItem; Selected: Boolean);
begin
  UpdateButtonStates;
end;

procedure TMainForm.btnLargeIconsClick(Sender: TObject);
begin
  ListView.ViewStyle := vsIcon;
  UpdateButtonStates;
end;

procedure TMainForm.btnSmallIconsClick(Sender: TObject);
begin
  ListView.ViewStyle := vsSmallIcon;
  UpdateButtonStates;
end;

procedure TMainForm.btnSaveIconClick(Sender: TObject);
var
  ImageList: TCustomImageList;
  Bitmap: TBitmap;
  X, Index: Integer;
begin
  if SaveDialog.Execute then
  begin
    if ListView.ViewStyle = vsSmallIcon then
      ImageList := SysImageListSmall
    else if ListView.ViewStyle = vsIcon then
      ImageList := SysImageListLarge
    else
      Exit;
    Bitmap := TBitmap.Create;
    try
      Bitmap.Height := ImageList.Height;
      Bitmap.Width := ImageList.Width * ListView.SelCount;
      Bitmap.PixelFormat := pf24bit;
      Bitmap.Canvas.Brush.Color := clFuchsia;
      Bitmap.Canvas.FillRect(Bitmap.Canvas.ClipRect);
      X := 0;
      for Index := ListView.Selected.Index to ListView.Items.Count - 1 do
        if ListView.Items[Index].Selected then
        begin
          ImageList.Draw(Bitmap.Canvas, X, 0, Index);
          Inc(X, ImageList.Width);
          if X >= Bitmap.Width then Break;
        end;
      Bitmap.SaveToFile(SaveDialog.FileName);
    finally
      Bitmap.Free;
    end;
  end;
end;

end.
