{ $Id: DerivedDataType.pas,v 1.3 2001/08/03 10:55:19 laa Exp $}

{
    This file is part of the TTranslator 
    Copyright (C) 2001 Polycon Ab

    TTranslator is free software; you can redistribute it and/or modify
    it under the terms of the version 2 of the GNU General Public License
    as published by the Free Software Foundation. For more information
    about licensing, please refer to
    http://www.polycon.fi/translator/licensing.html

    TTranslator is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Foobar; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
}

{----------------------------------------------------------------------------
  DerivedDataType    ???

  What:

  Company:           Polycon Ab
  Authors:           ???
----------------------------------------------------------------------------}

unit DerivedDataType;

interface
{$i common.inc}

uses
{$ifndef LINUX}
  Graphics,
{$else}
  QGraphics,
{$endif LINUX}
  Classes, SysUtils,
  DataType;

type
  TIntegerStringType = class(TStringType)
  protected
    constructor Create;

    function GetAlignment : TAlignment; override;
  end;

  TPictureType = class(TObjectType)
  protected
    function GetAsPicture(AValue : TValue) : TPicture; virtual;
  public
    function Value(Val : TPicture) : TValue;

    function FormattedValue(AValue : TValue; DataFormatter : TDataFormatter) : TValue; override;
    function ParseText(AString : String; var ParsedValue : TValue; DataFormatter : TDataFormatter) : Boolean; override;
  end;

function AsPicture(Value : TValue) : TPicture;
function ValueFromPicture(Val : TPicture) : TValue;

var
  IntegerStringType : TIntegerStringType;
  PictureType : TPictureType;

implementation

{ TIntegerStringType }

constructor TIntegerStringType.Create;
begin
  Inherited Create(Length(IntToStr(MaxInt)), False);
end;

function TIntegerStringType.GetAlignment: TAlignment;
begin
  Result := taRightJustify;
end;

{ TPictureType }

function TPictureType.GetAsPicture(AValue : TValue) : TPicture;
var
  AObject : TObject;
begin
  AObject := AsObject(AValue);
  if (AObject <> nil) and
     not (AObject is TPicture) then
    raise Exception.Create(Self.ClassName + '.GetAsPicture: You can not get Pictures from this DataType!')
  else
    Result := TPicture(AObject);
end;

function TPictureType.FormattedValue(AValue : TValue; DataFormatter : TDataFormatter) : TValue;
begin
  Result := Inherited FormattedValue(AValue, DataFormatter);
end;

function TPictureType.ParseText(AString : String; var ParsedValue : TValue; DataFormatter : TDataFormatter) : Boolean;
begin
  Result := Inherited ParseText(AString, ParsedValue, DataFormatter);
end;

function TPictureType.Value(Val : TPicture) : TValue;
begin
  Result := Inherited Value(Val);
end;

function AsPicture(Value : TValue) : TPicture;
begin
  Result := PictureType.GetAsPicture(Value);
end;

function ValueFromPicture(Val : TPicture) : TValue;
begin
  Result := PictureType.Value(Val);
end;

initialization
  IntegerStringType := TIntegerStringType.Create;
  PictureType := TPictureType.Create;

finalization
  IntegerStringType.Free;
  PictureType.Free;

end.
