unit ESColorMemo;

(*
 * ColorMemo Component
 * (c)2001 Vladimir Tsibulsky
 *)

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls;

resourcestring
   SDuplicateName = '  ''%s''';
   SItemNotFound = '  ''%s''';

type
{Forward declaration}
  TColorItems = class;
  TNamedItem = class;
  TColorItem = class;
  TColorItemCollection = class;

{ EColorItemsError }
  EColorItemsError = class(Exception);

{ TNamedItem }
  TNamedItem = class(TCollectionItem)
  private
    FName: string;
  protected
    function GetDisplayName: string; override;
    procedure SetDisplayName(const Value: string); override;
  published
    property Name: string read FName write SetDisplayName;
  end;

{ TColorItem }
  TColorItem = class(TNamedItem)
  private
   FColor : TColor;
   FText  : string;
  protected
    function GetDisplayName: string; override;
  public
    constructor Create(Owner: TColorItems; const Name: string); reintroduce; overload;
    procedure Assign(ASource: TPersistent); override;
  published
    property Color: TColor read FColor write FColor;
    property Text: string  read FText  write FText;
  end;


{ TColorItemCollection }
TColorItemCollection = class(TOwnedCollection)
  private
  protected
    procedure SetItemName(AItem: TCollectionItem); override;
  public
    constructor Create(AOwner: TPersistent;
      AClass: TCollectionItemClass);
    function Find(const AName: string): TNamedItem;
    procedure GetItemNames(List: TStrings);
    function IndexOf(const AName: string): Integer;
  end;


{ TColorItems}
 TColorItems = class(TColorItemCollection){TDefCollection}
  private
    FOwner : TMemo;
    function GetColorItem(Index: Integer): TColorItem;
    procedure SetColorItem(Index: Integer; Value: TColorItem);
  public
    constructor Create(AListBox: TMemo);
    function AddColorItem: TColorItem;
    function Find(const Name: string): TColorItem;
    procedure Add(const Name: string);
    property Items[Index: Integer]: TColorItem read GetColorItem write SetColorItem; default;
  end;

  TESColorMemo = class(TMemo)
  private
    FColorItems :  TColorItems;
    procedure WMPaint(var Message: TWMPaint); message WM_PAINT;
    procedure SetColorItems (AValue: TColorItems);
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property ColorItems: TColorItems read FColorItems write SetColorItems;
  end;

{ Global function}
procedure ColorItemsError(const Message: string; Component: TComponent = nil);
procedure ColorItemsErrorFmt(const Message: string; const Args: array of const;
  Component: TComponent = nil);


procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('ES Controls', [TESColorMemo]);
end;

constructor TESColorMemo.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FColorItems:=TColorItems.Create(Self);
end;

destructor TESColorMemo.Destroy;
begin
  FColorItems.Free;
  inherited Destroy;
end;

procedure TESColorMemo.WMPaint(var Message: TWMPaint);
  function Exists(St  :String; var lex :String; var Color :TColor):Boolean;
  var
    i,j,z  :Integer;
  begin
    Result:=False;
    z:=0;
    for i:=0 to FColorItems.Count-1 do begin
      j:=pos(FColorItems.Items[i].FText, St);
      if (j<>0)and((z=0)or(j<z))
      then begin
        z:=j;
        lex:=FColorItems.Items[i].FText;
        Color:=FColorItems.Items[i].FColor;
        Result:=True;
      end;
    end;
  end;
var
  PS           :TPaintStruct;
  DC           :HDC;
  Canvas       :TCanvas;
  i            :Integer;
  X,Y          :DWord;
  ALex         :String;
  AColor       ,
  OldColor     :TColor;
  St           ,
  St1          :String;
  Size         :TSize;
begin
  DC := Message.DC;
  if DC = 0
  then DC := BeginPaint(Handle, PS);
  Canvas:=TCanvas.Create;
  try
    OldColor := Font.Color;
    Canvas.Handle:=DC;
    Canvas.Font.Name:=Font.Name;
    with Canvas do begin
      Y:=2;//    
      for i:=0 to Pred(Lines.Count) do begin//  Memo
        X := 1;
        St:=Lines[i];
        while Exists(St, ALex, AColor) do begin
          Font.Color:=OldColor;
          St1:=copy(St, 1, pos(ALex, St)-1);//   
          Delete(St, 1, pos(ALex, St)-1);

          TextOut(X,Y, St1);
          GetTextExtentPoint32(DC, PChar(St1), Length(St1), Size);
          inc(X, Size.cx);//    

          Font.Color:=AColor;
          St1:=copy(St, 1, Length(ALex));// 
          Delete(St, 1, Length(ALex));

          TextOut(X,Y, St1);
          GetTextExtentPoint32(DC, PChar(St1), Length(St1), Size);
          inc(X, Size.cx);
        end;
        Font.Color:=OldColor;
        TextOut(X,Y, St);//      
        St:='size';//    
        GetTextExtentPoint32(DC, PChar(St), Length(St), Size);
        Inc(Y, Size.cy);//   
      end;
    end;
  finally
    if Message.DC = 0 then EndPaint(Handle, PS);
  end;
  Canvas.Free;
  inherited;
end;

procedure TESColorMemo.SetColorItems(AValue: TColorItems);
begin
  ColorItems.Assign(AValue);
end;

{ TColorItems }

constructor TColorItems.Create(AListBox: TMemo);
begin
  inherited Create(AListBox, TColorItem);
  FOwner := AListBox;
end;

function TColorItems.AddColorItem: TColorItem;
begin
  Result := TColorItem(inherited Add);
end;

procedure TColorItems.Add(const Name: string);
var
  ColorItem: TColorItem;
begin
  if IndexOf(Name) >= 0
  then ColorItemsErrorFmt(SDuplicateName, [Name], FOwner);

  ColorItem := AddColorItem;
  ColorItem.Name := Name;
end;


function TColorItems.Find(const Name: string): TColorItem;
begin
  Result := TColorItem(inherited Find(Name));
  if Result = nil
  then ColorItemsErrorFmt(SItemNotFound, [Name], FOwner);
end;

function TColorItems.GetColorItem(Index: Integer): TColorItem;
begin
  Result := TColorItem(inherited Items[Index]);
end;

procedure TColorItems.SetColorItem(Index: Integer; Value: TColorItem);
begin
  inherited Items[Index] := Value;
end;

{ TColorItemCollection }

constructor TColorItemCollection.Create( AOwner: TPersistent;
  AClass: TCollectionItemClass);
begin
  inherited Create(AOwner, AClass);
end;

procedure TColorItemCollection.SetItemName(AItem: TCollectionItem);
begin
  with TNamedItem(AItem) do
    if (Name = '') then
      Name := Copy(ClassName, 2, 9) + IntToStr(ID+1);
end;


function TColorItemCollection.IndexOf(const AName: string): Integer;
begin
  for Result := 0 to Count - 1 do
    if AnsiCompareText(TNamedItem(Items[Result]).Name, AName) = 0 then Exit;
  Result := -1;
end;

function TColorItemCollection.Find(const AName: string): TNamedItem;
var
  I: Integer;
begin
  I := IndexOf(AName);
  if I < 0 then Result := nil else Result := TNamedItem(Items[I]);
end;

procedure TColorItemCollection.GetItemNames(List: TStrings);
var
  I: Integer;
begin
  List.BeginUpdate;
  try
    List.Clear;
    for I := 0 to Count - 1 do
      with TNamedItem(Items[I]) do
        if Name <> '' then List.Add(Name);
  finally
    List.EndUpdate;
  end;
end;


{TColorItem}

constructor TColorItem.Create(Owner: TColorItems; const Name: string);
begin
  inherited Create(Owner);
  FName := Name;
end;

procedure TColorItem.Assign(ASource: TPersistent);
var
  S: TColorItem;
begin
  if ASource is TColorItem then
  begin
    if Collection <> nil then Collection.BeginUpdate;
    try
      S := TColorItem(ASource);
      Name  := S.Name;
      Text  := S.Text;
      Color := S.Color;
    finally
      if Collection <> nil then Collection.EndUpdate;
    end;
  end
  else inherited;
end;

function TColorItem.GetDisplayName: string;
begin
  Result := inherited GetDisplayName;
end;


{ TNamedItem }

function TNamedItem.GetDisplayName: string;
begin
  Result := FName;
end;

procedure TNamedItem.SetDisplayName(const Value: string);
begin
  if (Value <> '') and (AnsiCompareText(Value, FName) <> 0) and
    (Collection is TColorItemCollection) and
    (TColorItemCollection(Collection).IndexOf(Value) >= 0)
  then ColorItemsErrorFmt(SDuplicateName, [Value, Collection.ClassName]);
  FName := Value;
  inherited;
end;


procedure ColorItemsError(const Message: string; Component: TComponent = nil);
begin
  if Assigned(Component) and (Component.Name <> '') then
    raise EColorItemsError.Create(Format('%s: %s', [Component.Name, Message])) else
    raise EColorItemsError.Create(Message);
end;

procedure ColorItemsErrorFmt(const Message: string; const Args: array of const;
  Component: TComponent = nil);
begin
  ColorItemsError(Format(Message, Args), Component);
end;

end.
