{$B-,H+,J+,Q-,T-,X+}

{$UNDEF D3PLUS}
{$UNDEF D4PLUS}
{$IFDEF VER100}{$DEFINE D3PLUS}{$ENDIF}
{$IFDEF VER120}{$DEFINE D3PLUS}{$DEFINE D4PLUS}{$ENDIF}
{$IFDEF VER130}{$DEFINE D3PLUS}{$DEFINE D4PLUS}{$ENDIF}
{$IFDEF VER140}{$DEFINE D3PLUS}{$DEFINE D4PLUS}{$ENDIF}

unit GpTextFile;

(*:Interface to 8/16-bit text files and streams. Uses GpHugeF unit for file
   access.
   @author Primoz Gabrijelcic
   @desc <pre>
   (c) 2000 Primoz Gabrijelcic
   Free for personal and commercial use. No rights reserved.

   Author           : Primoz Gabrijelcic
   Creation date    : 1999-11-01
   Last modification: 2001-12-15
   Version          : 3.01a
   Requires         : GpHugeF 3.01, GpTextStream 1.0
   </pre>
*)(*
   History:
     3.01a: 2001-12-15
       - Updated to compile with Delphi 6.

     3.01: 2001-07-17
       - TGpTextStream class moved into a separate unit (GpTextStream).
       
     3.0b: 2001-06-22
       - WriteString was not always returning a value. Fixed.

     3.0a: 2001-06-18
       - Bug fixed: function WriteString crashed if its argument was empty
         string.

     3.0: 2001-05-15
       - TGpTextFileStream class split into two classes. Basic functionality
         (Unicode encoding/decoding) was moved into TGpTextStream that can work
         on any TStream descendant. TGpTextFileStream now only opens the file
         and forwards it to the TGpTextStream.

     2.06: 2001-03-22
       - Added overloaded TGpTextFile.Write(s: string), available on D4 and
         better.

     2.05: 2001-02-27
       - Added property AcceptedDelimiters to TGpTextFile and TGpTextFileStream.
         You can use it do specify what CR-LF combinations should be treated as
         a line delimiter. 
         If not set (or set to []), classes will behave as before - TGpTextFile
         will use CR, LF, CRLF, /2028/, and /000D/000A/ for line delimiters and
         TGpTextFileStream will use LF, CRLF, /2028/, and /000D/000A/ for line
         delimiters.

     2.04: 2001-01-31
       - All raised exceptions now have HelpContext set. All possible
         HelpContext values are enumerated in 'const' section at the very
         beginning of the unit. Thanks to Peter Evans for the suggestion.
       - TGpTextFileStream.Win32Check made protected (was public by mistake).

     2.03a: 2000-12-11
       - Fixed Append and AppendSafe to create a file if it does not exist.

     2.03: 2000-10-25
       - Removed one FreeAndNil call preventing unit to compile under Delphi 2,
         3, and 4.

     2.02: 2000-10-21
       - Added support for Unicode pseudo-codepage 1200 (CP_UNICODE).

     2.01: 2000-10-12
       - Added UTF-8 support (CP_UTF8 codepage) to TGpTextFile and
         TGpTextFileStream. To work with Unicode file in UTF-8 encoding, set
         cfUnicode flag and CP_UTF8 code page at the same time.
         When working in UTF-8 mode, TGpTextFile doesn't support lines longer
         than 2.147.483.647 bytes.
         - When opening files in unknown format, it is probably best to first
           Reset it with flags=[cfUnicode] and codepage=CP_UTF8. Then you can
           check if file is 16-bit Unicode (Is16bit). If it is not, you can
           change the Codepage to some other value (1252, for example) or just
           leave it at CP_UTF8 (reasonable default for processing XML files).
       - UTF-8 support will read and skip Byte Order Mark (EF BB BF) but will
         only write BOM if cfWriteUTF8BOM is specified in flags.
       - New functions TGpTextFile.Is16bit and TGpTextFileStream.Is16bit return
         true when file is Unicode with 16-bit (UCS-2) encoding. IsUnicode
         returns true when file is Unicode with 16-bit (UCS-2) or 8-bit (UTF-8)
         encoding.
       - Added symbolic constants for ISO code pages not defined in Windows.pas:
         ISO-8859-1, ISO-8859-2, ISO-8859-3, ISO-8859-4, ISO-8859-5, ISO-8859-6,
         ISO-8859-7, ISO-8859-8 (symbols are named ISO_8859_n).

     2.0: 2000-10-06
       - Created TGpTextFileStream class, descendant of TStream, which offers
         similar functionality as TGpTextFile - automatic detection of Unicode
         files, automatic code page remapping etc.
       - TGpTextFile.SetCodepage made public so it is possible to change code
         page on the fly.
       - Added parameters 'flags', 'waitObject', and 'codePage' to
         TGpTextFile.AppendSafe.
       - Order of 'flags' and 'bufferSize' parameters reversed in most methods.
       - Position of 'diskLockTimeout' and 'diskRetryDelay' parameters changed
         in all methods. 
       - Fully documented.
       - All language-dependant string constants moved to resourcestring section.

     1.08: 2000-10-05
       - Added codePage parameter to all Reset and Rewrite functions. Default
         value of 0 specifies conversion according to current codepage and any
         other number specifies codepage that should be used for conversion.

     1.07: 2000-08-01
       - All exceptions generated in this unit were converted to EGpTextFile
         exceptions (descendant of EGpHugeFile).
       - All Windows-generated exceptions not caught in TGpHugeFile are now
         converted to EGpTextFile exception.
       - Added parameter bufferSize to Append and AppendSafe.
       - Append* opened file in non-buffered mode. Fixed.

     1.06: 2000-06-22
       - Added overloaded version of Writeln.

     1.05: 2000-05-15
       - Rewrite now opens file in buffered mode.

     1.04: 2000-04-19
       - Added new ResetSafe/RewriteSafe parameter - waitObject. It is forwarded to
         TGpHugeFile.ResetEx/RewriteEx.

     1.03: 2000-03-03
       - Added OpenFlags parameter to Reset and ResetSafe. In D4 or higher its
         value defaults to [] so no source code changes to old applications will
         be required. Currently only supported OpenFlag is opCloseOnEOF, which
         enables hfoCloseOnEOF flag in TGpHugeFile (see GpHugeF.pas for more
         details).

     1.02: 1999-11-24
       - Added bufferSize parameter to Reset* and Rewrite*. By default
         (bufferSize = 0), 64 KB buffer is allocated.

     1.01a: 1999-11-03
       - Append fixed.

     1.01: 1999-11-02
       - Added ResetSafe and RewriteSafe methods.

     1.0: 1999-11-01
       - First published version.
*)

interface

uses
  Windows,
  Classes,
  GpHugeF,
  GpTextStream;

// HelpContext values for all raised exceptions. All EGpHugeFile exception are
// re-raised without modifying HelpContext (which was already assigned in
// GpHugeF unit).
const
  //:Exception was handled and converted to EGpTextFile but was not expected and is not categorised.
  hcTFUnexpected              = 2000;
  //:Failed to append file.
  hcTFFailedToAppend          = 2003;
  //:Failed to reset file.
  hcTFFailedToReset           = 2004;
  //:Failed to rewrite file.
  hcTFFailedToRewrite         = 2005;
  //:Cannot append reversed Unicode file - not supported.
  hcTFCannotAppendReversed    = 2006;
  //:Cannot write to reversed Unicode file - not supported.
  hcTFCannotWriteReversed     = 2007;
  //:Parameter to Write method is invalid.
  hcTFInvalidParameter        = 2008;

type
  {:Base exception class for exceptions created in TGpTextFile and descendants.
  }
  EGpTextFile       = class(EGpHugeFile);

  {:Base exception class for exceptions created in TGpTextFileStream.
  }
  EGpTextFileStream = class(EGpHugeFileStream);

  {:Text file creation flags.
    @enum cfUnicode          Create Unicode file.
    @enum cfReverseByteOrder Create unicode file with reversed byte order
                             (Motorola format). Set only on Reset, readonly.
                             Currently ignored in Rewrite.
    @enum cfUse2028          Use standard /2028/ instead of /000D/000A/ for line
                             delimiter (MS Notepad and MS Word do not understand
                             $2028 delimiter). Applies to Unicode files only.
    @enum cfUseLF            Use /LF/ instead of /CR/LF/ for line delimiter.
                             Applies to 8-bit files only.
    @enum cfWriteUTF8BOM     Write UTF-8 Byte Order Mark to the beginning of
                             file.
  }
  TCreateFlag = (cfUnicode, cfReverseByteOrder, cfUse2028, cfUseLF,
    cfWriteUTF8BOM);

  {:Set of all creation flags.
  }
  TCreateFlags = set of TCreateFlag;

  {:Text file open (reset) flags.
    @enum ofCloseOnEOF Remaps to TGpHugeFile hfCloseOnEOF.
  }
  TOpenFlag = (ofCloseOnEOF);

  {:Set of all open flags.
  }
  TOpenFlags = set of TOpenFlag;

  {:Line delimiters.
    @enum ldCR       Carriage return (Mac style).
    @enum ldLF       Line feed (Unix style).
    @enum ldCRLF     Carriage return + Line feed (DOS style).
    @enum ldLFCR     Line feed + Carriage return (very unusual combination).
    @enum ld2028     /2028/ Unicode delimiter.
    @enum ld000D000A /000D/000A/ Windows-style Unicode delimiter.
  }
  TLineDelimiter = (ldCR, ldLF, ldCRLF, ldLFCR, ld2028, ld000D000A);

  {:Set of all line delimiters.
  }
  TLineDelimiters = set of TLineDelimiter;

  {:Unified 8/16-bit text file access. All strings passed as Unicode, conversion
    to/from 8-bit is done automatically according to specified code page.
    Access is buffered but direct-access functions (FilePos, Seek) are supported
    nevertheless.
  }
  TGpTextFile = class(TGpHugeFile)
  private
    tfCFlags    : TCreateFlags;
    tfCodePage  : word;
    tfLineDelims: TLineDelimiters;
    tfSmallBuf  : pointer;
  protected
    function  AllocTmpBuffer(size: integer): pointer; virtual;
    procedure FreeTmpBuffer(var buffer: pointer); virtual;
    procedure PrepareBuffer; virtual;
    procedure SetCodepage(cp: word); virtual;
    procedure WriteString(ws: WideString); virtual;
  public
    destructor Destroy; override;
    procedure Append(
      flags: TCreateFlags {$IFDEF D4plus}= []{$ENDIF};
      bufferSize: integer {$IFDEF D4plus}= 0{$ENDIF};
      codePage: word      {$IFDEF D4plus}= 0{$ENDIF});
    function  AppendSafe(
      flags: TCreateFlags      {$IFDEF D4plus}= []{$ENDIF};
      bufferSize: integer      {$IFDEF D4plus}= 0{$ENDIF};
      diskLockTimeout: integer {$IFDEF D4plus}= 0{$ENDIF};
      diskRetryDelay: integer  {$IFDEF D4plus}= 0{$ENDIF};
      waitObject: THandle      {$IFDEF D4plus}= 0{$ENDIF};
      codePage: word           {$IFDEF D4plus}= 0{$ENDIF}): THFError;
    function  EOF: boolean;
    function  Is16bit: boolean;
    function  IsUnicode: boolean;
    function  Readln: WideString;
    procedure Reset(
      flags: TOpenFlags   {$IFDEF D4plus}= []{$ENDIF};
      bufferSize: integer {$IFDEF D4plus}= 0{$ENDIF};
      codePage: word      {$IFDEF D4plus}= 0{$ENDIF});
    function  ResetSafe(
      flags: TOpenFlags        {$IFDEF D4plus}= []{$ENDIF};
      bufferSize: integer      {$IFDEF D4plus}= 0{$ENDIF};
      diskLockTimeout: integer {$IFDEF D4plus}= 0{$ENDIF};
      diskRetryDelay: integer  {$IFDEF D4plus}= 0{$ENDIF};
      waitObject: THandle      {$IFDEF D4plus}= 0{$ENDIF};
      codePage: word           {$IFDEF D4plus}= 0{$ENDIF}): THFError;
    procedure Rewrite(
      flags: TCreateFlags {$IFDEF D4plus}= []{$ENDIF};
      bufferSize: integer {$IFDEF D4plus}= 0{$ENDIF};
      codePage: word      {$IFDEF D4plus}= 0{$ENDIF});
    function  RewriteSafe(
      flags: TCreateFlags      {$IFDEF D4plus}= []{$ENDIF};
      bufferSize: integer      {$IFDEF D4plus}= 0{$ENDIF};
      diskLockTimeout: integer {$IFDEF D4plus}= 0{$ENDIF};
      diskRetryDelay: integer  {$IFDEF D4plus}= 0{$ENDIF};
      waitObject: THandle      {$IFDEF D4plus}= 0{$ENDIF};
      codePage: word           {$IFDEF D4plus}= 0{$ENDIF}): THFError;
    procedure Write(params: array of const); {$IFDEF D4plus} overload;
    procedure Write(s: string); overload; {$ENDIF D4plus}
    procedure Writeln(ln: WideString {$IFDEF D4plus}= ''{$ENDIF}); {$IFDEF D4plus} overload;
    procedure Writeln(params: array of const); overload; {$ENDIF D4plus}
    {:Accepted line delimiters (CR, LF or any combination).
    }
    property AcceptedDelimiters: TLineDelimiters read tfLineDelims
      write tfLineDelims;
    {:Code page used to convert 8-bit files to Unicode and back. May be changed
      while file is open (and even partially read). If set to 0, current default
      code page will be used.
    }
    property Codepage: word read tfCodepage write SetCodepage;
  end; { TGpTextFile }

  {:Wrapper for TGpTextStream that automatically creates TGpHugeFileStream for
    a specified file in the constructor and destroys it in the destructor.
  }
  TGpTextFileStream = class(TGpTextStream)
  private
    tfsStream: TGpHugeFileStream;
  protected
    function  GetFileName: string; virtual;
    function  GetWindowsError: DWORD; override;
    function  StreamName(param: string = ''): string; override;
  public
    constructor Create(
      const fileName: string; access: TGpHugeFileStreamAccess;
      openFlags: TOpenFlags     {$IFDEF D4plus}= []{$ENDIF};
      createFlags: TCreateFlags {$IFDEF D4plus}= []{$ENDIF};
      codePage: word            {$IFDEF D4plus}= 0{$ENDIF}
      );
    destructor  Destroy; override;
    {:Name of underlying file.
    }
    property  FileName: string read GetFileName;
  end; { TGpTextFileStream }

implementation

uses
  SysUtils,
  SysConst;

const
  {:Header for 'normal' Unicode file (Intel format).
  }
  CUnicodeNormal  : WideChar = WideChar($FEFF);

  {:Header for 'reversed' Unicode file (Motorola format).
  }
  CUnicodeReversed: WideChar = WideChar($FFFE);

  {:First two bytes of UTF-8 BOM.
  }
  CUTF8BOM12: WideChar = WideChar($BBEF);

  {:Third byte of UTF-8 BOM.
  }
  CUTF8BOM3: Char = Char($BF);

  {:Size of preallocated buffer used for 8 to 16 to 8 bit conversions in
    TGpTextFile.
  }
  CtsSmallBufSize = 2048; // 1024 WideChars

{$IFDEF D3plus}
resourcestring
{$ELSE}
const
{$ENDIF}
  sCannotAppendReversedUnicodeFile   = 'TGpTextFile(%s):Cannot append reversed Unicode file.';
  sCannotAppendReversedUnicodeStream = '%s:Cannot append reversed Unicode file.';
  sCannotConvertOddNumberOfBytes     = '%s:Cannot convert odd number of bytes: %d';
  sCannotWriteReversedUnicodeFile    = 'TGpTextFile(%s):Cannot write to reversed Unicode file.';
  sCannotWriteReversedUnicodeStream  = '%s:Cannot write to reversed Unicode file.';
  sFailedToAppendFile                = 'TGpTextFile(%s):Failed to append.';
  sFailedToResetFile                 = 'TGpTextFile(%s):Failed to reset file.';
  sFailedToRewriteFile               = 'TGpTextFile(%s):Failed to rewrite file.';
  sInvalidParameter                  = 'TGpTextFile(%s):Invalid parameter!';
  sStreamFailed                      = '%s failed. ';

{:Converts Ansi string to Unicode string using specified code page.
  @param   s        Ansi string.
  @param   codePage Code page to be used in conversion.
  @returns Converted wide string.
}
function StringToWideString(const s: AnsiString; codePage: word): WideString;
var
  l: integer;
begin
  if s = '' then
    Result := ''
  else begin
    l := MultiByteToWideChar(codePage, MB_PRECOMPOSED, PChar(@s[1]), -1, nil, 0);
    SetLength(Result, l-1);
    if l > 1 then
      MultiByteToWideChar(CodePage, MB_PRECOMPOSED, PChar(@s[1]), -1, PWideChar(@Result[1]), l-1);
  end;
end; { StringToWideString }

{:Converts Unicode string to Ansi string using specified code page.
  @param   ws       Unicode string.
  @param   codePage Code page to be used in conversion.
  @returns Converted ansi string.
}
function WideStringToString (const ws: WideString; codePage: Word): AnsiString;
var
  l: integer;
begin
  if ws = '' then
    Result := ''
  else begin
    l := WideCharToMultiByte(codePage,
           WC_COMPOSITECHECK or WC_DISCARDNS or WC_SEPCHARS or WC_DEFAULTCHAR,
           @ws[1], -1, nil, 0, nil, nil);
    SetLength(Result, l-1);
    if l > 1 then
      WideCharToMultiByte(codePage,
        WC_COMPOSITECHECK or WC_DISCARDNS or WC_SEPCHARS or WC_DEFAULTCHAR,
        @ws[1], -1, @Result[1], l-1, nil, nil);
  end;
end; { WideStringToString }

{:Convers buffer of WideChars into UTF-8 encoded form. Target buffer must be
  pre-allocated and large enough (each WideChar will use at most three bytes
  in UTF-8 encoding).                                                            <br>
  RFC 2279 (http://www.ietf.org/rfc/rfc2279.txt) describes the conversion:       <br>
  $0000..$007F => $00..$7F                                                       <br>
  $0080..$07FF => 110[bit10..bit6] 10[bit5..bit0]                                <br>
  $0800..$FFFF => 1110[bit15..bit12] 10[bit11..bit6] 10[bit5..bit0]
  @param   unicodeBuf   Buffer of WideChars.
  @param   uniByteCount Size of unicodeBuf, in bytes.
  @param   utf8Buf      Pre-allocated buffer for UTF-8 encoded result.
  @returns Number of bytes used in utf8Buf buffer.
  @since   2.01
}
function WideCharBufToUTF8Buf(const unicodeBuf; uniByteCount: integer;
  var utf8Buf): integer;
var
  iwc: integer;
  pch: PChar;
  pwc: PWideChar;
  wc : word;

  procedure AddByte(b: byte);
  begin
    pch^ := char(b);
    Inc(pch);
  end; { AddByte }

begin { WideCharBufToUTF8Buf }
  pwc := @unicodeBuf;
  pch := @utf8Buf;
  for iwc := 1 to uniByteCount div SizeOf(WideChar) do begin
    wc := Ord(pwc^);
    Inc(pwc);
    if (wc >= $0001) and (wc <= $007F) then begin
      AddByte(wc AND $7F);
    end
    else if (wc >= $0080) and (wc <= $07FF) then begin
      AddByte($C0 OR ((wc SHR 6) AND $1F));
      AddByte($80 OR (wc AND $3F));
    end
    else begin // (wc >= $0800) and (wc <= $FFFF)
      AddByte($E0 OR ((wc SHR 12) AND $0F));
      AddByte($80 OR ((wc SHR 6) AND $3F));
      AddByte($80 OR (wc AND $3F));
    end;
  end; //for
  Result := integer(pch)-integer(@utf8Buf);
end; { WideCharBufToUTF8Buf }

{:Converts UTF-8 encoded buffer into WideChars. Target buffer must be
  pre-allocated and large enough (at most utfByteCount number of WideChars will
  be generated).                                                                 <br>
  RFC 2279 (http://www.ietf.org/rfc/rfc2279.txt) describes the conversion:       <br>
  $00..$7F => $0000..$007F                                                       <br>
  110[bit10..bit6] 10[bit5..bit0] => $0080..$07FF                                <br>
  1110[bit15..bit12] 10[bit11..bit6] 10[bit5..bit0] => $0800..$FFFF
  @param   utf8Buf      UTF-8 encoded buffer.
  @param   utfByteCount Size of utf8Buf, in bytes.
  @param   unicodeBuf   Pre-allocated buffer for WideChars.
  @param   leftUTF8     Number of bytes left in utf8Buf after conversion (0, 1,
                        or 2).
  @returns Number of bytes used in unicodeBuf buffer.
  @since   2.01
}
function UTF8BufToWideCharBuf(const utf8Buf; utfByteCount: integer;
 var unicodeBuf; var leftUTF8: integer): integer;
var
  c1 : byte;
  c2 : byte;
  ch : byte;
  pch: PChar;
  pwc: PWideChar;
begin
  pch := @utf8Buf;
  pwc := @unicodeBuf;
  leftUTF8 := utfByteCount;
  while leftUTF8 > 0 do begin
    ch := byte(pch^);
    Inc(pch);
    if (ch AND $80) = 0 then begin // 1-byte code
      word(pwc^) := ch;
      Inc(pwc);
      Dec(leftUTF8);
    end
    else if (ch AND $E0) = $C0 then begin // 2-byte code
      if leftUTF8 < 2 then
        break;
      c1 := byte(pch^);
      Inc(pch);
      word(pwc^) := (word(ch AND $1F) SHL 6) OR (c1 AND $3F);
      Inc(pwc);
      Dec(leftUTF8,2);
    end
    else begin // 3-byte code
      if leftUTF8 < 3 then
        break;
      c1 := byte(pch^);
      Inc(pch);
      c2 := byte(pch^);
      Inc(pch);
      word(pwc^) :=
        (word(ch AND $0F) SHL 12) OR
        (word(c1 AND $3F) SHL 6) OR
        (c2 AND $3F);
      Inc(pwc);
      Dec(leftUTF8,3);
    end;
  end; //while
  Result := integer(pwc)-integer(@unicodeBuf);
end; { UTF8BufToWideCharBuf }

{:Returns default Ansi codepage for LangID or 'defCP' in case of error (LangID
  does not specify valid language ID).
  @param   LangID Language ID.
  @param   defCP  Default value that is to be returned if LangID doesn't specify
                  a valid language ID.
  @returns Default Ansi codepage for LangID or 'defCP' in case of error.
}
function GetDefaultAnsiCodepage (LangID: LCID; defCP: integer): word;
var
  p: array [0..255] of char;
begin
  if GetLocaleInfo (LangID, 4100, p, High (p)) > 0 then
    Result := StrToIntDef(p,defCP)
  else
    Result := defCP;
end; { GetDefaultAnsiCodepage }

{ TGpTextFile }

{:Allocates small buffer if not already allocated.
  @since   2.01
}
procedure TGpTextFile.PrepareBuffer;
begin
  if not assigned(tfSmallBuf) then
    GetMem(tfSmallBuf,CtsSmallBufSize);
end; { TGpTextFile.PrepareBuffer }

{:Allocates buffer for 8/16/8 bit conversions. If requested size is small
  enough, returns pre-allocated buffer, otherwise allocates new buffer.
  @param   size Requested size in bytes.
  @returns Pointer to buffer.
}
function TGpTextFile.AllocTmpBuffer(size: integer): pointer;
begin
  if size <= CtsSmallBufSize then
    Result := tfSmallBuf
  else
    GetMem(Result,size);
end; { TGpTextFile.AllocTmpBuffer }

{:Frees buffer for 8/16/8 bit conversions. If pre-allocated buffer is passed,
  nothing will be done.
  @param   buffer Conversion buffer.
}
procedure TGpTextFile.FreeTmpBuffer(var buffer: pointer);
begin
  if buffer <> tfSmallBuf then begin
    FreeMem(buffer);
    buffer := nil;
  end;
end; { TGpTextFile.FreeTmpBuffer }

{:Simplest form of Append.
  @param   flags      Create flags. Only cfUse2028 and cfUseLF flags are used.
  @param   bufferSize Size of buffer. 0 means default size (BUF_SIZE, currently
                      64 KB).
  @param   codePage   Code page to be used for 8/16/8 bit conversion. If 0,
                      default code page for currently used language will be
                      used.
  @raises  EGpTextFile if file could not be appended.
}
procedure TGpTextFile.Append(flags: TCreateFlags; bufferSize: integer;
  codePage: word);
begin
  if AppendSafe(flags,bufferSize,0,0,0,codePage) <> hfOK then
    raise EGpTextFile.CreateFmtHelp(sFailedToAppendFile,[FileName],hcTFFailedToAppend);
end; { TGpTextFile.Append }

{:Full form of Append. Will retry if file is locked by another application (if
  diskLockTimeout and diskRetryDelay are specified). Allows caller to specify
  additional options. Does not raise an exception on error (except appending
  reversed Unicode file).
  @param   flags           Create flags. Only cfUse2028 and cfUseLF flags are
                           used.
  @param   bufferSize      Size of buffer. 0 means default size (BUF_SIZE,
                           currently 64 KB).
  @param   diskLockTimeout Max time (in milliseconds) AppendSafe will wait for
                           lock file to become free.
  @param   diskRetryDelay  Delay (in milliseconds) between attempts to open
                           locked file.
  @param   waitObject      Handle of 'terminate' event (semaphore, mutex). If
                           this parameter is specified (not zero) and becomes
                           signalled, AppendSafe will stop trying to open locked
                           file and will exit with.
  @param   codePage        Code page to be used for 8/16/8 bit conversion. If 0,
                           default code page for currently used language will be
                           used.
  @raises  EGpTextFile if file is 'reversed' Unicode file.
}
function TGpTextFile.AppendSafe(flags: TCreateFlags; bufferSize: integer;
  diskLockTimeout, diskRetryDelay: integer; waitObject: THandle;
  codePage: word): THFError;
var
  marker : WideChar;
  marker3: Char;
begin
  try
    if (cfUnicode in flags) and (codePage <> CP_UTF8) then
      codePage := CP_UNICODE;
    PrepareBuffer;
    SetCodepage(codePage);
    Result := ResetEx(1,bufferSize,diskLockTimeout,diskRetryDelay,
      [hfoBuffered,hfoCanCreate],waitObject);
    if Result = hfOK then begin
      tfCFlags := [];
      if FileSize >= SizeOf(WideChar) then begin
        BlockReadUnsafe(marker,SizeOf(WideChar));
        if marker = CUnicodeNormal then begin
          tfCFlags := tfCFlags + [cfUnicode];
          Codepage := CP_UNICODE;
        end
        else if marker = CUnicodeReversed then begin
          tfCFlags := tfCFlags + [cfUnicode,cfReverseByteOrder];
          Codepage := CP_UNICODE;
        end
        else if (marker = CUTF8BOM12) and (FileSize >= 3) then begin
          BlockReadUnsafe(marker3,SizeOf(Char));
          if marker3 = CUTF8BOM3 then begin
            tfCFlags := tfCFlags + [cfUnicode];
            Codepage := CP_UTF8;
          end;
        end;
      end
      else if (FileSize = 0) and IsUnicode then begin
        if Codepage <> CP_UTF8 then
          BlockWriteUnsafe(CUnicodeNormal,SizeOf(WideChar))
        else if cfWriteUTF8BOM in flags then begin
          BlockWriteUnsafe(CUTF8BOM12,SizeOf(WideChar));
          BlockWriteUnsafe(CUTF8BOM3,SizeOf(Char));
        end;
      end;
      if (not IsUnicode) and ((Codepage = CP_UTF8) or (Codepage = CP_UNICODE)) then
        tfCFlags := tfCFlags + [cfUnicode];
      if tfCFlags*[cfUnicode,cfReverseByteOrder] = [cfUnicode,cfReverseByteOrder] then
        raise EGpTextFile.CreateFmtHelp(sCannotAppendReversedUnicodeFile,[FileName],hcTFCannotAppendReversed);
      tfCFlags := tfCFlags + (flags * [cfUse2028,cfUseLF]);
      Seek(FileSize);
    end;
  except
    on EGpTextFile do
      raise;
    on Exception do
      Result := hfError;
  end;
end; { TGpTextFile.Append }

{:Cleanup.
  @since   2.01
}
destructor TGpTextFile.Destroy;
begin
  if assigned(tfSmallBuf) then begin
    FreeMem(tfSmallBuf);
    tfSmallBuf := nil;
  end;
  inherited;
end; { TGpTextFile.Destroy }

{:Checks if file pointer is at end of file.
  @returns True if file pointer is at end of file.
  @raises  EGpHugeFile on Windows errors.
}
function TGpTextFile.EOF: boolean;
begin
  Result := (FilePos >= FileSize);
end; { TGpTextFile.EOF }

{:Checks if file is 16-bit Unicode.
  @returns True if file is 16-bit Unicode.
  @since   2.01
}
function TGpTextFile.Is16bit: boolean;
begin
  Result := IsUnicode and (Codepage <> CP_UTF8);
end; { TGpTextFile.Is16bit }

{:Checks if file is Unicode (UCS-2 or UTF-8 encoding).
  @returns True if file is Unicode.
}
function TGpTextFile.IsUnicode: boolean;
begin
  Result := (cfUnicode in tfCFlags);
end; { TGpTextFile.IsUnicode }

{:Reads line from file. If file is 8-bit, LF, CR, CRLF, and LFCR are considered
  end-of-line terminators (if included in AcceptedDelimiters). If file is
  16-bit, both /000D/000A/ and /2028/ are considered end-of-line terminators (if
  included in AcceptedDelimiters). If file is 8-bit, line is converted to
  Unicode according to code page specified in Append, Reset or Rewrite.
  @returns Line without terminator characters.
  @raises  EGpHugeFile on Windows errors.
  @seeAlso Append, Reset, Rewrite
}
function TGpTextFile.Readln: WideString;
var
  buf     : array [1..1028] of byte; // size must be even for Unicode
  bufPos  : HugeInt;
  delimLen: integer;
  delimPos: integer;
  leftUtf8: integer;
  leof    : boolean;
  resLn   : WideString;
  size    : cardinal;
  uniBytes: integer;
  utf8Ln  : string;

  procedure ReverseBlock;
  var
    tmp: byte;
    i  : cardinal;
  begin
  {$R-}
    i := 1;
    while i < size do begin
      tmp := buf[i];
      buf[i] := buf[i+1];
      buf[i+1] := tmp;
      Inc(i,2);
    end; //while
  {$R+}
  end; { ReverseBlock }

  procedure FetchBlock;
  begin
    bufPos := FilePos;
    BlockRead(buf,SizeOf(buf)-2,size);
    if cfReverseByteOrder in tfCFlags then
      ReverseBlock;
    leof := (size < (SizeOf(buf)-2));
    // simplify LocateDelimiter
    if not leof then
      Dec(size,2)
    else begin
      buf[size+1] := 0;
      buf[size+2] := 0;
    end;
  end; { FetchBlock }

  procedure LocateDelimiter;
  var
    i  : integer;
    isu: boolean;
  begin
  {$R-}
    delimPos := size+1;
    delimLen := 0;
    isu := IsUnicode and (Codepage <> CP_UTF8);
    for i := 1 to size do begin
      if isu then begin
        if ((AcceptedDelimiters = []) or
            (ld000D000A in AcceptedDelimiters)) and
           ((buf[i] = $0D) and (buf[i+1] = $00) and
            (buf[i+2] = $0A) and (buf[i+3] = $00)) then
        begin
          delimPos := i;
          delimLen := 4;
          break;
        end
        else if ((AcceptedDelimiters = []) or
                 (ld2028 in AcceptedDelimiters)) and
                ((buf[i] = $28) and (buf[i+1] = $20)) then
        begin
          delimPos := i;
          delimLen := 2;
          break;
        end;
      end
      else begin
        if ((AcceptedDelimiters = []) or
            (ldCRLF in AcceptedDelimiters)) and
           ((buf[i] = $0D) and (buf[i+1] = $0A)) then
        begin
          delimPos := i;
          delimLen := 2;
          break;
        end
        else if ((AcceptedDelimiters = []) or
                 (ldLFCR in AcceptedDelimiters)) and
                 ((buf[i] = $0A) and (buf[i+1] = $0D)) then
        begin
          delimPos := i;
          delimLen := 2;
          break;
        end
        else if (((AcceptedDelimiters = []) or (ldCR in AcceptedDelimiters)) and (buf[i] = $0D)) or
                (((AcceptedDelimiters = []) or (ldLF in AcceptedDelimiters)) and (buf[i] = $0A)) then
        begin
          delimPos := i;
          delimLen := 1;
          break;
        end;
      end;
    end; //for
  {$R+}
  end; { LocateDelimiter }

  procedure PasteAndSeek;  
  var
    ansiLn: AnsiString;
    wideLn: WideString;
  begin
    if IsUnicode then begin
      if Codepage = CP_UTF8 then begin
        SetLength(ansiLn,delimPos-1);
        if Length(ansiLn) > 0 then
          Move(buf[1],ansiLn[1],Length(ansiLn));
        utf8Ln := utf8Ln + ansiLn;
      end
      else begin
        SetLength(wideLn,(delimPos-1) div SizeOf(WideChar));
        if Length(wideLn) > 0 then
          Move(buf[1],wideLn[1],Length(wideLn)*SizeOf(WideChar));
        resLn := resLn + wideLn;
      end;
    end
    else begin
      SetLength(ansiLn,delimPos-1);
      if Length(ansiLn) > 0 then
        Move(buf[1],ansiLn[1],Length(ansiLn));
      resLn := resLn + StringToWideString(ansiLn,tfCodePage);
    end;
    Seek(bufPos+delimPos+delimLen-1);
  end; { PasteAndSeek }

begin { TGpTextFile.Readln }
  try
    if Codepage = CP_UTF8 then
      utf8Ln := ''
    else
      resLn := '';
    repeat
      FetchBlock;
      if size = 0 then
        break;
      LocateDelimiter;
      PasteAndSeek;
      if delimLen > 0 then
        break;
    until leof;
    if Codepage = CP_UTF8 then begin
      if utf8Ln = '' then
        Result := ''
      else begin
        SetLength(Result,Length(utf8Ln)); // worst case
        uniBytes := UTF8BufToWideCharBuf(utf8Ln[1],Length(utf8Ln),Result[1],leftUtf8);
        SetLength(Result,uniBytes div SizeOf(WideChar));
      end;
    end
    else
      Result := resLn;
  except
    on E: EGpTextFile do raise;
    on E: EGpHugeFile do raise;
    on E: Exception   do raise EGpTextFile.CreateHelp(E.Message,hcTFUnexpected);
  end;
end; { TGpTextFile.Readln }

{:Simplest form of Reset.
  @param   bufferSize Size of buffer. 0 means default size (BUF_SIZE, currently
                      64 KB).
  @param   flags      Open flags. 
  @param   codePage   Code page to be used for 8/16/8 bit conversion. If 0,
                      default code page for currently used language will be
                      used.
  @raises  EGpTextFile if file could not be reset.
}
procedure TGpTextFile.Reset(flags: TOpenFlags; bufferSize: integer; 
  codePage: word);
begin
  if ResetSafe(flags,bufferSize,0,0,0,codePage) <> hfOK then
    raise EGpTextFile.CreateFmtHelp(sFailedToResetFile,[FileName],hcTFFailedToReset);
end; { TGpTextFile.Reset }

{:Full form of Reset. Will retry if file is locked by another application (if
  diskLockTimeout and diskRetryDelay are specified). Allows caller to specify
  additional options. Does not raise an exception on error.
  @param   flags           Open flags.
  @param   bufferSize      Size of buffer. 0 means default size (BUF_SIZE,
                           currently 64 KB).
  @param   diskLockTimeout Max time (in milliseconds) Reset will wait for lock
                           file to become free.
  @param   diskRetryDelay  Delay (in milliseconds) between attempts to open
                           locked file.
  @param   waitObject      Handle of 'terminate' event (semaphore, mutex). If
                           this parameter is specified (not zero) and becomes
                           signalled, Reset will stop trying to open locked file
                           and will exit with.
  @param   codePage        Code page to be used for 8/16/8 bit conversion. If 0,
                           default code page for currently used language will be
                           used.
  @raises  EGpHugeFile on Windows errors.
}
function TGpTextFile.ResetSafe(flags: TOpenFlags; bufferSize: integer;
  diskLockTimeout, diskRetryDelay: integer; waitObject: THandle;
  codePage: word): THFError;
var
  marker : WideChar;
  marker3: Char;
  options: THFOpenOptions;
begin
  try
    SetCodepage(codePage);
    PrepareBuffer;
    options := [hfoBuffered];
    if ofCloseOnEOF in flags then
      options := options + [hfoCloseOnEOF];
    Result := ResetEx(1,bufferSize,diskLockTimeout,diskRetryDelay,options,waitObject);
    if Result = hfOK then begin
      tfCFlags := [];
      if FileSize >= SizeOf(WideChar) then begin
        BlockReadUnsafe(marker,SizeOf(WideChar));
        if marker = CUnicodeNormal then begin
          tfCFlags := tfCFlags + [cfUnicode];
          Codepage := 1200;
        end
        else if marker = CUnicodeReversed then begin
          tfCFlags := tfCFlags + [cfUnicode,cfReverseByteOrder];
          Codepage := 1200;
        end
        else if (marker = CUTF8BOM12) and (FileSize >= 3) then begin
          BlockReadUnsafe(marker3,SizeOf(Char));
          if marker3 = CUTF8BOM3 then begin
            tfCFlags := tfCFlags + [cfUnicode];
            Codepage := CP_UTF8;
          end;
        end;
        if not IsUnicode then
          Seek(0);
      end;
      if (not IsUnicode) and ((Codepage = CP_UTF8) or (Codepage = CP_UNICODE)) then
        tfCFlags := tfCFlags + [cfUnicode];
    end;
  except
    Result := hfError;
  end;
end; { TGpTextFile.ResetSafe }

{:Simplest form of Rewrite.
  @param   flags      Create flags. 
  @param   bufferSize Size of buffer. 0 means default size (BUF_SIZE, currently
                      64 KB).
  @param   codePage   Code page to be used for 8/16/8 bit conversion. If 0,
                      default code page for currently used language will be
                      used.
  @raises  EGpTextFile if file could not be appended.
}
procedure TGpTextFile.Rewrite(flags: TCreateFlags; bufferSize: integer;
  codePage: word);
begin
  if RewriteSafe(flags,bufferSize,0,0,0,codePage) <> hfOK then
    raise EGpTextFile.CreateFmtHelp(sFailedToRewriteFile,[FileName],hcTFFailedToRewrite);
end; { TGpTextFile.Rewrite }

{:Full form of Rewrite. Will retry if file is locked by another application (if
  diskLockTimeout and diskRetryDelay are specified). Allows caller to specify
  additional options. Does not raise an exception on error.
  @param   flags           Create flags.
  @param   bufferSize      Size of buffer. 0 means default size (BUF_SIZE,
                           currently 64 KB).
  @param   diskLockTimeout Max time (in milliseconds) Rewrite will wait for
                           lock file to become free.
  @param   diskRetryDelay  Delay (in milliseconds) between attempts to open
                           locked file.
  @param   waitObject      Handle of 'terminate' event (semaphore, mutex). If
                           this parameter is specified (not zero) and becomes
                           signalled, Rewrite will stop trying to open locked
                           file and will exit with.
  @param   codePage        Code page to be used for 8/16/8 bit conversion. If 0,
                           default code page for currently used language will be
                           used.
  @raises  EGpTextFile if file is 'reversed' Unicode file.
  @raises  EGpHugeFile on Windows errors.
}
function TGpTextFile.RewriteSafe(flags: TCreateFlags; bufferSize: integer;
  diskLockTimeout, diskRetryDelay: integer; waitObject: THandle;
  codePage: word): THFError;
begin
  if (cfUnicode in flags) and (codePage <> CP_UTF8) then
    codePage := CP_UNICODE;
  PrepareBuffer;
  if ((Codepage = CP_UTF8) or (Codepage = CP_UNICODE)) then
    flags := flags + [cfUnicode];    
  if flags*[cfUnicode,cfReverseByteOrder] = [cfUnicode,cfReverseByteOrder] then
    raise EGpTextFile.CreateFmtHelp(sCannotWriteReversedUnicodeFile,[FileName],hcTFCannotWriteReversed);
  try
    SetCodepage(codePage);
    Result := RewriteEx(1,bufferSize,diskLockTimeout,diskRetryDelay,[hfoBuffered],waitObject);
    if Result = hfOK then begin
      Truncate;
      tfCFlags := flags;
      if IsUnicode then begin
        if Codepage <> CP_UTF8 then
          BlockWriteUnsafe(CUnicodeNormal,SizeOf(WideChar))
        else if cfWriteUTF8BOM in flags then begin
          BlockWriteUnsafe(CUTF8BOM12,SizeOf(WideChar));
          BlockWriteUnsafe(CUTF8BOM3,SizeOf(Char));
        end;
      end;
    end;
  except
    Result := hfError;
  end;
end; { TGpTextFile.RewriteSafe }

{:Internal method that sets current code page or locates default code page if
  0 is passed as a parameter.
  @param   cp Code page number or 0 for default code page.
}
procedure TGpTextFile.SetCodepage(cp: word);
begin
  if (cp = CP_UTF8) or (cp = CP_UNICODE) then begin
    tfCodePage := cp;
    tfCFlags := tfCFlags + [cfUnicode];
  end
  else begin
    if (cp = 0) and (not IsUnicode) then
      tfCodePage := GetDefaultAnsiCodepage(GetSystemDefaultLCID and $FFFF, 1252)
    else
      tfCodePage := cp;
    if not ((tfCodePage = 0) or (tfCodePage = CP_UNICODE)) then
      tfCFlags := tfCFlags - [cfUnicode];
  end;
end; { TGpTextFile.SetCodepage }

{:Writes string to the text file. If file is 8-bit, string is converted
  according to Codepage property.
  @param   ws String to be written.
  @raises  EGpHugeFile on Windows errors.
}
procedure TGpTextFile.WriteString(ws: WideString);
var
  ansiLn  : AnsiString;
  numBytes: integer;
  numChar : integer;
  tmpBuf  : pointer;
begin
  if ws = '' then Exit;
  if IsUnicode then begin
    if Codepage = CP_UTF8 then begin
      numChar := Length(ws);
      tmpBuf := AllocTmpBuffer(numChar*3); // worst case - 3 bytes per character
      try
        numBytes := WideCharBufToUTF8Buf(ws[1],Length(ws)*SizeOf(WideChar),tmpBuf^);
        BlockWriteUnsafe(tmpBuf^,numBytes);
      finally FreeTmpBuffer(tmpBuf); end;
    end
    else
      BlockWriteUnsafe(ws[1],Length(ws)*SizeOf(WideChar))
  end
  else begin
    ansiLn := WideStringToString(ws,tfCodePage);
    BlockWriteUnsafe(ansiLn[1],Length(ansiLn));
  end;
end; { TGpTextFile.WriteString }

{:Writes array of values to the text file. If file is 8-bit, values are
  converted according to Codepage property.
  @param   Values.
  @raises  EGpTextFile on unsupported parameter.
  @raises  EGpHugeFile on Windows errors.
}
procedure TGpTextFile.Write(params: array of const);
var
  i     : integer;
  wideLn: WideString;
const
  BoolChars: array [boolean] of char = ('F','T');
begin
  try
    wideLn := '';
    for i := 0 to High(params) do begin
      with params[i] do begin
        case VType of
          vtInteger:    wideLn := wideLn + IntToStr(VInteger);
          vtBoolean:    wideLn := wideLn + BoolChars[VBoolean];
          vtChar:       wideLn := wideLn + StringToWideString(VChar,tfCodePage);
          vtExtended:   wideLn := wideLn + StringToWideString(FloatToStr(VExtended^),tfCodePage);
          vtString:     wideLn := wideLn + StringToWideString(VString^,tfCodePage);
          vtPointer:    wideLn := wideLn + IntToHex(integer(VPointer),8);
          vtPChar:      wideLn := wideLn + StringToWideString(VPChar,tfCodePage);
          vtObject:     wideLn := wideLn + StringToWideString(VObject.ClassName,tfCodePage);
          vtClass:      wideLn := wideLn + StringToWideString(VClass.ClassName,tfCodePage);
          vtWideChar:   wideLn := wideLn + VWideChar;
          vtPWideChar:  wideLn := wideLn + VPWideChar^;
          vtAnsiString: wideLn := wideLn + StringToWideString(string(VAnsiString),tfCodePage);
          vtCurrency:   wideLn := wideLn + StringToWideString(CurrToStr(VCurrency^),tfCodePage);
          vtVariant:    wideLn := wideLn + StringToWideString(string(VVariant^),tfCodePage);
          vtWideString: wideLn := wideLn + WideString(VWideString);
          vtInt64:      wideLn := wideLn + IntToStr(VInt64^);
          else raise EGpTextFile.CreateFmtHelp(sInvalidParameter,[FileName],hcTFInvalidParameter);
        end;
      end;
    end;
    WriteString(wideLn);
  except
    on E: EGpTextFile do raise;
    on E: EGpHugeFile do raise;
    on E: Exception   do raise EGpTextFile.CreateHelp(E.Message,hcTFUnexpected);
  end;
end; { TGpTextFile.Write }

{$IFDEF D4plus}
procedure TGpTextFile.Write(s: string);
begin
  WriteString(StringToWideString(s,tfCodePage));
end; { TGpTextFile.Write }
{$ENDIF D4plus}

{:Writes line to the text file. If file is 8-bit, values are converted
  according to Codepage property. Uses line delimiter set in Rewrite/Append.
  @param   ln Line to be written.
  @raises  EGpHugeFile on Windows errors.
  @seeAlso Rewrite, Append
}
procedure TGpTextFile.Writeln(ln: WideString);
var
  ansiLn: AnsiString;
begin
  try
    WriteString(ln);
    if IsUnicode then begin
      if cfUse2028 in tfCFlags then 
        WriteString(WideChar($2028))
      else begin
        WriteString(WideChar($000D));
        WriteString(WideChar($000A));
      end;
    end
    else begin
      if cfUseLF in tfCFlags then
        ansiLn := Char($0D)
      else
        ansiLn := Char($0D)+Char($0A);
      BlockWriteUnsafe(ansiLn[1],Length(ansiLn));
    end;
  except
    on E: EGpTextFile do raise;
    on E: EGpHugeFile do raise;
    on E: Exception   do raise EGpTextFile.CreateHelp(E.Message,hcTFUnexpected);
  end;
end; { TGpTextFile.Writeln }

{:Writes array of values to the text file then terminates the line with line
  delimiter. If file is 8-bit, values are converted according to Codepage
  property. Uses line delimiter set in Rewrite/Append.
  @param   Values.
  @raises  EGpTextFile on unsupported parameter.
  @raises  EGpHugeFile on Windows errors.
  @seeAlso Rewrite, Append
}
procedure TGpTextFile.Writeln(params: array of const);
begin
  Write(params);
  Writeln('');
end; { TGpTextFile.Writeln }

{ TGpTextFileStream }

{:Opens file in required access mode, then passes the file stream to the
  inherited constructor.
  @param   fileName    Name of file to be accessed.
  @param   access      Required access mode.
  @param   openFlags   Open flags (used when access mode is accReset).
  @param   createFlags Create flags (used when access mode is accRewrite or
                       accAppend).
  @param   codePage    Code page to be used for 8/16/8 bit conversions. If set
                       to 0, current default code page will be used.
}
constructor TGpTextFileStream.Create(const fileName: string;
  access: TGpHugeFileStreamAccess; openFlags: TOpenFlags;
  createFlags: TCreateFlags; codePage: word);
var
  openOptions: THFOpenOptions;
begin
  openOptions := [hfoBuffered];
  if (access = GpHugeF.accRead) and (ofCloseOnEOF in openFlags) then
    openOptions := openOptions + [hfoCloseOnEOF];
  tfsStream := TGpHugeFileStream.Create(fileName,access,openOptions);
  inherited Create(tfsStream,TGpTSAccess(access),TGpTSCreateFlags(createFlags),
    codePage);
end; { TGpTextFileStream.Create }

destructor TGpTextFileStream.Destroy;
begin
  inherited;
  tfsStream.Free;
end; { TGpTextFileStream.Destroy }

{:Returns file name.
  @returns Returns file name or empty string if file is not open.
}
function TGpTextFileStream.GetFileName: string;
begin
  if assigned(tfsStream) then
    Result := tfsStream.FileName
  else
    Result := '';
end; { TGpTextFileStream.GetFileName }

{:Returns last Windows error code.
  @returns Last Windows error code.
}
function TGpTextFileStream.GetWindowsError: DWORD;
begin
  Result := inherited GetWindowsError;
  if (Result = 0) and assigned(tfsStream) then
    Result := tfsStream.WindowsError;
end; { TGpTextFileStream.GetWindowsError }

{:Returns error message prefix.
  @param   param Optional parameter to be added to the message prefix.
  @returns Error message prefix.
  @since   2001-05-15 (3.0)
}
function TGpTextFileStream.StreamName(param: string): string;
begin
  Result := 'TGpTextFileStream';
  if param <> '' then
    Result := Result + '.' + param;
  Result := Result + '(' + FileName + ')';
end; { TGpTextFileStream.StreamName }

end.
