{ $Id: PGCell.pas,v 1.7 2001/02/20 13:30:57 mvj Exp $ }

{
    This file is part of the TTranslator 
    Copyright (C) 2001 Polycon Ab

    TTranslator is free software; you can redistribute it and/or modify
    it under the terms of the version 2 of the GNU General Public License
    as published by the Free Software Foundation. For more information
    about licensing, please refer to
    http://www.polycon.fi/translator/licensing.html

    TTranslator is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Foobar; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
}

unit PGCell;

interface
{$ifndef WEBAPPLICATION}

uses
  graphics, classes,windows,SysUtils,
  DataType, PGInterface;

type
  TDefaultPGCellFormatter = class (TPGCellFormatter)
  private
    fFixedAccuracy : Boolean;
    fNegativeColor : TColor;
    fUseNegativeCOlor : Boolean;
    fDecimals : Integer;
    fEditorDecimals : Integer;
    fCurrency : Integer;
    fEditorCurrency : Integer;
    fDivisor : Double;
    fCell : TPGCell;
    fFormattedText : string;
    fEditorText : string;
//    fCellContent : TCellContent;
    fCellContent : TValue;
    fDecimalFormatString : string;
    fEditorDecimalFormatString : string;
    fCurrencyFormatString : string;
    fEditorCurrencyFormatString : string;
    procedure SetDivisor( Value : Double );
  protected
    DrawInNegativeColor : Boolean;
    { property get/set }
    function getIsEmpty : Boolean;override;
    function GetEditorText : String;
    procedure setCell( value : TPGCell);override;
    function getCell : TPGCell;override;
    procedure setDecimals( Value : Integer );
    procedure setEditorDecimals( Value : Integer );
    procedure setCurrency( Value : Integer );
    procedure setEditorCurrency( Value : Integer );
    procedure GenerateFormattedText;
    procedure GenerateFormattedInteger;
    procedure GenerateFormattedDecimal;
    procedure GenerateFormattedCurrency;
  public
    constructor Create;
    destructor Destroy;override;
    procedure RenderValue( Canvas : TCanvas; Rect : TRect );override;
    function ContentWidth( Canvas : TCanvas ) : Integer;override;
    function ContentHeight( Canvas : TCanvas ) : Integer;override;
    {/** Use fixed accuracy; UNIMPLEMENTED! */}
    property FixedAccuarcy : Boolean read FFixedAccuracy write FFixedAccuracy;
    {/** Set to true if negative color use, false if color same regardless of sign.
        Default true. */}
    property UseNegativeColor : Boolean read FUseNegativeColor write FUseNegativeColor;
    {/** Color to use for negative numbers. Default clRed */}
    property NegativeColor : TColor read FNegativeColor write FNegativeColor;
    {/** Divisor (default 1.0) */}
    property Divisor : Double read FDivisor write SetDivisor;
    {/** Decimals (0=ununsed). Default 2. */}
    property Decimals : Integer read FDecimals write SetDecimals;
    {/** Decimals in the editor (0=ununsed). Default 2. */}
    property EditorDecimals : Integer read fEditorDecimals write SetEditorDecimals;
    {/** Decimals used for currency (0=ununsed). Default 2. */}
//    property CurrencyDecimals : Integer read FCurrency write SetCurrency;
    {/** Decimals in the editor (0=ununsed). Default 2. */}
    property EditorCurrency : Integer read fEditorCurrency write SetEditorCurrency;
    {/** Get the formatted text for the inplace editor */}
    property EditorText : String read GetEditorText;
    property FormattedText : String read fFormattedText;
  end;

  TDefaultPGCellContentParser = class ( TPGCellContentParser )
  private
//    FDecimals : Integer;
    FDivisor : Double;
  protected
    constructor Create;
    procedure RemoveBlanks(var S : String);
  public
    function ParseAsText( Value : string; ParsedContent : TCellContent ) : Boolean;override;
    function ParseAsDecimal( Value : string; ParsedContent : TCellContent ) : Boolean;override;
    function ParseAsCurrency( Value : string; ParsedContent : TCellContent ) : Boolean;override;
    function ParseAsInteger( Value : string; ParsedContent : TCellContent ) : Boolean;override;
    function ParseAsPicture( Value : string; ParsedContent : TCellContent ) : Boolean;override;
    {/** Divisor (default 1.0) */}
    property Divisor : Double read FDivisor write FDivisor;
    {/** Decimals (0=ununsed). Default 2. */}
//    property Decimals : Integer read FDecimals write FDecimals;
  end;

{$endif WEBAPPLICATION}
implementation
{$ifndef WEBAPPLICATION}

uses
  PGDataFormatter, DataLib, DerivedDataType;

const
  MSGE_MoreThan = 'More than ';
  MSGE_DecimalsRequestedIfNotABugFixAssert = ' decimals requested. Probably a bug; if not change the number in this assert statment';
  DecimalLimit = 32;

{--- TDefaultPGCellFormatter. ---}

constructor TDefaultPGCellFormatter.Create;
begin
  inherited Create;
  fNegativeColor := clRed;
  fUseNegativeCOlor := True;
  Decimals := 2; // Implicit init of fDecimalFormatString
  EditorDecimals := 2; // Implicit init of fEditorDecimalFormatString
//  CurrencyDecimals := 2; // Implicit init of fCurrencyFormatString
  EditorCurrency := 2; // Implicit init of fEditorCurrencyFormatString
  fDivisor := 1.0;
//  fCellContent := TCellContent.Create;
end;

destructor TDefaultPGCellFormatter.Destroy;
begin
//  fCellContent.Free;
  inherited Destroy;
end;

{/** Generate formatted text */}

procedure TDefaultPGCellFormatter.GenerateFormattedText;
begin
//  Assert( fCellContent.ContentType = cctText,
//         Self.ClassName + '.GenerateFormattedText: Content is not text!');
  DrawInNegativeColor := False;
//  fFormattedText := fCellContent.AsText;
  fFormattedText := AsString(fCellContent);
  fEditorText := fFormattedText;
end;

procedure TDefaultPGCellFormatter.GenerateFormattedInteger;
begin
  DrawInNegativeColor := UseNegativeColor and (AsInteger(fCellContent) < 0);
  fFormattedText := IntToStr(CRound(AsInteger(fCellContent) / Divisor));
  fEditorText := fFormattedText;
end;

procedure TDefaultPGCellFormatter.GenerateFormattedDecimal;
begin
  drawInNegativeColor := UseNegativeColor and (AsDouble(fCellContent) < 0.0);
  fFormattedText := Trim(FormatFloat(fDecimalFormatString,
    AsDouble(fCellContent)/ Divisor));
  fEditorText := Trim(FormatFloat(fEditorDecimalFormatString,
    AsDouble(fCellContent)/ Divisor));
end;

procedure TDefaultPGCellFormatter.GenerateFormattedCurrency;
var
  AValue : Currency;
begin
  drawInNegativeColor := UseNegativeColor and (AsCurrency(fCellContent) < 0.0);
  AValue := AsCurrency(fCellContent) / Divisor;
  fFormattedText := Trim(FormatCurr(fDecimalFormatString, AValue));
  fEditorText := Trim(FormatCurr(fEditorCurrencyFormatString, AValue));
end;

function TDefaultPGCellFormatter.ContentWidth( Canvas : TCanvas ) : Integer;
begin
  if Cell.DataType is TPictureType then
    Result := AsPicture(fCellContent).Width
  else
    Result := Canvas.TextWidth( fFormattedText );
end;

function TDefaultPGCellFormatter.ContentHeight( Canvas : TCanvas ) : Integer;
begin
  if Cell.DataType is TPictureType then
    Result := AsPicture(fCellContent).Height
  else
    Result := Canvas.TextHeight( fFormattedText );
end;

procedure TDefaultPGCellFormatter.RenderValue( Canvas : TCanvas; Rect : TRect );
begin
  if Cell.DataType is TPictureType then
    Canvas.Draw( Rect.Left, Rect.Top, AsPicture(fCellContent).Graphic )
  else begin
    if drawInNegativeColor then
      Canvas.Font.Color := NegativeColor;
    Canvas.TextOut(Rect.Left,Rect.Top, fFormattedText );
  end;
end;

procedure TDefaultPGCellFormatter.setCell( value : TPGCell);
begin
  fCell := value;
  if fCell = nil then exit;
  fCell.GetValue( fCellContent );
end;

procedure TDefaultPGCellFormatter.setDecimals( Value : Integer );
var i : Integer;
begin
  Assert( Value < DecimalLimit, Self.ClassName + '.setDecimals: ' + MSGE_MoreThan + IntToStr(DecimalLimit) + MSGE_DecimalsRequestedIfNotABugFixAssert);
  if Value < 0 then Exit;
  fDecimalFormatString := '### ### ### ##0';
  fDecimals := Value;
  if (fDecimals > 0) then
  begin
    fDecimalFormatString := fDecimalFormatString + '.';
    for i := 1 to fDecimals do
      fDecimalFormatString := fDecimalFormatString + '0';
  end;
end;

procedure TDefaultPGCellFormatter.setEditorDecimals( Value : Integer );
var i : Integer;
begin
  Assert( Value < DecimalLimit, Self.ClassName + '.setEditorDecimals: ' + MSGE_MoreThan + IntToStr(DecimalLimit) + MSGE_DecimalsRequestedIfNotABugFixAssert);
  if Value < 0 then Exit;
  fEditorDecimalFormatString := '### ### ### ##0';
  fEditorDecimals := Value;
  if (fEditorDecimals > 0) then
  begin
    fEditorDecimalFormatString := fEditorDecimalFormatString + '.';
    for i := 1 to fEditorDecimals do
      fEditorDecimalFormatString := fEditorDecimalFormatString + '#';
  end;
end;

procedure TDefaultPGCellFormatter.setCurrency( Value : Integer );
var i : Integer;
begin
  Assert( Value < DecimalLimit, Self.ClassName + '.setCurrency: ' + MSGE_MoreThan + IntToStr(DecimalLimit) + MSGE_DecimalsRequestedIfNotABugFixAssert);
  if Value < 0 then Exit;
  fCurrencyFormatString := '### ### ### ##0';
  fCurrency := Value;
  if (fCurrency > 0) then
  begin
    fCurrencyFormatString := fCurrencyFormatString + '.';
    for i := 1 to fCurrency do
      fCurrencyFormatString := fCurrencyFormatString + '0';
  end;
end;

procedure TDefaultPGCellFormatter.setEditorCurrency( Value : Integer );
var i : Integer;
begin
  Assert( Value < DecimalLimit, Self.ClassName + '.setEditorCurrency: ' + MSGE_MoreThan + IntToStr(DecimalLimit) + MSGE_DecimalsRequestedIfNotABugFixAssert);
  if Value < 0 then Exit;
  fEditorCurrencyFormatString := '### ### ### ##0';
  fEditorCurrency := Value;
  if (fEditorCurrency > 0) then
  begin
    fEditorCurrencyFormatString := fEditorCurrencyFormatString + '.';
    for i := 1 to fEditorCurrency do
      if i > 2 then
        fEditorCurrencyFormatString := fEditorCurrencyFormatString + '#'
      else
        fEditorCurrencyFormatString := fEditorCurrencyFormatString + '0';
  end;
end;

function TDefaultPGCellFormatter.getCell : TPGCell;
begin
  Result := fCell;
end;

function TDefaultPGCellFormatter.getIsEmpty : Boolean;
begin
  Result := (fCell = nil) ;//or (fCellContent.ContentType = cctEmpty );
end;

function TDefaultPGCellFormatter.GetEditorText : String;
begin
  Result := fEditorText;
end;

procedure TDefaultPGCellFormatter.SetDivisor( Value : Double );
begin
  Assert(Value <> 0, Self.ClassName + '.SetDivisor: Invalid value 0!');
  fDivisor := Value;
end;

{-- TDefaultCellContentParser --}

constructor TDefaultPGCellContentParser.Create;
begin
  inherited Create;
  FDivisor := 1.0;
end;

function TDefaultPGCellContentParser.ParseAsText( Value : string; ParsedContent : TCellContent ) : Boolean;
begin
  ParsedContent.AsText := Value;
  Result := True;
end;

function TDefaultPGCellContentParser.ParseAsDecimal( Value : string; ParsedContent : TCellContent ) : Boolean;
var
  ValueAsDec : Double;
begin
  Result := True;
  try
  begin
    RemoveBlanks(Value);
    ValueAsDec := StrToFloat( Value );
    ParsedContent.AsDecimal := (ValueAsDec * FDivisor);
  end;
  except on EConvertError do
    Result := False;
  end;
end;

function TDefaultPGCellContentParser.ParseAsCurrency( Value : string; ParsedContent : TCellContent ) : Boolean;
var
  ValueAsCurr : Currency;
begin
  Result := True;
  try
  begin
    RemoveBlanks(Value);
    ValueAsCurr := StrToCurr( Value );
    ParsedContent.AsCurrency := (ValueAsCurr * FDivisor);
  end;
  except on EConvertError do
    Result := False;
  end;
end;

function TDefaultPGCellContentParser.ParseAsInteger( Value : string; ParsedContent : TCellContent ) : Boolean;
var
  ValueAsInt : Integer;
begin
  Result := True;
  try
  begin
    RemoveBlanks(Value);
    ValueAsInt := StrToInt( Value );
    ParsedContent.AsInteger := CRound(ValueAsInt * Divisor);
  end;
  except on EConvertError do
    Result := False;
  end;
end;

function TDefaultPGCellContentParser.ParseAsPicture( Value : string; ParsedContent : TCellContent ) : Boolean;
begin
  {Always fail...}
  Assert(False,'TDefaultCellContentParser.ParseAsPicture: Unimplemented');
  Result := False;
end;

procedure TDefaultPGCellContentParser.RemoveBlanks(var S : String);
var
  idx : Integer;
begin
  idx := -1;
  while idx <> 0 do
  begin
    idx := Pos(' ', S);
    if idx <> 0 then
      Delete(S, idx, 1);
  end;
end;

{$endif WEBAPPLICATION}
end.
