{ $Id: PGDataFormatter.pas,v 1.16 2001/09/06 15:25:26 lge Exp $}

{
    This file is part of the TTranslator 
    Copyright (C) 2001 Polycon Ab

    TTranslator is free software; you can redistribute it and/or modify
    it under the terms of the version 2 of the GNU General Public License
    as published by the Free Software Foundation. For more information
    about licensing, please refer to
    http://www.polycon.fi/translator/licensing.html

    TTranslator is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Foobar; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
}

unit PGDataFormatter;

interface
{$ifndef WEBAPPLICATION}

uses
  Windows, Graphics, DerivedDataType,
  Storages, Criteria, DataElements, DataType, PGInterface;

type
  TPGDataFormatter = class(TDataFormatter)
  private
    FNegativeColor : TColor;
    FUseNegativeColor : Boolean;
    FCell : TPGCell;
    FValue : TValue;
    FForEditor : Boolean;
//    FFormattedValue : String;
    FFormattedValue : TValue;
    FDataType : TDataType;
  protected
    { property get/set }
    function GetIsEmpty : Boolean;
    procedure SetCell( ACell : TPGCell); virtual;
    function GetCell : TPGCell;
    function GetDataType : TDataType; virtual;
    procedure SetDataType( AType : TDataType ); virtual;
    procedure PrepareCanvas( Canvas : TCanvas ); virtual;
  public
    constructor Create;
    destructor Destroy; override;

    function ParseText( Value : String; var ParsedValue : TValue;
        var ValueChanged : Boolean; var ErrorString : String ) : Boolean; virtual;
{    function ParseText( Value : String; var ParsedValue : TValue;
             var ErrorString : String ) : Boolean; virtual;}
    function ContentWidth( Canvas : TCanvas ) : Integer;
    function ContentHeight( Canvas : TCanvas ) : Integer;
    function ValueIsNegative : Boolean;

    procedure RenderValue( Canvas : TCanvas; Rect : TRect ); virtual;
    procedure DrawBackground( Canvas : TCanvas; Rect : TRect ); virtual;

  //    property Font : TFont read FFont write FFont;
    property Cell : TPGCell read getCell write setCell;
    property IsEmpty : Boolean read getIsEmpty;
    property ForEditor : Boolean read FForEditor write FForEditor;
    property DataType : TDataType read GetDataType write SetDataType;
    {/** Set to true if negative color use, false if color same regardless of sign.
        Default true. */}
    property UseNegativeColor : Boolean read FUseNegativeColor write FUseNegativeColor;
    {/** Color to use for negative numbers. Default clRed */}
    property NegativeColor : TColor read FNegativeColor write FNegativeColor;
    property FormattedValue : TValue read FFormattedValue;
  end;

{$endif WEBAPPLICATION}
implementation
{$ifndef WEBAPPLICATION}

uses
  SysUtils, Math;

{--- TPGDataFormatter ---}

constructor TPGDataFormatter.Create;
begin
  Inherited Create;
  FNegativeColor := clRed;
  FUseNegativeColor := False;
  FCell := nil;
  FForEditor := False;
end;

destructor TPGDataFormatter.Destroy;
begin
  Inherited Destroy;
end;

function TPGDataFormatter.GetIsEmpty : Boolean;
begin
  Result := (fCell = nil);
end;

procedure TPGDataFormatter.SetCell( ACell : TPGCell);
var
  aDataType : TDataType;
begin
  FDataType := nil;
  FFormattedValue := AnyStringType.DefaultValue;

  FCell := ACell;
  if (FCell = nil) then
    Exit;

  FCell.GetValue( FValue );
  aDataType := DataType;
  if aDataType <> nil then
    FFormattedValue := aDataType.FormattedValue(FValue, Self);
end;

function TPGDataFormatter.GetCell : TPGCell;
begin
  Result := FCell;
end;

function TPGDataFormatter.GetDataType : TDataType;
begin
  Result := FDataType;
  if (Result = nil) and
     (Cell <> nil) then
    Result := Cell.DataType;
  if Result = nil then
    Result := FValue.DataType;
end;

procedure TPGDataFormatter.SetDataType( AType : TDataType);
begin
  FDataType := AType;
end;

procedure TPGDataFormatter.PrepareCanvas( Canvas : TCanvas );
begin
  if UseNegativeColor and ValueIsNegative then
    Canvas.Font.Color := NegativeColor;
end;

procedure TPGDataFormatter.RenderValue( Canvas : TCanvas; Rect : TRect );
begin
  PrepareCanvas( Canvas );
  FCell.CellType.RenderValue(FormattedValue, Canvas, Rect);
end;

procedure TPGDataFormatter.DrawBackground(Canvas: TCanvas; Rect: TRect);
begin
  Canvas.FillRect(Rect);
end;

function TPGDataFormatter.ParseText( Value : String; var ParsedValue : TValue;
         var ValueChanged : Boolean; var ErrorString : String ) : Boolean;
var
  OldValue, OldFormattedValue : TValue;
begin
  try
    Cell.GetValue( OldValue );
    if DataType <> nil then
      OldFormattedValue := DataType.FormattedValue(OldValue, Self);
    ValueChanged := not AnyStringType.Equals( OldFormattedValue, ValueFromString(Value) );
    if ValueChanged then
      Result := DataType.ParseText(Value, ParsedValue, Self)
    else
    begin
      ParsedValue := OldValue;
      Result := False;
    end;
  except
    ErrorString := 'Couldn''t parse "' + Value + '".';
    Result := False;
  end;
end;

function TPGDataFormatter.ContentWidth( Canvas : TCanvas ) : Integer;
var
  APic : TPicture;
  MinWidth : Integer;
begin
  PrepareCanvas( Canvas );
  MinWidth := FCell.CellType.MinWidth;

  if FormattedValue.DataType is TPictureType then
  begin
    APic := AsPicture(FormattedValue);
    if APic = nil then
      Result := 0
    else
      Result := APic.Width;
  end
  else
    Result := Canvas.TextWidth( AsString(FormattedValue) );

  Result := MaxIntValue([MinWidth, Result]);
end;

function TPGDataFormatter.ContentHeight( Canvas : TCanvas ) : Integer;
var
  APic : TPicture;
  MinHeight : Integer;
begin
  PrepareCanvas( Canvas );
  MinHeight := FCell.CellType.MinHeight;

  if FormattedValue.DataType is TPictureType then
  begin
    APic := AsPicture(FormattedValue);
    if APic = nil then
      Result := 0
    else
      Result := APic.Height
  end
  else
    Result := Canvas.TextHeight( AsString(FormattedValue) );

  Result := MaxIntValue([MinHeight, Result]);
end;

function TPGDataFormatter.ValueIsNegative : Boolean;
begin
  Result := DataType.IsNegative[FValue]
end;

{$endif WEBAPPLICATION}

end.
