{ $Id: PGEdit.pas,v 1.24 2001/08/03 08:17:56 laa Exp $ }

{
    This file is part of the TTranslator 
    Copyright (C) 2001 Polycon Ab

    TTranslator is free software; you can redistribute it and/or modify
    it under the terms of the version 2 of the GNU General Public License
    as published by the Free Software Foundation. For more information
    about licensing, please refer to
    http://www.polycon.fi/translator/licensing.html

    TTranslator is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Foobar; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
}

{---------------------------------------------------------
  What:              TPolyGridInplaceEdit
  									 TPGPopupListBox
  Version
  Company:           Polycon Ab
  Authors:
---------------------------------------------------------}

unit PGEdit;

interface

{$i common.inc}

uses SysUtils, Messages, WinTypes, Classes, Graphics, Menus, Controls, Forms,
  StdCtrls, Mask, Grids, Buttons, WinProcs, CellType, PGInterfaces;

type
  TBGPopupListbox = class;

  { TPolyGridInplaceEdit adds support for a button on the in-place editor,
    which can be used to drop down a stringlist-based pick list, or
    (if style is esButton) fire the grid event OnEditButtonClick. }

  TPolyGridInplaceEdit = class(TInplaceEdit, IInplaceEdit)
  private
    FButtonWidth: Integer;
		FPickList: TBGPopupListbox;
    FCellType : TCellType;
    FInfoSupplier : ICellTypeInfoSupplier;
    FListVisible: Boolean;
    FTracking: Boolean;
    FPressed: Boolean;
    { Polygrid new}
    FComboBitmap : graphics.TBitmap;
    fOriginalText : string;
    fOldText : string;
    fAutoFill : Boolean;

    procedure ListMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
//    procedure SetEditStyle(Value: TEditStyle);
    procedure StopTracking;
    procedure TrackButton(X,Y: Integer);
    procedure CMCancelMode(var Message: TCMCancelMode); message CM_CancelMode;
    procedure WMCancelMode(var Message: TMessage); message WM_CancelMode;
    procedure WMKillFocus(var Message: TMessage); message WM_KillFocus;
    procedure WMLButtonDblClk(var Message: TWMLButtonDblClk); message wm_LButtonDblClk;
    procedure WMPaint(var Message: TWMPaint); message wm_Paint;
    procedure WMSetCursor(var Message: TWMSetCursor); message WM_SetCursor;

    // Interface Methods
    procedure InitListBox(Items : TStrings);
    function GetText : String;
    procedure SetText(Value : String);
    procedure AfterButtonKeyDown;
    procedure AfterRenderEditor(ARect : TRect);
    procedure ButtonMouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure ComboMouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure MapEditorToParent(var ARect : TRect);
    procedure GetSize(var Width, Height : Integer);
    function EditorPressed : Boolean;
    procedure doUndo;
    // plain puckoness of delphi, need to define this crap for interfaces
{$ifndef VER130}
    function QueryInterface(const IID: TGUID; out Obj): Integer; stdcall;
{$endif VER130}
    function _AddRef: Integer; stdcall;
    function _Release: Integer; stdcall;
    procedure RefreshCellType;
  protected
    {procedure BoundsChanged; override;}
    function EditCanModify: Boolean; override;

    procedure CloseUp(Accept: Boolean);
    procedure DoDropDownKeys(var Key: Word; Shift: TShiftState);
    procedure DropDown;
		procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure DblClick; override;
    procedure PaintWindow(DC: HDC); override;
    procedure WndProc(var Message: TMessage); override;
  public
    constructor Create(Owner: TComponent); override;
    destructor Destroy;override;
    procedure SetCellType(ACellType: TCellType; ctInfoSupplier : ICellTypeInfoSupplier);
    procedure DeletePress;
    procedure UpdateContents; override;
    property CellType: TCellType read FCellType;// write SetCellType;
    property PickList: TBGPopupListbox read FPickList;
    property OldText : string read fOldText write fOldText;
    property AutoFill : Boolean read fAutoFill;
    procedure UpdateText;
  end;

  { TBGPopupListbox }

  TBGPopupListbox = class(TCustomListbox)
  private
    FSearchText: PChar; {String;}
{$ifndef win32}
    FSearchTickCount : Longint;
{$endif}
  protected
    procedure CreateParams(var Params: TCreateParams); override;
    procedure CreateWnd; override;
{$ifndef win32}
    procedure KeyPress(var Key: Char); override;
{$endif}
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  end;

implementation

uses
  PolyGrid, Dialogs;

procedure ClearPString(PString: PChar; MaxLength: LongInt);
var
  i: LongInt;
begin
  for i := 0 to MaxLength - 1 do
    PString[i] := #0;
end;

constructor TBGPopupListbox.Create(AOwner: TComponent);
begin
  Inherited Create(AOwner);
  FSearchText := StrAlloc(33);
  FSearchText := #0;
end;

destructor TBGPopupListbox.Destroy;
begin
  {StrDispose(FSearchText);}
  Inherited Destroy;
end;

procedure TBGPopupListbox.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  with Params do
  begin
   ExStyle := ExStyle or WS_EX_TOPMOST;
   { Style := Style or WS_BORDER;
    ExStyle := WS_EX_TOOLWINDOW;
    WindowClass.Style := CS_SAVEBITS;}
  end;
end;


procedure TBGPopupListbox.CreateWnd;
begin
  inherited CreateWnd;
  WinProcs.SetParent(Handle, 0);
  WinProcs.CallWindowProc(DefWndProc, Handle, wm_SetFocus, 0, 0);
end;

{$ifndef win32}
procedure TBGPopupListbox.Keypress(var Key: Char);
var
  TickCount: Integer;
begin
  case Key of
    #8, #27: ClearPString(FSearchText, 32);
    #32..#255:
      begin
        TickCount := WinProcs.GetTickCount;
				if TickCount - FSearchTickCount > 2000 then ClearPString(FSearchText, 32);
				FSearchTickCount := TickCount;
        if StrLen(FSearchText) < 31 then FSearchText[StrLen(FSearchText)] := Key;
        WinProcs.SendMessage(Handle, LB_SelectString, WORD(-1), LongInt(FSearchText));
        Key := #0;
      end;
  end;
  inherited Keypress(Key);
end;
{$endif}

procedure TBGPopupListbox.MouseUp(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
begin
  inherited MouseUp(Button, Shift, X, Y);
  TPolyGridInplaceEdit(Owner).CloseUp((X >= 0) and (Y >= 0) and
      (X < Width) and (Y < Height));
end;

{ Interface IInplaceEdit }

// plain puckoness of delphi, need to define this crap for interfaces
{$ifndef VER130}
function TPolyGridInplaceEdit.QueryInterface(const IID: TGUID; out Obj): Integer; stdcall;
const
  E_NOINTERFACE = $80004002;
begin
  if GetInterface(IID, Obj) then Result := 0 else Result := E_NOINTERFACE;
end;
{$endif VER130}

function TPolyGridInplaceEdit._AddRef: Integer; stdcall;
begin
  result := 1;
end;

function TPolyGridInplaceEdit._Release: Integer; stdcall;
begin
  result := 1;
end;

{ PXGridInplaceEdit }

constructor TPolyGridInplaceEdit.Create(Owner: TComponent);
begin
  inherited Create(Owner);
  FButtonWidth := GetSystemMetrics(SM_CXVSCROLL);
  FCellType := NormalCellType;

  FComboBitmap := graphics.TBitmap.Create;
  FComboBitmap.Handle := LoadBitmap(0, PChar(OBM_COMBO )); {DELPHI2: 32738}
  fOldText := '';
  fAutoFill := False;
end;

destructor TPolyGridInplaceEdit.Destroy;
begin
  FComboBitmap.Free;
  inherited Destroy;
end;
{
procedure TPolyGridInplaceEdit.BoundsChanged;
var
  R: TRect;
begin
  Inherited BoundsChanged;
  SetRect(R, 2, 2, Width - 2, Height);
  if FEditStyle <> esSimple then Dec(R.Right, FButtonWidth);
  SendMessage(Handle, EM_SETRECTNP, 0, LongInt(@R));
  SendMessage(Handle, EM_SCROLLCARET, 0, 0);
end;
}
procedure TPolyGridInplaceEdit.UpdateText;
var
  aIdx : Integer;
begin
  if Assigned( FPickList ) then
  begin
    aIdx := FPickList.Items.IndexOf( Text );
    FPickList.ItemIndex := aIdx;
  end;
  Modified := True;
end;

function TPolyGridInplaceEdit.EditCanModify: Boolean;
begin
  Result := Inherited EditCanModify;// True;
end;

procedure TPolyGridInplaceEdit.CloseUp(Accept: Boolean);
var
  ListValue: String;
  IsModified : Boolean;
begin
  IsModified := False;
  ListValue := '';
  if FListVisible then
  begin
    if GetCapture <> 0 then SendMessage(GetCapture, WM_CANCELMODE, 0, 0);
    if FPickList.ItemIndex <> -1 then
      ListValue := FPickList.Items[FPicklist.ItemIndex];

    SetWindowPos(FPickList.Handle, 0, 0, 0, 0, 0, SWP_NOZORDER or
      SWP_NOMOVE or SWP_NOSIZE or SWP_NOACTIVATE or SWP_HIDEWINDOW);
    FListVisible := False;
    Invalidate;
    if Accept then
    begin
      if Text <> ListValue then
        IsModified := True;

      { Save value }
      if (ListValue <> '') and
         EditCanModify then
        Text := ListValue;
    end;
  end;
  if IsModified then
    Modified := True;
end;

procedure TPolyGridInplaceEdit.DoDropDownKeys(var Key: Word; Shift: TShiftState);
begin
  case Key of
    VK_UP, VK_DOWN:
      if ssAlt in Shift then
      begin
        if FListVisible then CloseUp(True) else DropDown;
        Key := 0;
      end;
    VK_RETURN, VK_ESCAPE:
      if FListVisible and not (ssAlt in Shift) then
      begin
        CloseUp(Key = VK_RETURN);
        Key := 0;
      end;
  end;
end;

procedure TPolyGridInplaceEdit.DropDown;
var
  P: TPoint;
  Y: Integer;
  SearchText: PChar;
  MaxLen: LongInt;
begin
  if not FListVisible then
  begin
    FPickList.Width := Width;
    FPickList.Color := Color;
    FPickList.Font := Font;

    { Adjust the height of the listbox }
    if FPickList.Items.Count < 12 then
      FPickList.Height := FPickList.Items.Count * FPickList.ItemHeight + 4
    else
      FPickList.Height := 8 * FPickList.ItemHeight + 4;

    P := Parent.ClientToScreen(Point(Left, Top));
    Y := P.Y + Height;
    if Y + FPickList.Height > Screen.Height then Y := P.Y - FPickList.Height;

    SetWindowPos(FPickList.Handle, HWND_TOP, P.X, Y, 0, 0,
      SWP_NOSIZE or SWP_NOACTIVATE or SWP_SHOWWINDOW);

    { This block of code selects the right line in the listbox }
    if MaxLength = 0 then
    begin
      SearchText := StrAlloc(100);
      MaxLen := 100;
    end
    else
    begin
      SearchText := StrAlloc(MaxLength + 1);
      MaxLen := MaxLength + 1;
    end;
    ClearPString(SearchText, MaxLen);
    StrPCopy(SearchText, Text);
    SendMessage(FPickList.Handle, LB_SelectString, WORD(-1), LongInt(SearchText));
    FPickList.Invalidate;
    StrDispose(SearchText);

    FListVisible := True;
    Invalidate;
    WinTypes.SetFocus(Handle);
  end;
end;

procedure TPolyGridInplaceEdit.KeyDown(var Key: Word; Shift: TShiftState);
var
  ExecInh : Boolean;
begin
  if not ( (Shift = []) or (Shift = [ssShift]) ) or
    ( Key in [VK_LBUTTON..VK_ESCAPE, VK_PRIOR..VK_HELP] ) then
    fAutoFill := False
  else
    fAutoFill := True;

  if Key = VK_ESCAPE then
{$ifdef D4_OR_HIGHER}
    Undo
{$endif D4_OR_HIGHER}
  else if Key in [VK_DELETE] then
    DeletePress;

  ExecInh := True;
  CellType.KeyDown(Key, Shift, Self, FInfoSupplier, ExecInh);
  if ExecInh then
    inherited KeyDown(Key, Shift);
end;

procedure TPolyGridInplaceEdit.AfterButtonKeyDown;
var
  Msg: TMsg;
begin
  PeekMessage(Msg, Handle, WM_CHAR, WM_CHAR, PM_REMOVE);
end;

procedure TPolyGridInplaceEdit.ListMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if Button = mbLeft then
    CloseUp(PtInRect(FPickList.ClientRect, Point(X, Y)));
end;

procedure TPolyGridInplaceEdit.MouseDown(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
var
  ExecInh : Boolean;
begin
  ExecInh := True;
  RefreshCellType;
  CellType.MouseDown(Button, Shift, X, Y, Self, FInfoSupplier, ExecInh);
  if ExecInh then
    inherited MouseDown(Button, Shift, X, Y);
end;

procedure TPolyGridInplaceEdit.ComboMouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if (Button = mbLeft) and
     PtInRect(Rect(Width - FButtonWidth, 0, Width, Height), Point(X,Y)) then
  begin
    if FListVisible then
      CloseUp(False)
    else
    begin
      MouseCapture := True;
      FTracking := True;
      TrackButton(X, Y);
      DropDown;
    end;
  end;
end;

procedure TPolyGridInplaceEdit.ButtonMouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if (Button = mbLeft)  and
     PtInRect(Rect(Width - FButtonWidth, 0, Width, Height), Point(X,Y)) then
  begin
    FTracking := True;
    TrackButton(X, Y);
    Modified := True;
  end;
end;

procedure TPolyGridInplaceEdit.MouseMove(Shift: TShiftState; X, Y: Integer);
var
  ListPos: TPoint;
  MousePos: TSmallPoint;
begin
  if FTracking then
  begin
    TrackButton(X, Y);
    if FListVisible then
    begin
      ListPos := FPickList.ScreenToClient(ClientToScreen(Point(X, Y)));
      if PtInRect(FPickList.ClientRect, ListPos) then
      begin
        StopTracking;
        MousePos := PointToSmallPoint(ListPos);
        SendMessage(FPickList.Handle, WM_LBUTTONDOWN, 0, LongInt(MousePos));
        Exit;
      end;
    end;
  end;
  inherited MouseMove(Shift, X, Y);
end;

procedure TPolyGridInplaceEdit.MouseUp(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
var
  ExecInh : Boolean;
begin
  ExecInh := True;
  RefreshCellType;
  CellType.MouseUp(Button, Shift, X, Y, Self, FInfoSupplier, ExecInh);
  if ExecInh then
    inherited MouseUp(Button, Shift, X, Y);
end;

procedure TPolyGridInplaceEdit.DblClick;
begin
  inherited DblClick;
  TPolyGrid(Parent).ShowAnts := False;
end;

procedure TPolyGridInplaceEdit.UpdateContents;
begin
  inherited UpdateContents;
  fAutoFill := False;
  fOriginalText := Text;
  fOldText := Text;
  if FPickList <> nil then
    SendMessage(FPickList.Handle, LB_SelectString, WORD(-1), LongInt(PChar(Text)));
end;

procedure TPolyGridInplaceEdit.MapEditorToParent(var ARect : TRect);
begin
  MapWindowPoints(Handle, TPolyGrid(Parent).Handle, ARect, 2);
end;

procedure TPolyGridInplaceEdit.GetSize(var Width, Height : Integer);
begin
  Width := Self.Width;
  Height := Self.Height;
end;

function TPolyGridInplaceEdit.EditorPressed : Boolean;
begin
  Result := FPressed;
end;

procedure TPolyGridInplaceEdit.PaintWindow(DC: HDC);
begin
  CellType.RenderEditor(TPolyGrid(Parent).Canvas, Self, FInfoSupplier);
  inherited PaintWindow(DC);
end;

procedure TPolyGridInplaceEdit.AfterRenderEditor(ARect : TRect);
begin
  SendMessage(Handle, EM_SETRECTNP, 0, LongInt(@ARect));
end;

procedure TPolyGridInplaceEdit.SetCellType(ACellType: TCellType; ctInfoSupplier : ICellTypeInfoSupplier);
begin
  FCellType := ACellType;
  FInfoSupplier := ctInfoSupplier;
  CellType.Activate(Self, ctInfoSupplier);
  Repaint;
end;

procedure TPolyGridInplaceEdit.InitListBox(Items : TStrings);
begin
  if FPickList = nil then
  begin
    FPickList := TBGPopupListbox.Create(Self);
    FPickList.Visible := False;
    FPickList.Parent := Self;
    FPickList.OnMouseUp := ListMouseUp;
    FPickList.IntegralHeight := True;
    FPickList.ItemHeight := 11;
  end;
  FPickList.Items := Items;
end;

function TPolyGridInplaceEdit.GetText : String;
begin
  Result := Text;
end;

procedure TPolyGridInplaceEdit.SetText(Value : String);
begin
  Text := Value;
end;

procedure TPolyGridInplaceEdit.StopTracking;
begin
  if FTracking then
  begin
    TrackButton(-1, -1);
    FTracking := False;
    MouseCapture := False;
  end;
end;

procedure TPolyGridInplaceEdit.TrackButton(X,Y: Integer);
var
  NewState: Boolean;
  R: TRect;
begin
  SetRect(R, ClientWidth - FButtonWidth, 0, ClientWidth, ClientHeight);
  NewState := PtInRect(R, Point(X, Y));

  if FPressed <> NewState then
  begin
    FPressed := NewState;
    InvalidateRect(Handle, @R, False);
  end;
end;

procedure TPolyGridInplaceEdit.RefreshCellType;
begin
  TPolyGrid(Parent).RefreshCell;
end;

procedure TPolyGridInplaceEdit.doUndo;
begin
  Text := fOriginalText;
  SelStart := 0;
  SelLength := Length( Text );
  UpdateText;
end;

procedure TPolyGridInplaceEdit.DeletePress;
var
  AStr : String;
  CursPos : Integer;
//  ShouldAutoFill : Boolean;
begin
//  ShouldAutoFill := fAutoFill;
  fAutoFill := False;
  if SelLength > 0 then
    SelText := ''
  else
  begin
    CursPos := SelStart;
    AStr := Text;
    Delete( AStr, SelStart+1, 1 );
    Text := AStr;
    SelStart := CursPos;
  end;
  fAutoFill := True;//ShouldAutoFill;
  UpdateText;
end;

procedure TPolyGridInplaceEdit.CMCancelMode(var Message: TCMCancelMode);
begin
  if (Message.Sender <> Self) and (Message.Sender <> FPickList) then
    CloseUp(False);
end;

procedure TPolyGridInplaceEdit.WMCancelMode(var Message: TMessage);
begin
  StopTracking;
  inherited;
end;

procedure TPolyGridInplaceEdit.WMKillFocus(var Message: TMessage);
var index : Integer;
begin
  inherited;
  CloseUp(False);

  if EditCanModify then
  begin
    { Save value & possible picklist index }
    { NOTE!!!! The picklist index is currently unimplemented and always set to -1!}
    if (FPickList <> nil) and
       (CellType is TComboCellType) then
    begin
      index := FPickList.ItemIndex;
      { Not selected from list; maybe the user typed a correct entry? }
      if index = -1 then
        index := FPickList.Items.IndexOf(Text);
    end
    else
      Index := -1;

    TPolyGrid(Parent).SetCell(Text, index );
  end;
end;

procedure TPolyGridInplaceEdit.WMLButtonDblClk(var Message: TWMLButtonDblClk);
begin
  with Message do
  if (CellType is TButtonEditCellType) and
     PtInRect(Rect(Width - FButtonWidth, 0, Width, Height), Point(XPos, YPos)) then
    Exit;
  inherited;
end;

procedure TPolyGridInplaceEdit.WMPaint(var Message: TWMPaint);
begin
//  inherited;//WMPaint(Message);
//  CellType.RenderEditor(TPolyGrid(Parent).Canvas, Self, FInfoSupplier);
  PaintHandler(Message);
end;

procedure TPolyGridInplaceEdit.WMSetCursor(var Message: TWMSetCursor);
var
  P: TPoint;
begin
  GetCursorPos(P);
  if (CellType is TButtonEditCellType) and
    PtInRect(Rect(Width - FButtonWidth, 0, Width, Height), ScreenToClient(P)) then
    SetCursor(LoadCursor(0, idc_Arrow))
  else
    inherited;
end;

procedure TPolyGridInplaceEdit.WndProc(var Message: TMessage);
begin
  case Message.Msg of
    wm_KeyDown, wm_SysKeyDown, wm_Char:
      if (CellType is TComboCellType) then
      with TWMKey(Message) do
      begin
        DoDropDownKeys(CharCode, KeyDataToShiftState(KeyData));
        if (CharCode <> 0) and FListVisible then
        begin
          with TMessage(Message) do
            SendMessage(FPickList.Handle, Msg, WParam, LParam);
          Exit;
        end;
      end;
    WM_UNDO: doUndo;
  end;
  inherited WndProc(Message);
end;

end.
