{ $Id: PGInterface.pas,v 1.15 2000/11/13 12:53:36 laa Exp $}

{
    This file is part of the TTranslator 
    Copyright (C) 2001 Polycon Ab

    TTranslator is free software; you can redistribute it and/or modify
    it under the terms of the version 2 of the GNU General Public License
    as published by the Free Software Foundation. For more information
    about licensing, please refer to
    http://www.polycon.fi/translator/licensing.html

    TTranslator is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Foobar; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
}

unit PGInterface;

interface
{$ifndef WEBAPPLICATION}

uses
  Graphics, Classes, Windows, Sysutils, Controls,
  DataType, DataTypes, PGInterfaces, PGTypes, CellType;

type
  TPGCell = class;

  TCellContent = class
  private
    fContentType : TCellContentType;
    fText : string;
    fDecimal : Double;
    fCurrency : Currency;
    fInteger : Integer;
    fPicture : TPicture;
  protected
    procedure setAsText( value : String );
    procedure setAsInteger( value : Integer );
    procedure setAsDecimal( value : Double  );
    procedure setAsCurrency( value : Currency  );
    procedure setAsPicture( value : TPicture  );
    function getAsText : string;
    function getAsDecimal : Double;
    function getAsCurrency : Currency;
    function getAsInteger : Integer;
    function getAsPicture : TPicture;
  public
    procedure Assign( value : TCellContent );
    property ContentType : TCellContentType read fContentType write fContentType;
    property AsText : string read getAsText write setAsText;
    property AsDecimal : Double read getAsDecimal write setAsDecimal;
    property AsCurrency : Currency read getAsCurrency write setAsCurrency;
    property AsInteger : Integer read getAsInteger write setAsInteger;
    property AsPicture : TPicture read getAsPicture write setAsPicture;
  end;

  {/** The abstract class TGridCell class defines the interface that cells
       shown in a polypolygrid must support. The class of "Real" cells in a polygrid
       are of a derived type. */}

  TPGCell = class(TObject, IIconInfoSupplier, ICellTypeInfoSupplier)
  protected
    FCol : Integer;
    FRow : Integer;
    fCellType : TCellType;
    fCellAlign : TCellAlign;
    fReadOnly : Boolean;
    fCalculated : Boolean;
    fChecked : Boolean;
    fSelected : Boolean;
    fBorderStyle : TBorderStyle;
    fFillEmpty : Boolean;
    fItemIndex : Integer;
    fValid : Boolean;
    fFixed : Boolean;
    fSetCellForCheckBox : Boolean;
//    FDataType : TDataType;
    function getCellType : TCellType; virtual;
    procedure setCellType( value : TCellType ); virtual;
    function getCellAlign : TCellAlign; virtual;
    procedure setCellAlign( value : TCellAlign ); virtual;
    function getReadOnly : Boolean; virtual;
    procedure setReadOnly( value : Boolean ); virtual;
    function getValid : Boolean; virtual;
    procedure setValid( value : Boolean ); virtual;
    function getFixed : Boolean; virtual;
    procedure setFixed( value : Boolean ); virtual;
    function getCalculated: Boolean; virtual;
    procedure setCalculated( value : Boolean ); virtual;
    function getChecked : Boolean; virtual;
    procedure setChecked( value : Boolean ); virtual;
    function getSelected : Boolean; virtual;
    procedure setSelected( value : Boolean ); virtual;
    function getBorderStyle : TBorderStyle; virtual;
    procedure setBorderStyle( value : TBorderStyle ); virtual;
    function getFillEmpty : Boolean; virtual;
    procedure setFillEmpty( value : Boolean ); virtual;
    function getItemIndex : Integer; virtual;
    procedure setItemIndex( value : Integer ); virtual;
    function getEditorText : string; virtual;
//    procedure setDataType( AType : TDataType ); virtual;
    function getDataType : TDataType; virtual;

    // methods from the Interface IconInfoSupplier
    function GetIcon(Sender : TObject; APicture : TPicture) : Boolean; virtual;
    function GetIconAlign(Sender : TObject; DefaultIconAlign : TCellAlign) : TCellAlign; virtual;

    // methods from the Interface ICellTypeInfoSupplier
    procedure EditButtonClick(var Value : TValue); virtual;
    function GetEditorFace(Sender : TObject; APicture : TPicture) : Boolean; virtual;

    // plain puckoness of delphi, need to define this crap for interfaces
{$ifdef VER130}
    function QueryInterface(const IID: TGUID; out Obj): HResult; stdcall;
{$else}
    function QueryInterface(const IID: TGUID; out Obj): Integer; stdcall;
{$endif VER130}
    function _AddRef: Integer; stdcall;
    function _Release: Integer; stdcall;
  public
    constructor Create;
    {/** Returns value of cell trough TFormattedValue Object */}
//    procedure GetValue( Value : TCellContent); virtual; abstract;
    procedure GetValue(var Value : TValue); virtual; abstract;
    {/** Set value of cell */}
//    procedure SetValue( Value : TCellContent; var Reject : Boolean;
    procedure SetValue(Value : TValue; var Reject : Boolean;
      var RejectReason : string ); virtual; abstract;
    {/** Reset the cell */}
    procedure ResetAll; virtual;
    {/** return combo box strings if celltype is combo*/}
    function GetComboStrings : TStrings; virtual; abstract;

    property Col : Integer read FCol write FCol;
    property Row : Integer read FRow write FRow;
    {/** Type of Cell ( Normal, Combo, Checkbox, Button ) */}
    property CellType : TCellType read getCellType write setCellType;
    {/** Alignment of cell */}
    property CellAlign : TCellAlign read getCellAlign write setCellAlign;
    {/** Is cell readonly */}
    property ReadOnly : Boolean read getReadOnly write setReadOnly;
    {/** Is this cell valid */}
    property Valid : Boolean read getValid write setValid;
    {/** Is this cell fixed/locked */}
    property Fixed : Boolean read getFixed write setFixed;
    {/** Cell is calculated */}
    property Calculated : Boolean read getCalculated write setCalculated;
    {/** Is cell checked */}
    property Checked : Boolean read getChecked write setChecked;
    {/** Show sortorder icon */}
//    property ShowIcon : Boolean read GetShowIcon write SetShowIcon;
    {/** In what order is the data sorted */}
//    property SortOrder : TSortOrder read GetSortOrder write SetSortOrder;
    {/** Is cell selected */}
    property Selected : Boolean read getSelected write setSelected;
    {/** Border style of Cell*/}
    property BorderStyle : TBorderStyle read getBorderStyle write setBorderStyle;
    {/** Fill empty cells */}
    property FillEmpty : Boolean read getFillEmpty write setFillEmpty;
    {/** Item index for (for combobox type cells). -1 -> value chosen not in Combo */}
    property ItemIndex : Integer read getItemIndex write setItemIndex;
    {/** Text to show in the editor. Default implementation is a simple
      conversion of Value (eg value=(float) 1.333 -> EditorText = '1.333' */}
    property EditorText : string read getEditorText;
    property DataType : TDataType read GetDataType;// write SetDataType;
    property SetCellForCheckBox : Boolean read fSetCellForCheckBox write fSetCellForCheckBox;
  end;

  {/** Helper class that formats strings,integers,floats and Pictures onto a given
       canvas in a given area. */}

  TPGCellFormatter = class
  protected
    procedure setCell( value : TPGCell); virtual; abstract;
    function getCell : TPGCell; virtual; abstract;
    function getIsEmpty : Boolean; virtual; abstract;
  public
    {/** Renders the formatted value onto the Canvas into Rect */}
    procedure RenderValue( Canvas : TCanvas; Rect : TRect ); virtual; abstract;
    function ContentWidth( Canvas : TCanvas ) : Integer; virtual; abstract;
    function ContentHeight( Canvas : TCanvas ) : Integer; virtual; abstract;
    property Cell : TPGCell read getCell write setCell;
    property IsEmpty : Boolean read getIsEmpty;
  end;

  TPGCellContentParser = class
  public
    function ParseAsText( Value : string; ParsedContent : TCellContent ) : Boolean; virtual; abstract;
    function ParseAsDecimal( Value : string; ParsedContent : TCellContent ) : Boolean; virtual; abstract;
    function ParseAsCurrency( Value : string; ParsedContent : TCellContent ) : Boolean; virtual; abstract;
    function ParseAsInteger( Value : string; ParsedContent : TCellContent ) : Boolean; virtual; abstract;
    function ParseAsPicture( Value : string; ParsedContent : TCellContent ) : Boolean; virtual; abstract;
  end;

  TColorOperation = ( coINVALID, coNone, coSet, coAND, coOR, coXOR, coInvert );

  const tsNoFontName : TFontName = '';
        tsNoFontStyle : TFontStyles = [];
        tsNoFontSize : Integer = 0;
        tsNoFontPitch : TFontPitch = fpDefault;

type

  // Derived from TPersistent to allow editing in object inspector
  TTextStyle = class (TPersistent )
  private
    fFontName : TFontName;
    fFontStyle : TFontStyles;
    fFontSize : Integer;
    fFontPitch : TFontPitch;
    fColor : TColor;
    fBgColor : TColor;
    fColorOperation : TColorOperation;
    fBgColorOperation : TColorOperation;
    fOnChange : TNotifyEvent;

    function getFontName : TFontName;
    procedure setFontName( value : TFontName );
    function getFontStyle : TFontStyles;
    procedure setFontStyle( value: TFontStyles);
    function getFontSize : Integer;
    procedure setFontSize( value: Integer);
    function getFontPitch : TFontPitch;
    procedure setFontPitch( value: TFontPitch);
    function getColor : TColor;
    procedure setColor( value: TColor);
    function getBgColor : TColor;
    procedure setBgColor( value: TColor);
    function getColorOperation : TColorOperation;
    procedure setColorOperation( value: TColorOperation);
    function getBgColorOperation : TColorOperation;
    procedure setBgColorOperation( value: TColorOperation);
  public
    constructor Create;
    destructor Destroy;override;
    procedure Apply( Font : TFont; var fg : TColor; var bg : TColor );
    property OnChange : TNotifyEvent read fOnChange write fOnChange;
  published
    property FontName : TFontName read getFontName write setFontName;
    property FontStyle : TFontStyles read getFontStyle write setFontStyle;
    property FontSize : Integer read getFontSize write setFontSize;
    property FontPitch : TFontPitch read getFontPitch write setFontPitch;
    property Color : TColor read getColor write setColor;
    property BgColor : TColor read getBgColor write setBgColor;
    property ColorOperation : TColorOperation read getColorOperation write setColorOperation;
    property BgColorOperation : TColorOperation read getBgColorOperation write setBgColorOperation;
  end;

{$endif WEBAPPLICATION}
implementation
{$ifndef WEBAPPLICATION}

{$IFDEF DEBUG}
  uses EditCell;
{$ENDIF DEBUG}

{-- TCellContent --}

procedure TCellContent.Assign( value : TCellContent );
begin
  fContentType := value.fContentType;
  Assert( fContentType in [cctText,cctInteger,cctDecimal,cctCurrency,cctPicture],
          'TCellContent.Assign:: Unknown content type!');
  case fContentType of
  cctText: fText := value.fText;
  cctInteger: fInteger := value.fInteger;
  cctDecimal: fDecimal := value.fDecimal;
  cctCurrency: fCurrency := value.fCurrency;
  cctPicture: fPicture := value.fPicture;
  end;
end;

procedure TCellContent.setAsText( value : String );
begin
  fContentType := cctText;
  fText := value;
end;

procedure TCellContent.setAsInteger( value : Integer );
begin
  fContentType := cctInteger;
  fInteger := value;
end;

procedure TCellContent.setAsDecimal( value : Double  );
begin
  fContentType := cctDecimal;
  fDecimal := value;
end;

procedure TCellContent.setAsCurrency( value : Currency  );
begin
  fContentType := cctCurrency;
  fCurrency := value;
end;

procedure TCellContent.setAsPicture( value : TPicture  );
begin
  fContentType := cctPicture;
  fPicture := value;
end;

function TCellContent.getAsText: string;
begin
  if ContentType <> cctText then
  begin
    Result := '';
    raise Exception.Create('TCellContent.getAsText: Content is not text');
  end
  else
    Result := fText;
end;

function TCellContent.getAsDecimal : Double;
begin
  if ContentType <> cctDecimal then
    raise Exception.Create('TCellContent.getAsDecimal: Content is not a decimal number')
  else
    Result := fDecimal;
end;

function TCellContent.getAsCurrency : Currency;
begin
  if ContentType <> cctCurrency then
    raise Exception.Create('TCellContent.getAsCurrency: Content is not a decimal number')
  else
    Result := fCurrency;
end;

function TCellContent.getAsInteger : Integer;
begin
  if ContentType <> cctInteger then
    raise Exception.Create('TCellContent.getAsInteger: Content is not a integer')
  else
    Result := fInteger;
end;

function TCellContent.getAsPicture : TPicture;
begin
  if ContentType <> cctPicture then
    raise Exception.Create('TCellContent.getAsPicture Content is not a Picture')
  else
    Result := fPicture;
end;


{--- TPGCell --- }

constructor TPGCell.Create;
begin
  inherited Create;
  Valid := True;
  SetCellForCheckBox := False;
end;

procedure TPGCell.ResetAll;
begin
  FCol := -1;
  FRow := -1;
  fReadOnly := True;
  fCalculated := False;
  fChecked := False;
  fSelected := False;
  fFillEmpty := False;
  fItemIndex := -1;
  fValid := True;
  fFixed := False;
end;

function TPGCell.getEditorText : string;
var
//  value : TCellContent;
  value : TValue;
begin
//  value := TCellContent.Create;
  GetValue( value );
  Result := AsString(value);
(*
  case value.ContentType of
    cctEmpty: Result := '';
    cctText: Result := value.AsText;
    cctInteger: Result := IntToStr( value.AsInteger );
    cctDecimal: Result := FloatToStrF( value.AsDecimal, ffGeneral, 15, 15 );//FloatToStr( value.AsDecimal ); { May need additional formatting to avoid scietific notation }
    cctCurrency: Result := CurrToStrF( value.AsCurrency, ffGeneral, 15 );//FloatToStr( value.AsDecimal ); { May need additional formatting to avoid scietific notation }
    cctPicture: Result := '<<UNEDITABLE, content-type: Picture>>'
  end;
  *)
end;

{procedure TPGCell.setDataType( AType : TDataType );
begin
  FDataType := AType;
end;
}
function TPGCell.getDataType : TDataType;
var
  AValue : TValue;
begin
  GetValue(AValue);
  Result := AValue.DataType;
end;

function TPGCell.getCellType : TCellType;
begin
  Result := fCellType;
end;

procedure TPGCell.setCellType( value : TCellType );
begin
  fCellType := Value;
end;

function TPGCell.getCellAlign : TCellAlign;
begin
  Result := fCellAlign;
end;

procedure TPGCell.setCellAlign( value : TCellAlign );
begin
  fCellAlign := value;
end;

function TPGCell.getReadOnly : Boolean;
begin
  Result := fReadOnly;
end;

procedure TPGCell.setReadOnly( value : Boolean );
begin
  fReadOnly := value;
end;

function TPGCell.getValid : Boolean;
begin
  Result := fValid;
end;

procedure TPGCell.setValid( value : Boolean );
begin
  fValid := value;
end;

function TPGCell.getFixed : Boolean;
begin
  Result := fFixed;
end;

procedure TPGCell.setFixed( value : Boolean );
begin
  fFixed := value;
end;

function TPGCell.getCalculated: Boolean;
begin
  Result := fCalculated;
end;

procedure TPGCell.setCalculated( value : Boolean );
begin
  fCalculated := Value;
end;

function TPGCell.getChecked : Boolean;
begin
  Result := fChecked;
end;

procedure TPGCell.setChecked( value : Boolean );
begin
  fChecked := value;
end;

function TPGCell.getSelected : Boolean;
begin
  Result := fSelected;
end;

procedure TPGCell.setSelected( value : Boolean );
begin
  fSelected := Value;
end;

function TPGCell.getBorderStyle : TBorderStyle;
begin
  Result := fBorderStyle;
end;

procedure TPGCell.setBorderStyle( value : TBorderStyle );
begin
  fBorderStyle := Value;
end;

function TPGCell.getFillEmpty : Boolean;
begin
  Result := fFillEmpty;
end;

procedure TPGCell.setFillEmpty( value : Boolean );
begin
  fFillEmpty := value;
end;

function TPGCell.getItemIndex : Integer;
begin
  Result := fItemIndex;
end;

procedure TPGCell.setItemIndex( value : Integer );
begin
  fItemIndex := value;
end;

function TPGCell.GetIcon(Sender : TObject; APicture : TPicture) : Boolean;
begin
  Result := False;
end;

function TPGCell.GetIconAlign(Sender : TObject; DefaultIconAlign : TCellAlign) : TCellAlign;
begin
  Result := DefaultIconAlign;
end;

procedure TPGCell.EditButtonClick(var Value : TValue);
{$IFDEF DEBUG}
var
  dlg : TFormEditCell;
  EditString : String;
{$ENDIF DEBUG}
begin
{$IFDEF DEBUG}
  dlg := TFormEditCell.Create(nil);
  EditString := AsString(Value);
  dlg.Run(EditString);
  Value := ValueFromString(EditString);
  dlg.free;
{$ENDIF DEBUG}
end;

function TPGCell.GetEditorFace(Sender : TObject; APicture : TPicture) : Boolean;
begin
  Result := False;
end;

// plain puckoness of delphi, need to define this crap for interfaces
{$ifdef VER130}
function TPGCell.QueryInterface(const IID: TGUID; out Obj): HResult; stdcall;
{$else}
function TPGCell.QueryInterface(const IID: TGUID; out Obj): Integer; stdcall;
const
  E_NOINTERFACE = $80004002;
{$endif VER130}
begin
  if GetInterface(IID, Obj) then Result := 0 else Result := E_NOINTERFACE;
end;

function TPGCell._AddRef: Integer; stdcall;
begin
  result := 1;
end;

function TPGCell._Release: Integer; stdcall;
begin
  result := 1;
end;

{-- TTextStyle --}

constructor TTextStyle.Create;
begin
  FontName := tsNoFontName;
  FontStyle := tsNoFontStyle;
  FontSize := tsNoFontSize;
  FontPitch := tsNoFontPitch;
  Color := clBlack;
  BgColor := clWhite;
  ColorOperation := coNone;
  bgColorOperation := coNone;
end;

destructor TTextStyle.Destroy;
begin

end;

procedure TTextStyle.Apply( Font : TFont;var fg : TColor;var bg : TColor );
  function ApplyColorOperation( base, color : TColor; operation : TColorOperation ) : TColor;
  begin
    Assert( operation in [coNone..coInvert],
        'TTextStyle.ApplyStyle.ApplyColorOperation:Invalid operation code');
    Result := TColor(0);
    case operation of
    coNone: Result := base;
    coSet:  Result := color;
    coAND:  Result := base and color;
    coOR:   Result := base or color;
    coXOR:  Result := base xor color;
    coInvert:Result:= not base;
    end;
  end;
begin
  if FontName <> tsNoFontName then
    Font.Name := FontName;
  if FontStyle <> tsNoFontStyle then
    Font.Style := Font.Style + FontStyle;
  if FontSize <> tsNoFontSize then
    Font.Size := FontSize;
  if FontPitch <> tsNoFontPitch then
    Font.Pitch := FontPitch;
  fg := ApplyColorOperation( fg, Color, ColorOperation );
  bg := ApplyColorOperation( bg, BgColor, bgColorOperation );
end;

  {--property get set. Boring code...zzzz....}

function TTextStyle.getFontName : TFontName;
begin
  Result := fFontName;
end;

procedure TTextStyle.setFontName( value : TFontName );
begin
  if value <> fFontName then
  begin
    fFontName := value;
    if Assigned( OnChange ) then OnChange( self );
  end;
end;

function TTextStyle.getFontStyle : TFontStyles;
begin
  Result := fFontStyle;
end;

procedure TTextStyle.setFontStyle( value: TFontStyles);
begin
  if value <> fFontStyle then
  begin
    fFontStyle := value;
    if Assigned( OnChange ) then OnChange( self );
  end;
end;

function TTextStyle.getFontSize : Integer;
begin
  Result := fFontSize;
end;

procedure TTextStyle.setFontSize( value: Integer);
begin
  if value <> fFontSize then
  begin
    fFontSize := value;
    if Assigned( OnChange ) then OnChange( self );
  end;
end;

function TTextStyle.getFontPitch : TFontPitch;
begin
  Result := fFontPitch;
end;

procedure TTextStyle.setFontPitch( value: TFontPitch);
begin
  if value <> fFontPitch then
  begin
    fFontPitch := value;
    if Assigned( OnChange ) then OnChange( self );
  end;
end;

function TTextStyle.getColor : TColor;
begin
  Result := fColor;
end;

procedure TTextStyle.setColor( value: TColor);
begin
  if value <> fColor then
  begin
    fColor := value;
    if Assigned( OnChange ) then OnChange( self );
  end;
end;

function TTextStyle.getBgColor : TColor;
begin
  Result := fBgColor;
end;

procedure TTextStyle.setBgColor( value: TColor);
begin
  if value <> fBgColor then
  begin
    fBgColor := value;
    if Assigned( OnChange ) then OnChange( self );
  end;
end;

function TTextStyle.getColorOperation : TColorOperation;
begin
  Result := fColorOperation;
end;

procedure TTextStyle.setColorOperation( value: TColorOperation);
begin
  if value <> fColorOperation then
  begin
    fColorOperation := value;
    if Assigned( OnChange ) then OnChange( self );
  end;
end;

function TTextStyle.getBgColorOperation : TColorOperation;
begin
  Result := fBgColorOperation;
end;

procedure TTextStyle.setBgColorOperation( value: TColorOperation);
begin
  if value <> fBgColorOperation then
  begin
    fBgColorOperation := value;
    if Assigned( OnChange ) then OnChange( self );
  end;
end;

{$endif WEBAPPLICATION}
end.
