unit RegUtils;

interface

uses SysUtils, Windows;

{ Utility routines }

function RegHash(const KeyName: String): Integer;
// Calculates hash function for key names

function GetLevel(const Path: String): Integer;
// Depth of current path (number of back-slashes)

function ExtractLastKey (const Path: String): String;
function ExtractFirstKey(const Path: String): String;
// Get first key level and last key level names

{ Helpers to show data as a string }

function BinaryToStr(const Buffer: array of Byte; const BufferSize: Integer):String;
function BinaryToBytes32(const Buffer: array of Byte; const BufferSize: Integer): String;
function DwordToStr(const Buffer: Int64): String;

function DecodeSharePassword(const Hash: array of Byte; const Length: Byte): String;
// Decodes XOR cipher used to store shared resources' passwords

function Max(const A,B: Integer): Integer;
{ Substitution for Math unit }

implementation

function RegHash(const KeyName: String): Integer;
var
  I,C,H: Integer;
begin
  H := 0;
  for I := 1 to Length(KeyName) do
  begin
    C := Ord(UpCase(KeyName[I]));
    if  C < 128 then Inc(H,C);
  end;
  Result := H;
end;

function GetLevel(const Path: String): Integer;
var
  I,J: Integer;
begin
  J := 0;
  for I := 1 to Length(Path) do
   if Path[I] = '\' then inc(J);
  Result := J;
end;

function ExtractLastKey(const Path: String): String;
var
  I,J: Integer;
  S  : String;
begin
  S := Path;
  J := Length(Path);
  I := J;
  if J = 0 then Exit;
  while (S[I] <> '\') and (I > 1) do Dec(I);
  if S = '\' then Result := ''
  else Result := Copy(S,I+1,J-I);
end;

function ExtractFirstKey(const Path: String): String;
var
  I: Integer;
  S: String;
begin
  S := Path;
  Result := '';
  if S <> '' then
  begin
    Delete(S,1,1);
    I := Pos('\',S);
    if I = 0 then Result := S
    else Result := Copy(S,1,I - 1);
  end;
end;

function BinaryToStr(const Buffer: array of Byte; const BufferSize: Integer): String;
var
  S: String;
  I: Integer;
begin
  for I := 0 to (BufferSize - 1) do
     S := S + ' ' + IntToHex(Buffer[I],2);
  Result := S;
end;

function BinaryToBytes32(const Buffer: array of Byte; const BufferSize: Integer): String;
var
  Len: Integer;
begin
  if BufferSize > 32 then Len := 32
  else Len := BufferSize;
  Result := BinaryToStr(Buffer,Len);
end;

function DwordToStr(const Buffer: Int64): String;
var
  S: String;
begin
  S := IntToHex(Integer(Buffer),8);
  Result := '0x' + S +' ('+IntToStr(Buffer)+')';
end;



function DecodeSharePassword(const Hash: array of Byte; const Length: Byte): String;
const
  MS_PASSWORD: array [0..7] of Byte = (53,154,77,166,83,169,212,106);
var
  I: Integer;
  P: String;
  C: Byte;
begin
  for I := 0 to Length - 1 do
  begin
    C := Hash[I] xor MS_PASSWORD[I];
    P := P + Chr(C);
  end;
  Result := P;
  OemToChar(PChar(Result),PChar(Result));
end;

function Max(const A,B: Integer): Integer;
begin
  if A > B then Result := A
  else Result := B;
end;

end.