{****************************************************************************
 * Purpose    :Control and retrive info from a Winamp application           *
 * Start Date :2001-08-12                                                   *
 * Last Mod   :2001-09-07                                                   *
 * Authors    : Harry Zachrisson (harry@sverige.net)                        *
 * Lengthy description:                                                     *
 * This unit contains functions that can control an open WinAmp application.*
 * It can command it to play, pause, play next track etc                    *
 * It also contains support functions for retreiving MP3 tag info.          *
 * from a file.                                                             *
 *                                                                          *
 ****************************************************************************}

unit WinAmpControl;

interface

Type
  TID3Rec = packed record
    Tag     : array[0..2] of Char;
    Title   : array[0..29] of Char;
    Artist  : array[0..29] of Char;
    Album   : array[0..29] of Char;
    Comment : array[0..29] of Char;
    Year    : array[0..3] of Char;
    Genre   : Byte;
  End;

  Const
    WA_ISSTOPED=0;
    WA_ISPLAYING=1;
    WA_ISPAUSED=3;
    WA_SEQUENCE=0;
    WA_RANDOM=1;
    ID3_GENRE_NODATA=126;

    Function  WinAmpDir():String;  //Directory were winamp.exe is installed
    Function  GetID3Info (mp3File:string; VAR ID3Out:TID3Rec):Boolean;  //return a ID3 Record
    Function  WA_IsRunning():Boolean; //True if winamp is running
    Function  WA_Version():Integer;   //return Winamps software version
    Procedure WA_Visible(Visible:Boolean);
    Function  WA_IsVisible:Boolean;
    Procedure WA_Close;
    Function  WA_TrackLength():Integer; //Returns current tracklength in seconds (no decimals, Winamp have to be in play or pause mode)
    Function  WA_TrackPos():Integer;
    Procedure WA_SetTrackPos(TrackPos:Integer);
    Procedure WA_WritePlayList();
    Function  WA_PlayState ():Integer;
    Function  WA_CurrentTrackNo ():Integer;
    Function  WA_NoOfTracksLoaded ():Integer;
    Function  WA_GetTrackFileName (TrackNo:Integer; VAR TrackFileName:String):Boolean;
    Procedure WA_PlayTrack (TrackNo:Integer);
    Procedure WA_Stop();
    Procedure WA_Pause();
    Procedure WA_Play();
    Procedure WA_NextTrack();
    Procedure WA_PrevTrack();
    Procedure WA_SeekBack();
    Procedure WA_SeekForward();
    Procedure WA_SetPlaySequence(PlaySequence:Integer);
    Function  WA_PlaySequence():Integer;
    Function  WA_NextArtist ():Boolean;

implementation
Uses
  Windows,Messages, SysUtils, shellAPI, Classes,Registry;




{ DONE 5 -oHarry Zachrisson : Write it to retrive the value from the registry or ini file }
//Returns the directory were winamp resides
Function  WinAmpDir():String;
var
  Reg: TRegistry;
  Path :String;
begin
  Reg := TRegistry.Create;
  Result :='';
  try
    Reg.RootKey := HKEY_LOCAL_MACHINE;
     if Reg.OpenKey('\Software\Microsoft\Windows\CurrentVersion\Uninstall\Winamp', False)
     Then Begin
       Result:=Reg.ReadString('TestFile');
       Path:=Reg.ReadString('UninstallString');
       Path:=Uppercase(Path);
       If Path[1]='"' then Delete (Path,1,1);  // delete leading " caracter
       while Path[Length(Path)]<>'\' do Delete (Path,Length(Path),1); //delete trailing characters until we find the last backspace
       Result :=Path;
     End;
  finally
    Reg.CloseKey;
    Reg.Free;
  end;
end;



//Returns true if Winamp ver 1.xx to 2.xx is running
Function  WA_IsRunning():Boolean;
 var hwndWinamp : THandle;
begin
  hwndWinamp:=FindWindow('Winamp v1.x', nil);
  Result:=Boolean(SendMessage(hwndWinamp,WM_USER,0,0));
End ;


//Retrives ID3tag info from an MP3 file. Returns false if file did not contain a tag
Function GetID3Info (mp3File:string; VAR ID3Out:TID3Rec):Boolean;
  Procedure ClearID3Data (VAR ID3Data:TID3Rec);
  Begin
    ID3Data.Tag:='NO';
    ID3Data.Title:='';
    ID3Data.Artist:='';
    ID3Data.Comment:='';
    ID3Data.Album:='';
    ID3Data.Year:='';
    ID3Data.Genre:=ID3_GENRE_NODATA;
  End;
var
  ID3FromFile : TID3Rec;
  fmp3: TFileStream;
begin
  ClearID3Data (ID3Out);
  Result:=False; //preset to "did not found any ID3 tag in file"
  If FileExists (mp3File) Then
    Begin
    fmp3:=TFileStream.Create(mp3File, fmShareDenyNone);
    try
      fmp3.position:=fmp3.size-128;
      fmp3.Read(ID3FromFile,SizeOf(ID3FromFile));
    finally
      fmp3.free;
    end;

    if ID3FromFile.Tag = 'TAG' then //ID3 tag was found in file
      begin
      ID3Out:=ID3FromFile;
      IF Id3Out.Genre>ID3_GENRE_NODATA then Id3Out.Genre:=ID3_GENRE_NODATA; //Supress all undefined genres
      Result:=True;
    end; // if ID3FromFile.Tag

  end;// If FileExists
end;

//Returns software version of running WinAmp
Function  WA_Version():Integer;
 var hwndWinamp : THandle;
begin
  hwndWinamp:=FindWindow('Winamp v1.x', nil);
  Result:=Integer(SendMessage(hwndWinamp,WM_USER,0,0));
End ;

//Hides or Shows Winamp window
procedure WA_Visible(Visible:Boolean);
var
  hwndWinamp : THandle;
  ShowMode :Integer;
begin
  hwndWinamp := FindWindow('Winamp v1.x', nil);
  if Visible then ShowMode:=SW_SHOW Else ShowMode:=SW_HIDE;
  ShowWindowAsync(hwndWinamp,ShowMode);
end;

//Returns true if Winamp window is visible
Function  WA_IsVisible():Boolean;
 var hwndWinamp : THandle;
begin
  hwndWinamp:=FindWindow('Winamp v1.x', nil);
  Result:=IsWindowVisible(hwndWinamp);
End ;


//Command WinAmp to close down
Procedure WA_Close();
var
  hwndWinamp : THandle;
begin
  hwndWinamp := FindWindow('Winamp v1.x', nil);
  PostMessage(hwndWinamp,WM_Command,40001,0)
End;

//Returns Track length in seconds
Function  WA_TrackLength():Integer;
 var hwndWinamp : THandle;
begin
  hwndWinamp:=FindWindow('Winamp v1.x', nil);
  Result:=Integer(SendMessage(hwndWinamp,WM_USER,1,105));
End ;

//Returns playposition in Track in milliseconds, returns -1 if not playing
Function  WA_TrackPos():Integer;
 var hwndWinamp : THandle;
begin
  hwndWinamp:=FindWindow('Winamp v1.x', nil);
  Result:=Integer(SendMessage(hwndWinamp,WM_USER,0,105));
End ;

//Play at spcified position (in milliseconds)
Procedure WA_SetTrackPos(TrackPos:Integer);
 var hwndWinamp : THandle;
begin
  hwndWinamp := FindWindow('Winamp v1.x', nil);
  SendMessage(hwndWinamp,WM_USER,TrackPos,106);
End ;

//Command Winamp to create a file called winamp.m3u from its internal playlist.
//This file will be used by the function WA_GetTrackFileName
Procedure WA_WritePlayList();
 var hwndWinamp : THandle;
begin
  hwndWinamp := FindWindow('Winamp v1.x', nil);
  SendMessage(hwndWinamp,WM_USER,0,120);
End ;

//Return Winamps Playstate. Can be one of the three constants WA_IsStoped, WA_IsPlaying, WA_IsPaused
Function WA_PlayState ():Integer;
 var hwndWinamp : THandle;
begin
  hwndWinamp := FindWindow('Winamp v1.x', nil);
  Result:=Integer(SendMessage(hwndWinamp,WM_USER,0,104));
End ;

//Returns the current (playing) track in WinAmp
Function WA_CurrentTrackNo ():Integer;
 var hwndWinamp : THandle;
begin
  hwndWinamp := FindWindow('Winamp v1.x', nil);
  Result:=Integer(SendMessage(hwndWinamp,WM_USER,0,125))+1;
End ;

//Number of tracks in Winamps internal playlist
Function WA_NoOfTracksLoaded ():Integer;
 var hwndWinamp : THandle;
begin
  hwndWinamp := FindWindow('Winamp v1.x', nil);
  Result:=Integer(SendMessage(hwndWinamp,WM_USER,0,124));
End ;

//Retrive the filename and path of a specific track
//This function depends on an external file generated by Winamp.
//Call Procedure "WA_WritePlayList" to generate the file
Function WA_GetTrackFileName (TrackNo:Integer; VAR TrackFileName:String):Boolean;
var
  PlayList: TextFile;
  Line: string;
  LineCounter:Integer;
begin
Result:=False; //preset to "did not found the requested track info"
 If NOT FileExists (WinAmpDir+'winamp.m3u') then WA_WritePlayList();
{$I-}
  AssignFile(PlayList,WinAmpDir+'winamp.m3u');  //load playlistfile
  Reset(PlayList);
{$I+}
  if IOResult = 0 then
    Begin
    LineCounter:=0;
    While ((LineCounter<TrackNo) AND (NOT SeekEof (PlayList))) Do //parse file until the correct line is found or to end of file
      Begin
      Readln(PlayList,Line);     //Read one line of data from the file
      If Not ((Line[1])=('#')) Then  //do not count lines with WinAmp internal data
        Begin
        Inc(LineCounter);  // inc line conter = track conter in Winamp. eg. track 4 is on line 4
      End; //If
    End;//While
    CloseFile(PlayList);
    If LineCounter=TrackNo then   // if found the correct line
      Begin
      TrackFileName:=Line;  //copy the info from the file
      Result:=True;         //data was found
    End;
  End;//IOResult
End ;



//Command WinAmp to stop playing
Procedure WA_Stop();
var
  hwndWinamp : THandle;
begin
 hwndWinamp := FindWindow('Winamp v1.x', nil);
 PostMessage(hwndWinamp,WM_Command,40047,0)
End;


//Command WinAmp to start playing if stoped or paused
Procedure WA_Play();
  var hwndWinamp : THandle;
begin
  If WA_NoOfTracksLoaded >0 then //Do not trigger play on empty playlist (this will bring up a file dialog in Winamp)
    Begin
    hwndWinamp := FindWindow('Winamp v1.x', nil);
    Case WA_PlayState Of
      WA_IsStoped: PostMessage(hwndWinamp,WM_Command,40045,0); //Play if stoped
      WA_IsPaused: PostMessage(hwndWinamp,WM_Command,40046,0); //UnPause if stoped (continues on the same spot in the track)
    end; //Case
  end;//If  WA_NoOfTracksLoaded
End;


//Command WinAmp to play a specific track from its internal playlist
Procedure WA_PlayTrack (TrackNo:Integer);
 var hwndWinamp : THandle;
begin
  WA_Stop;
  hwndWinamp := FindWindow('Winamp v1.x', nil);
  SendMessage(hwndWinamp,WM_USER,TrackNo-1,121);
  WA_Play;
end;


//Command WinAmp to Pause
Procedure WA_Pause();
  var hwndWinamp : THandle;
begin
  If WA_PlayState=WA_IsPlaying Then // Only command WA to pause if it is playing otherwise we wold risk to unpause.
    Begin
    hwndWinamp := FindWindow('Winamp v1.x', nil);
    PostMessage(hwndWinamp,WM_Command,40046,0); //Pause
  End; //If
End;


//Command WinAmp to play previous track from its internal playlist
Procedure WA_PrevTrack();
var
  hwndWinamp : THandle;
begin
 hwndWinamp := FindWindow('Winamp v1.x', nil);
 PostMessage(hwndWinamp,WM_Command,40044,0)
End;


//Command WinAmp to next previous track from its internal playlist
Procedure WA_NextTrack();
var
  hwndWinamp : THandle;
begin
  hwndWinamp := FindWindow('Winamp v1.x', nil);
  PostMessage(hwndWinamp,WM_Command,40048,0)
End;


//Command WinAmp to skip back 5 seconds
procedure WA_SeekBack();
var
  hwndWinamp : THandle;
begin
  hwndWinamp := FindWindow('Winamp v1.x', nil);
  PostMessage(hwndWinamp,WM_Command,40144,0)
end;


//Command WinAmp to skip forward 5 seconds
procedure WA_SeekForward();
var
  hwndWinamp : THandle;
begin
  hwndWinamp := FindWindow('Winamp v1.x', nil);
  PostMessage(hwndWinamp,WM_Command,40148,0)
end;


//Set Winamps shuffle state. Set PlaySequence to either WA_RANDOM or WA_SEQUENCE
procedure WA_SetPlaySequence(PlaySequence:Integer);
 var hwndWinamp : THandle;
begin
  hwndWinamp := FindWindow('Winamp v1.x', nil);
  SendMessage(hwndWinamp,WM_USER,PlaySequence,252);
end;


//Number of tracks in Winamps internal playlist
Function WA_PlaySequence():Integer;
 var hwndWinamp : THandle;
begin
  hwndWinamp := FindWindow('Winamp v1.x', nil);
  Result:=Integer(SendMessage(hwndWinamp,WM_USER,0,250));
End ;

//Play the the next track that is not the same artist as the current track
Function WA_NextArtist ():Boolean;
Var
  CurrentArtist:String;
  CurrentTrackNo:Integer;
  PlayList: TextFile;
  Line: string;
  LineCounter:Integer;
  ID3DAta:TID3Rec;
Begin
  CurrentTrackNo:=WA_CurrentTrackNo;
  Result:=False; //preset to "did not found another artist before end of list"
  If CurrentTrackNo<WA_NoOfTracksLoaded Then //do nothing if current track is the last track
    Begin
    If NOT FileExists (WinAmpDir+'winamp.m3u') then WA_WritePlayList(); //if not exported a playlistfile then force it now
    {$I-}
    AssignFile(PlayList,WinAmpDir+'winamp.m3u');  //load playlistfile
    Reset(PlayList);
    {$I+}
    if IOResult = 0 then //If file could be opened
      Begin
      LineCounter:=0;
      While ((LineCounter<CurrentTrackNo) AND (NOT SeekEof (PlayList))) Do //parse file until the correct line is found or to end of file
        Begin
        Readln(PlayList,Line);     //Read one line of data from the file
        If Not ((Line[1])=('#')) Then  //do not count lines with WinAmp internal data
          Begin
          Inc(LineCounter);  // inc line conter = track conter in Winamp. eg. track 4 is on line 4
        End; //If
      End;//While

      If LineCounter=CurrentTrackNo then   // if found the current track
        Begin
        GetID3Info (Line,ID3Data); //Retrive the Artist from the current track
        CurrentArtist:=ID3Data.Artist; //Store current artist
        repeat
          Readln(PlayList,Line);     //Read the next line of data from the file
          If Not ((Line[1])=('#')) Then  //do not count lines with WinAmp internal data
            Begin
            Inc(LineCounter);  // inc line conter
            GetID3Info (Line,ID3Data) //Retrive the Artist data from this track
          End; //If
        until (ID3Data.Artist<>CurrentArtist) or SeekEof (PlayList); //Until found a new artist or end of file

        IF ID3Data.Artist<>CurrentArtist then //If found a track with a different artist
          Begin
          Result:=True;//succes
          WA_PlayTrack(LineCounter); //Play the track with the different artist.
        End;// IF ID3Data.Artist<>CurrentArtist

      End;//If LineCounter=CurrentTrackNo
      CloseFile(PlayList);
    End;//IOResult
  End; // If CurrentTrackNo<WA_NoOfTracksLoaded
End;// WA_NextArtist

end.
