{====================================================================
	   / .
	     .
---------------------------------------------------------------------
	5 oktober 2001.
	Copyleft.
	Written by Artem Khomenko.
====================================================================}

unit hsStrUtils;

interface

uses Classes;

type
	eArgType = (atValue, atReference);


	//   .
	procedure SkipWhites(var buf : PChar);

	//      .
	procedure SkipLine(var buf : PChar);

	//  ,  .
	Procedure SkipComments(var buf : PChar);

	//   - ,   _.
	function ExtractBufWord(var buf : PChar) : string;
	function ExtractLexem(var buf : PChar) : string;

	//       ,    .
	//  .
	function ExtractQuoteLexem(var buf : PChar; includeQuoteMark : boolean = true) : string;
	function ExtractQuoteLexemStr(str : string; includeQuoteMark : boolean = true) : string;

	//        .  ..
	//      .
	Procedure CollectWordsToList(list : TStrings; const str : string);

	// . ,    .
	function isWhites(ch : char) : boolean;
	// . ,    .
	function isAlphas(ch : char) : boolean;

	// ,  ,   .
	Function RemoveQuotes(const str : string) : string;

	//      .
	Procedure StrToLines(const longStr: string; AList : TStrings);

	//    name=value   .
	//  includeMacroSign ,   
	//  value   <%...%>.  ,  .
	//       ,   - .
	Procedure StringAsParams(list : TStringList; const str : string; includeMacroSign : boolean = false);
	Function ParamsAsString(const list : TStringList) : string;

	//   FormatDateTime .
	Function FormatStrToDateTime(Format : string; strDate : string) : TDateTime;

implementation

uses SysUtils, Windows;

//
//   .
//
procedure SkipWhites(var buf : PChar);
begin
	while (buf^ <> #0) and IsWhites(buf^) do
		inc(buf);
end;

//
//      .
//
procedure SkipLine(var buf : PChar);
begin
	while (buf^ <> #0) and (buf^ <> #10) do
		inc(buf);

	//     .
	if buf^ = #10 then
		inc(buf);
end;

//
//  ,  .
//
Procedure SkipComments(var buf : PChar);
var
	comments	: integer;
	changes	: boolean;
begin
	repeat
		changes	:= false;
		//  .
		SkipWhites(buf);

		//    ,   .
		if (buf^ = '/') and ((buf + 1)^ = '/') then begin
			SkipLine(buf);
			SkipWhites(buf);
			changes	:= true;
		end;

		//    ,  .
		//
		if (buf^ = '/') and ((buf + 1)^ = '*') then begin
			comments	:= 1;
			while buf^ <> #0 do begin
				inc(buf);

				//     , 
				// .
				if (buf^ = '/') and ((buf + 1)^ = '*') then begin inc(comments); inc(buf, 2); end
				else if (buf^ = '*') and ((buf + 1)^ = '/') then begin dec(comments); inc(buf, 2); end;
				if comments = 0 then break;
			end;
			changes	:= true;
		end;
	until changes = false;
end;

//
//       ,    .
//  .
//
function ExtractQuoteLexem(var buf : PChar; includeQuoteMark : boolean = true) : string;
begin
	SkipWhites(buf);

	if buf^ = '"' then begin

		if includeQuoteMark then Result	:= '"'
		else Result	:= '';

		inc(buf);
		while (buf^ <> #0) and (buf^ <> '"') do begin Result	:= Result + buf^; inc(buf); end;
		inc(buf);

		if includeQuoteMark then Result	:= Result + '"';

	end else
		Result	:= ExtractBufWord(buf);
end;

function ExtractQuoteLexemStr(str : string; includeQuoteMark : boolean = true) : string;
var buf	: PChar;
begin
	buf	:= PChar(str);
	Result	:= ExtractQuoteLexem(buf, includeQuoteMark);
end;


//
// . ,    .
//
function IsWhites(ch : char) : boolean;
begin
	Result	:= ch in [' ', #0, #9, #10, #13];
end;

// . ,    .
function isAlphas(ch : char) : boolean;
begin
	Result	:= IsCharAlphaNumeric(ch) or (ch in ['_', '\', '.']);
end;


//
//   - ,   _.
//
function ExtractBufWord(var buf : PChar) : string;
begin
	Result	:= '';
	while IsAlphas(buf^) do begin
		Result	:= Result + buf^;
		inc(buf);
	end;
end;

function ExtractLexem(var buf : PChar) : string;
begin
	Result	:= '';
	while not IsWhites(buf^) do begin
		Result	:= Result + buf^;
		inc(buf);
	end;
end;


//
//        .  ..
//      .
//
Procedure CollectWordsToList(list : TStrings; const str : string);
var
	buf	: PChar;
	tmp	: string;
begin
	buf	:= PChar(str);

	while buf^ <> #0 do begin
		SkipComments(buf);

		if buf^ = ',' then begin
			inc(buf);
			SkipComments(buf);
		end;

		tmp	:= ExtractBufWord(buf);
		if tmp <> '' then list.Append(tmp);
	end;
end;

//
// ,  ,   .
//
Function RemoveQuotes(const str : string) : string;
var len : integer;
begin
	len	:= Length(str);
	if (len > 0) and (str[1] = '"') and (str[len] = '"') then
		Result	:= Copy(str, 2, len - 2)
	else
		Result	:= str;
end;


//
//      .
//
Procedure StrToLines(const longStr: string; AList : TStrings);
var
	p	: PChar;
	s  : string;
begin
	p	:= PChar(longStr);
	s	:= '';

	while P^ <> #0 do begin
		//   -   .
		if p^ = #127 then begin
			if s <> '' then begin
				AList.Append(s);
				s	:= '';
			end;

			//  .
			inc(p);
		end else begin
			//    else    
			//    .

			//  .
			s	:= s + p^;
			inc(p);
		end;
	end;

	//   .
	if s <> '' then AList.Append(s);
end;


//
//    name=value   .
//  includeMacroSign ,   
//  value   <%...%>.  ,  .
//
Procedure StringAsParams(list : TStringList; const str : string; includeMacroSign : boolean = false);
label l_save, l_next;
var
	pstr	: PChar;
	rName, rValue	: string;
	paramType	: eArgType;
	startWithMacro	: boolean;
begin
	list.Clear;
	pstr	:= PChar(str);

	//    ,  .
	if (Length(str) > 0) and (str[1] <> #127) then goto l_next;

	while (pstr^ = #127) and ((pstr + 1)^ <> #0) do begin
		inc(pstr);

l_next:

		rName	:= '';		rValue	:= '';	paramType	:= atValue;

		//      '='.
		//
		while (pstr^ <> '=') and (pstr^ <> #0) do begin
			rName	:= rName + pstr^;
			Inc(pstr);
		end;
		if pstr^ = #0 then goto l_save;

		//   '='.
		Inc(pstr);

		// ,        .
		if (pstr^ = '<') and ((pstr + 1)^ = '%') then begin
			paramType	:= atReference;
			startWithMacro	:= true;

			if not includeMacroSign then Inc(pstr, 2);
		end else begin
			paramType		:= atValue;
			startWithMacro	:= false;
		end;

		//        .
		//
		while (pstr^ <> #127) and (pstr^ <> #0) do begin
			//   '%>' ,   
			//      .
			if startWithMacro and not includeMacroSign and (pstr^ = '%') and ((pstr + 1)^ = '>') then
				Inc(pstr)
			else
				rValue	:= rValue + pstr^;
			Inc(pstr);
		end;

l_save:
		//    .
		//
		list.AddObject(rName, TObject(paramType));
		list.Append(rValue);
	end;
end;


Function ParamsAsString(const list : TStringList) : string;
var	i, c : integer;
begin
	Result	:= '';

	i	:= 0;
	c	:= list.Count - 2;
	while i <= c do begin
		Result	:= Result + #127 + list[i] + '=' + list[i + 1];
		inc(i, 2);
	end;
end;


//
//   FormatDateTime .
//
Function FormatStrToDateTime(Format : string; strDate : string) : TDateTime;
// throws EConvertError()

type
	TTokenType = (fmtUndef, fmtC, fmtD, fmtD2, fmtD3, fmtD4, fmtD5, fmtD6,
						fmtM, fmtM2, fmtM3, fmtM4, fmtY2, fmtY4, fmtH, fmtH2,
						fmtN, fmtN2, fmtS, fmtS2, fmtZ, fmtZ3, fmtT, fmtT2,
						fmtAM_PM, fmtA_P, fmtAMPM, fmtDateSep, fmtTimeSep, fmtUserStr
					);
var
	//        .
	pFormat, pDate	: PChar;

	//   .
	formatToken	: string;

	//   .
	date : TSystemTime;

	isPilotAnal	: boolean;


	//
	//     .
	//
	Function GetFormatToken	: TTokenType;
	begin
		if pFormat^ = #0 then
			Result	:= fmtUndef
		else if Pos(pFormat^, #9#10#13' .,`~!@#$%^&*()_-=+\|]}[{?;') > 0 then begin
			Result	:= fmtUserStr;
			formatToken	:= pFormat^;
			inc(pFormat);
		end else if pFormat^ = 'C' then	begin
			Result	:= fmtC;
			formatToken	:= 'C';
			inc(pFormat);
		end else if (pFormat^ = 'D') and ((pFormat + 1)^ <> 'D') then begin
			Result	:= fmtD;
			formatToken	:= 'D';
			inc(pFormat);
		end else if (pFormat^ = 'D') and ((pFormat + 1)^ = 'D') and ((pFormat + 2)^ <> 'D') then begin
			Result	:= fmtD2;
			formatToken	:= 'DD';
			inc(pFormat, 2);
		end else if (pFormat^ = 'D') and ((pFormat + 1)^ = 'D') and ((pFormat + 2)^ = 'D') and ((pFormat + 3)^ <> 'D') then begin
			Result	:= fmtD3;
			formatToken	:= 'DDD';
			inc(pFormat, 3);
		end else if (pFormat^ = 'D') and ((pFormat + 1)^ = 'D') and ((pFormat + 2)^ = 'D') and ((pFormat + 3)^ = 'D') and ((pFormat + 4)^ <> 'D') then begin
			Result	:= fmtD4;
			formatToken	:= 'DDDD';
			inc(pFormat, 4);
		end else if (pFormat^ = 'D') and ((pFormat + 1)^ = 'D') and ((pFormat + 2)^ = 'D') and ((pFormat + 3)^ = 'D') and ((pFormat + 4)^ = 'D') and ((pFormat + 5)^ <> 'D') then begin
			Result	:= fmtD5;
			formatToken	:= 'DDDDD';
			inc(pFormat, 5);
		end else if (pFormat^ = 'D') and ((pFormat + 1)^ = 'D') and ((pFormat + 2)^ = 'D') and ((pFormat + 3)^ = 'D') and ((pFormat + 4)^ = 'D') and ((pFormat + 5)^ = 'D') then begin
			Result	:= fmtD6;
			formatToken	:= 'DDDDDD';
			inc(pFormat, 6);
		end else if (pFormat^ = 'M') and ((pFormat + 1)^ <> 'M') then begin
			Result	:= fmtM;
			formatToken	:= 'M';
			inc(pFormat);
		end else if (pFormat^ = 'M') and ((pFormat + 1)^ = 'M') and ((pFormat + 2)^ <> 'M') then begin
			Result	:= fmtM2;
			formatToken	:= 'MM';
			inc(pFormat, 2);
		end else if (pFormat^ = 'M') and ((pFormat + 1)^ = 'M') and ((pFormat + 2)^ = 'M') and ((pFormat + 3)^ <> 'M') then begin
			Result	:= fmtM3;
			formatToken	:= 'MMM';
			inc(pFormat, 3);
		end else if (pFormat^ = 'M') and ((pFormat + 1)^ = 'M') and ((pFormat + 2)^ = 'M') and ((pFormat + 3)^ = 'M') and ((pFormat + 4)^ <> 'M') then begin
			Result	:= fmtM4;
			formatToken	:= 'MMMM';
			inc(pFormat, 4);
		end else if (pFormat^ = 'Y') and ((pFormat + 1)^ = 'Y') and ((pFormat + 2)^ <> 'Y') then begin
			Result	:= fmtY2;
			formatToken	:= 'YY';
			inc(pFormat, 2);
		end else if (pFormat^ = 'Y') and ((pFormat + 1)^ = 'Y') and ((pFormat + 2)^ = 'Y') and ((pFormat + 3)^ = 'Y') then begin
			Result	:= fmtY4;
			formatToken	:= 'YYYY';
			inc(pFormat, 4);
		end else if (pFormat^ = 'H') and ((pFormat + 1)^ <> 'H') then begin
			Result	:= fmtH;
			formatToken	:= 'H';
			inc(pFormat);
		end else if (pFormat^ = 'H') and ((pFormat + 1)^ = 'H') then begin
			Result	:= fmtH2;
			formatToken	:= 'HH';
			inc(pFormat, 2);
		end else if (pFormat^ = 'N') and ((pFormat + 1)^ <> 'N') then begin
			Result	:= fmtN;
			formatToken	:= 'N';
			inc(pFormat);
		end else if (pFormat^ = 'N') and ((pFormat + 1)^ = 'N') then begin
			Result	:= fmtN2;
			formatToken	:= 'NN';
			inc(pFormat, 2);
		end else if (pFormat^ = 'S') and ((pFormat + 1)^ <> 'S') then begin
			Result	:= fmtS;
			formatToken	:= 'S';
			inc(pFormat);
		end else if (pFormat^ = 'S') and ((pFormat + 1)^ = 'S') then begin
			Result	:= fmtS2;
			formatToken	:= 'SS';
			inc(pFormat, 2);
		end else if (pFormat^ = 'Z') and ((pFormat + 1)^ <> 'Z') then begin
			Result	:= fmtZ;
			formatToken	:= 'Z';
			inc(pFormat);
		end else if (pFormat^ = 'Z') and ((pFormat + 1)^ = 'Z') and ((pFormat + 2)^ = 'Z') then begin
			Result	:= fmtZ3;
			formatToken	:= 'ZZZ';
			inc(pFormat, 3);
		end else if (pFormat^ = 'T') and ((pFormat + 1)^ <> 'T') then begin
			Result	:= fmtT;
			formatToken	:= 'T';
			inc(pFormat);
		end else if (pFormat^ = 'T') and ((pFormat + 1)^ = 'T') then begin
			Result	:= fmtT2;
			formatToken	:= 'TT';
			inc(pFormat, 2);
		end else if (pFormat^ = 'A') and ((pFormat + 1)^ = 'M') and ((pFormat + 2)^ = '/') and ((pFormat + 3)^ = 'P') and ((pFormat + 4)^ = 'M') then begin
			Result	:= fmtAM_PM;
			formatToken	:= 'AM/PM';
			inc(pFormat, 5);
		end else if (pFormat^ = 'A') and ((pFormat + 1)^ = '/') and ((pFormat + 2)^ = 'P') then begin
			Result	:= fmtA_P;
			formatToken	:= 'A/P';
			inc(pFormat, 3);
		end else if (pFormat^ = 'A') and ((pFormat + 1)^ = 'M') and ((pFormat + 2)^ = 'P') and ((pFormat + 3)^ = 'M') then begin
			Result	:= fmtAMPM;
			formatToken	:= 'AMPM';
			inc(pFormat, 4);
		end else if (pFormat^ = '''') or (pFormat^ = '"') then begin
			Result	:= fmtUserStr;
			formatToken	:= ExtractQuoteLexem(pFormat, isPilotAnal);
		end else if (pFormat^ = '/') then begin
			Result	:= fmtDateSep;
			formatToken	:= '/';
			inc(pFormat);
		end else if (pFormat^ = ':') then begin
			Result	:= fmtTimeSep;
			formatToken	:= ':';
			inc(pFormat);
		end else
			Result	:= fmtUndef;
	end;

	Procedure ReportError;
	begin
		raise EConvertError.Create('On format: ' + pFormat + ' string: ' + pDate);
	end;

	//
	//	      ,  .
	//
	Function ScanNumber(numLen : integer) : integer;
	var
		str 	: string;
		i		: integer;
	begin
		str	:= '';
		for i := 0 to numLen - 1 do
			if (pDate[i] >= '0') and (pDate[i] <= '9') then
				str	:= str + pDate[i]
			else
				break;
		inc(pDate, i);
		Result	:= StrToInt(str);
	end;

	//
	//  ,      
	//  .
	//
	Function ScanStr(template : string) : boolean;
	begin
		Result	:= AnsiSameText(Copy(pDate, 1, Length(template)), template);
		if Result then
			inc(pDate, Length(template));
	end;

	//
	//  Sun-Sat,  1 - 7
	//
	Function ScanArray(strArray : array of string) : integer;
	begin
		for Result := 0 to High(strArray) do
			if ScanStr(strArray[Result]) then
				break;
		if Result > High(strArray) then
			ReportError
		else
			inc(Result);
	end;

begin
	DateTimeToSystemTime(0, date);

	if Format = '' then pFormat	:= 'C'
	else pFormat	:= PChar(AnsiUpperCase(Format));
	pDate		:= PChar(AnsiUpperCase(strDate));

	isPilotAnal	:= true;

	//
	//     ,
	//   fmtC, fmtD5, fmtD6, fmtT, fmtT2.
	//
	Format	:= '';
	while true do begin
		case GetFormatToken of
			fmtUndef	: break;
			fmtC     : Format	:= Format + ShortDateFormat + ' ' + LongTimeFormat;
			fmtD5    : Format	:= Format + ShortDateFormat;
			fmtD6    : Format	:= Format + LongDateFormat;
			fmtT     : Format	:= Format + ShortTimeFormat;
			fmtT2    : Format	:= Format + LongTimeFormat;
		else
			Format	:= Format + formatToken;
		end;
	end;

	pFormat	:= PChar(AnsiUpperCase(Format));
	isPilotAnal	:= false;

	//
	//  .
	//
	repeat
		//  .
		case GetFormatToken of
			fmtUndef	: ReportError;
			fmtD,
			fmtD2    : date.wDay	:= ScanNumber(2);
			fmtD3    : date.wDayOfWeek	:= ScanArray(ShortDayNames);
			fmtD4    : date.wDayOfWeek	:= ScanArray(LongDayNames);
			fmtM,
			fmtM2    : date.wMonth	:= ScanNumber(2);
			fmtM3    : date.wMonth	:= ScanArray(ShortMonthNames);
			fmtM4    : date.wMonth	:= ScanArray(LongMonthNames);
			fmtY2    : begin date.wYear	:= ScanNumber(2); if date.wYear >= 50  then inc(date.wYear, 1900) else inc(date.wYear, 2000) end;
			fmtY4    : date.wYear	:= ScanNumber(4);
			fmtH,
			fmtH2    : date.wHour	:= ScanNumber(2);
			fmtN,
			fmtN2    : date.wMinute	:= ScanNumber(2);
			fmtS,
			fmtS2    : date.wSecond	:= ScanNumber(2);
			fmtZ,
			fmtZ3    : date.wMilliseconds	:= ScanNumber(3);
			fmtAM_PM : begin if ScanStr('PM') then inc(date.wHour, 12) else if not ScanStr('AM') then ReportError; end;
			fmtA_P   : begin if ScanStr('P') then inc(date.wHour, 12) else if not ScanStr('A') then ReportError; end;
			fmtAMPM  : begin if ScanStr(TimePMString) then inc(date.wHour, 12) else if not ScanStr(TimeAMString) then ReportError; end;
			fmtDateSep	: if not ScanStr(DateSeparator) then ReportError;
			fmtTimeSep  : if not ScanStr(TimeSeparator) then ReportError;
			fmtUserStr  : if not ScanStr(formatToken) then ReportError;
		end;

	until (pFormat^ = #0) or (pDate^ = #0);

   Result	:= SystemTimeToDateTime(date);
end;



end.
