{
 -----------------------------------------------------------------------------

 MODUL          mdCTRLS.PAS

 COPYRIGHT      micro dynamics GmbH
 AUTHOR(S)      Michael Frank
 UPDATE         11.01.2002

 COMPILER       Delphi 5
 PLATFORM       Windows 95/98/Me/NT4/2000/XP

 DESCRIPTION    These components support Windows XP Visual Styles, if
                available.
                They are usable as direct replacements of the original
                Borland Delphi aequivalents.

 -----------------------------------------------------------------------------
 Revision History:   Original Version  12.01
               .
 11.01.2002    Version 1.10
 29.12.2001    Version 1.00
 -----------------------------------------------------------------------------
}
unit mdCTRLS;

{$DEFINE DEF_LANGUAGE}         // disable for english version

//============================================================================

interface

//============================================================================

uses
  Windows, Messages, Controls, ImgList, SysUtils, Classes, Forms,
  CheckLst, ComCtrls, StdCtrls, ExtCtrls, Graphics, DsgnIntf,

  mdTHEMES;

const
  tagDISABLED = $01000000;

type

{ TXPThemeAPI }

  TXPThemeAPI = class(TComponent)
  public
    constructor Create(AOwner: TComponent); override;
  end;

{ TXPCheckListBox }

  TXPCheckListBox = class(TCheckListBox)
  private
    function GetItemEnabled (Index : integer) : boolean;
    function GetState (Index : integer) : TCheckBoxState;
    procedure CMEnabledChanged (var Message : TMessage); message CM_ENABLEDCHANGED;
    procedure CNDrawItem (var Message : TWMDrawItem); message CN_DRAWITEM;
    procedure DrawItemCheck (Rect : TRect; State : TCheckBoxState;
                             Enabled : boolean);
    procedure DrawItemText (Index : integer; Rect : TRect;
                             State : TOwnerDrawState);
  protected
    procedure DrawItem (Index : integer; Rect : TRect;
                        State : TOwnerDrawState); override;
    procedure KeyPress (var Key : Char); override;
    procedure MouseDown (Button : TMouseButton; Shift : TShiftState;
                         X, Y : integer); override;
    procedure ToggleClickCheck (Index : integer);
  end;

  TTransparentPaint = class(TObject)
  private
    fControl : TWinControl;
    fOldBitmap : THandle;
    fTempBitmap : THandle;
    fTempDC : THandle;
    fTransparent : boolean;
    procedure DeleteTempDC;
    procedure InternalPaint;
    procedure SetTransparent (Value : boolean);
  protected
    property Transparent : boolean read fTransparent write SetTransparent default false;
  public
    constructor Create (AControl : TWinControl);
    destructor Destroy; override;
  end;

{ TXPGroupBox }

  TXPGroupBox = class(TGroupBox)
  private
    fObject : TTransparentPaint;
    fTransparent : boolean;
    procedure SetTransparent (Value : boolean);
    procedure CMEnabledChanged (var Message : TMessage); message CM_ENABLEDCHANGED;
  protected
    procedure WMEraseBkgnd (var Message : TWMEraseBkgnd); message WM_ERASEBKGND;
    procedure WMMove (var Message : TMessage); message WM_MOVE;
    procedure WMPaint (var Message : TWMPaint); message WM_PAINT;
    procedure WMSize (var Message : TMessage); message WM_SIZE;
  public
    constructor Create (AOwner : TComponent); override;
    destructor Destroy; override;
    procedure Paint; override;
  published
    property Transparent : boolean read fTransparent write SetTransparent;
  end;

{ TXPCheckBox }

  TXPCheckBox = class(TCheckBox)
  private
    fObject : TTransparentPaint;
    fTransparent : boolean;
    procedure SetTransparent (Value : boolean);
    procedure CMEnabledChanged (var Message : TMessage); message CM_ENABLEDCHANGED;
  protected
    procedure WMEraseBkgnd (var Message : TWMEraseBkgnd); message WM_ERASEBKGND;
    procedure WMMove (var Message : TMessage); message WM_MOVE;
    procedure WMPaint (var Message : TWMPaint); message WM_PAINT;
    procedure WMSize (var Message : TMessage); message WM_SIZE;
  public
    constructor Create (AOwner : TComponent); override;
    destructor Destroy; override;
  published
    property Transparent : boolean read fTransparent write SetTransparent;
  end;

{ TXPRadioButton }

  TXPRadioButton = class(TRadioButton)
  private
    fObject : TTransparentPaint;
    fTransparent : boolean;
    procedure SetTransparent (Value : boolean);
    procedure CMEnabledChanged (var Message : TMessage); message CM_ENABLEDCHANGED;
  protected
    procedure WMEraseBkgnd (var Message : TWMEraseBkgnd); message WM_ERASEBKGND;
    procedure WMMove (var Message : TMessage); message WM_MOVE;
    procedure WMPaint (var Message : TWMPaint); message WM_PAINT;
    procedure WMSize (var Message : TMessage); message WM_SIZE;
  public
    constructor Create (AOwner : TComponent); override;
    destructor Destroy; override;
  published
    property Transparent : boolean read fTransparent write SetTransparent;
  end;

{ TXPTrackBar }

  TXPTrackBar = class(TTrackBar)
  private
    fObject : TTransparentPaint;
    fTransparent : boolean;
    procedure SetTransparent (Value : boolean);
    procedure CMEnabledChanged (var Message : TMessage); message CM_ENABLEDCHANGED;
  protected
    procedure WMEraseBkgnd (var Message : TWMEraseBkgnd); message WM_ERASEBKGND;
    procedure WMMove (var Message : TMessage); message WM_MOVE;
    procedure WMPaint (var Message : TWMPaint); message WM_PAINT;
    procedure WMSize (var Message : TMessage); message WM_SIZE;
  public
    constructor Create (AOwner : TComponent); override;
    destructor Destroy; override;
  published
    property Transparent : boolean read fTransparent write SetTransparent;
  end;

{ TXPPageControl }

  TXPPageControl = class(TPageControl);

  TXPTabSheet = class(TTabSheet)
  private
    procedure WMEraseBkgnd (var Message : TWMEraseBkgnd); message WM_ERASEBKGND;
  protected
    procedure CreateParams (var Params : TCreateParams); override;
  end;

  TXPPageControlEditor = class(TDefaultEditor)
  public
    procedure ExecuteVerb (Index : integer); override;
    function GetVerb (Index : integer): string; override;
    function GetVerbCount : integer; override;
  end;

procedure Register;

//============================================================================

implementation

//============================================================================

var
  iDrawing : integer;
  TransparentClasses : TList;

//----------------------------------------------------------------------------
procedure Register;
//----------------------------------------------------------------------------

begin
  RegisterComponents('XPControls', [TXPThemeAPI, TXPCheckListBox, TXPPageControl,
      TXPGroupBox, TXPCheckBox, TXPRadioButton, TXPTrackBar]);

  RegisterComponentEditor(TXPPageControl, TXPPageControlEditor);
  RegisterComponentEditor(TXPTabSheet, TXPPageControlEditor);
  RegisterNoIcon([TXPTabSheet]);
  RegisterClasses([TXPTabSheet]);
end;

//============================================================================
// class TXPThemeAPI
//============================================================================

//----------------------------------------------------------------------------
constructor TXPThemeAPI.Create (AOwner : TComponent);
//----------------------------------------------------------------------------

var
  i, Instances : integer;

begin
  inherited;

  if not (AOwner is TForm) then
    raise Exception.Create('TXPThemeAPI: Control parent must be a form!');

  i:= 0;
  Instances:= 0;
  while (i < AOwner.ComponentCount) and (Instances < 2) do
  begin
    if (AOwner.Components[i] is TXPThemeAPI) then
      inc(Instances,1);
    inc(i,1);
  end;

  if (Instances > 1) then
    raise Exception.Create('TXPThemeAPI: Drop one instance only on form!');
end;

//============================================================================
// class TXPTabSheet
//============================================================================

//----------------------------------------------------------------------------
procedure TXPTabSheet.CreateParams (var Params : TCreateParams);
//----------------------------------------------------------------------------

begin
  inherited CreateParams(Params);
  Params.Style:= Params.Style and not WS_CLIPCHILDREN;
end;

//----------------------------------------------------------------------------
procedure TXPTabSheet.WMEraseBkgnd (var Message : TWMEraseBkgnd);
//----------------------------------------------------------------------------

var
  hhTheme : HTHEME;
  DrawRect : TRect;

begin
  hhTheme:= 0;

  if UseThemes and not (csDesigning in ComponentState) then
    hhTheme:= OpenThemeData(0,'Tab');

  if (hhTheme <> 0) then
  begin
    try
      DrawRect:= GetClientRect;
      DrawThemeBackground(hhTheme, Message.DC, TABP_BODY, 0, @DrawRect, nil);
    finally
      CloseThemeData(hhTheme);
      Message.Result:= 1;
    end;
  end else
    inherited;
end;

//============================================================================
// class TXPCheckListBox
//============================================================================

//----------------------------------------------------------------------------
function TXPCheckListBox.GetState (Index : integer) : TCheckBoxState;
//----------------------------------------------------------------------------

begin
  result:= State[Index];
end;

//----------------------------------------------------------------------------
function TXPCheckListBox.GetItemEnabled (Index : integer) : boolean;
//----------------------------------------------------------------------------

begin
  result:= ItemEnabled[Index];
end;

//----------------------------------------------------------------------------
procedure TXPCheckListBox.CMEnabledChanged (var Message : TMessage);
//----------------------------------------------------------------------------

begin
  if not Enabled then
    ItemIndex:= -1;
  inherited;
end;

//----------------------------------------------------------------------------
procedure TXPCheckListBox.CNDrawItem (var Message : TWMDrawItem);
//----------------------------------------------------------------------------

begin
  with Message.DrawItemStruct^ do
  begin
    if UseRightToLeftAlignment then
      dec(rcItem.Right, 1)
    else
      inc(rcItem.Left, 1);
  end;
  inherited;
end;

//----------------------------------------------------------------------------
procedure TXPCheckListBox.DrawItem (Index : integer; Rect : TRect;
                                    State : TOwnerDrawState);
//----------------------------------------------------------------------------

var
  fEnable : boolean;

begin
  if (Index < Items.Count) then
  begin
    { Force lbStandard list to ignore OnDrawItem event. }
    if (Style <> lbStandard) and Assigned(OnDrawItem) then
    begin
      OnDrawItem(Self, Index, Rect, State);
    end else
    begin
      fEnable:= Self.Enabled and GetItemEnabled(Index);
      DrawItemCheck(Rect, GetState(Index), fEnable);
      if not fEnable then
        Canvas.Font.Color:= clGrayText;
      DrawItemText(Index, Rect, State);
    end;
  end;
end;

//----------------------------------------------------------------------------
procedure TXPCheckListBox.DrawItemText (Index : integer; Rect : TRect;
                                        State : TOwnerDrawState);
//----------------------------------------------------------------------------

var
  Flags : Dword;

begin
  Canvas.FillRect(Rect);
  Flags:= DrawTextBiDiModeFlags(DT_SINGLELINE or DT_VCENTER or DT_NOPREFIX);
  if not UseRightToLeftAlignment then
    inc(Rect.Left, 2)
  else
    dec(Rect.Right, 2);
  DrawText(Canvas.Handle, PChar(Items[Index]), Length(Items[Index]), Rect, Flags);
end;

//----------------------------------------------------------------------------
procedure TXPCheckListBox.DrawItemCheck (Rect : TRect; State : TCheckBoxState;
                                         Enabled : boolean);
//----------------------------------------------------------------------------

var
  hhTheme : HTHEME;
  iCheckWidth : integer;
  iDrawState : integer;

begin
  hhTheme:= 0;

  iCheckWidth:= GetCheckWidth;

  if UseRightToLeftAlignment then
  begin
    Rect.Left:= Rect.Right;
    Rect.Right:= Rect.Left + iCheckWidth;
  end else
  begin
    Rect.Right:= Rect.Left;
    Rect.Left:= Rect.Right - iCheckWidth;
  end;

  dec(iCheckWidth, 2);
  Rect.Left := Rect.Left + (Rect.Right - Rect.Left - iCheckWidth) div 2;
  Rect.Top := Rect.Top + (Rect.Bottom - Rect.Top - iCheckWidth) div 2;
  Rect.Right:= Rect.Left + iCheckWidth;
  Rect.Bottom:= Rect.Top + iCheckWidth;

  with Canvas do
  begin
    if UseThemes and not (csDesigning in ComponentState) then
      hhTheme:= OpenThemeData(0,'Button');

    if (hhTheme <> 0) then
    begin
      case State of
        cbChecked :
          if Enabled then
            iDrawState:= CBS_CHECKEDNORMAL
          else
            iDrawState:= CBS_CHECKEDDISABLED;

        cbUnchecked :
          if Enabled then
            iDrawState:= CBS_UNCHECKEDNORMAL
          else
            iDrawState:= CBS_UNCHECKEDDISABLED;

        else // cbGrayed
          if Enabled then
            iDrawState:= CBS_MIXEDNORMAL
          else
            iDrawState:= CBS_MIXEDDISABLED;
      end;

      try
        DrawThemeBackground(hhTheme, Handle, BP_CHECKBOX, iDrawState, @Rect, nil);
      finally
        CloseThemeData(hhTheme);
      end;
    end else

    begin
      case State of
        cbChecked :
          iDrawState:= DFCS_BUTTONCHECK or DFCS_CHECKED;

        cbUnchecked :
          iDrawState:= DFCS_BUTTONCHECK;

        else // cbGrayed
          iDrawState:= DFCS_BUTTON3STATE or DFCS_CHECKED;
      end;

      if not Enabled then
        iDrawState:= iDrawState or DFCS_INACTIVE;

      if Flat then
        iDrawState:= iDrawState or DFCS_FLAT;

      DrawFrameControl(Handle, Rect, DFC_BUTTON, iDrawState);
    end;
  end;
end;

//----------------------------------------------------------------------------
procedure TXPCheckListBox.KeyPress (var Key : Char);
//----------------------------------------------------------------------------

begin
  if (Key = ' ') then
    ToggleClickCheck(ItemIndex)
  else
    inherited;
end;

//----------------------------------------------------------------------------
procedure TXPCheckListBox.MouseDown (Button : TMouseButton; Shift : TShiftState;
                                     X, Y : integer);
//----------------------------------------------------------------------------

var
  Index : integer;

begin
  inherited;
  if (Button = mbLeft) then
  begin
    Index:= ItemAtPos(Point(X,Y), true);
    if (Index <> -1) and GetItemEnabled(Index) then
    begin
      if UseRightToLeftAlignment then
      begin
        Dec(X, ItemRect(Index).Right - GetCheckWidth);
        if not ((X > 0) and (X < GetCheckWidth)) then
          ToggleClickCheck(Index);
      end else
      begin
        if not (X - ItemRect(Index).Left < GetCheckWidth) then
          ToggleClickCheck(Index);
      end;
    end;
  end;
end;

//----------------------------------------------------------------------------
procedure TXPCheckListBox.ToggleClickCheck;
//----------------------------------------------------------------------------

var
  State : TCheckBoxState;

begin
  if (Index >= 0) and (Index < Items.Count) and GetItemEnabled(Index) then
  begin
    State:= Self.State[Index];
    case State of
      cbUnchecked :
        if AllowGrayed then
          State:= cbGrayed
        else
          State:= cbChecked;

      cbChecked :
        State:= cbUnchecked;

      cbGrayed :
        State:= cbChecked;
    end;
    Self.State[Index]:= State;
    ClickCheck;
  end;
end;

//============================================================================
// class TTransparentPaint
//============================================================================

//----------------------------------------------------------------------------
function GetParentDC (ParentControl : TWinControl) : THandle;
//----------------------------------------------------------------------------

begin
  if (ParentControl is TXPGroupBox) then
    result:= TXPGroupBox(ParentControl).fObject.fTempDC
  else
    result:= 0;
end;

//----------------------------------------------------------------------------
function IsTransparentControl (AControl : TWinControl) : boolean;
//----------------------------------------------------------------------------

var
  i : integer;

begin
  i:= 0;
  result:= false;
  while (i < TransparentClasses.Count) and not result do
  begin
    if (AControl is TWinControlClass(TransparentClasses.Items[i])) then
      result:= true;
    inc(i,1);
  end;
end;

//----------------------------------------------------------------------------
procedure SaveBackground (AControl : TWinControl;
                          var hTempDC, hTempBitmap, hOldBitmap : THandle);
//----------------------------------------------------------------------------

var
  DC : THandle;
  ParentDC : THandle;
  FormDC : THandle;
  FormBitmap : THandle;
  OldFormBitmap : THandle;

begin
  inc(iDrawing,1);
  with AControl do
  begin
   if (Parent <> nil) and
      (not (Parent is TForm) or not (csLoading in ComponentState)) then
    begin
      DC:= GetDC(Handle);
      ParentDC := GetDC(Parent.Handle);
      FormDC:= CreateCompatibleDC(ParentDC);
      FormBitmap:= CreateCompatibleBitmap(ParentDC, Parent.Width, Parent.Height);
      OldFormBitmap:= SelectObject(FormDC, FormBitmap);
      if (hTempDC = 0) then
      begin
        hTempDC:= CreateCompatibleDC(DC);
        hTempBitmap:= CreateCompatibleBitmap(DC, Width, Height);
        hOldBitmap:= SelectObject(hTempDC, hTempBitmap);
      end;

      IntersectClipRect(FormDC, Left, Top, Left + Width, Top + Height);
      Parent.PaintTo(FormDC, 0, 0);
      BitBlt(hTempDC, 0, 0, Width, Height, FormDC, Left, Top, SRCCOPY);
      SelectObject(FormDC, OldFormBitmap);

      DeleteObject(FormBitmap);
      DeleteDC(FormDC);
      ReleaseDC(Parent.Handle, ParentDC);
      ReleaseDC(Handle, DC);
    end
  end;
  dec(iDrawing,1);
end;

//----------------------------------------------------------------------------
function TransparentStretchBlt (DestDC : HDC; DestX, DestY, DestW, DestH : integer;
                                SourceDC : HDC; SourceX, SourceY, SourceW, SourceH : integer;
                                MaskDC : HDC; MaskX, MaskY : integer) : boolean;
//----------------------------------------------------------------------------

const
  ROP_DSTCOPY = $00AA0029;

var
  MemDC : THandle;
  MemBmp : THandle;
  Save : THandle;
  crText : TColorRef;
  crBack : TColorRef;

begin
  if (Win32Platform = VER_PLATFORM_WIN32_NT) and
     (SourceW = DestW) and
     (SourceH = DestH) then
  begin
    MemBmp:= CreateCompatibleBitmap(SourceDC, 1, 1);
    MemBmp:= SelectObject(MaskDC, MemBmp);

    MaskBlt(DestDC,
            DestX, DestY,
            DestW, DestH,
            SourceDC,
            SourceX, SourceY,
            MemBmp,
            MaskX, MaskY,
            MakeRop4(ROP_DSTCOPY, SRCCOPY));

    MemBmp:= SelectObject(MaskDC, MemBmp);
    DeleteObject(MemBmp);
  end else
  begin
    MemDC:= CreateCompatibleDC(0);
    MemBmp:= CreateCompatibleBitmap(SourceDC, SourceW, SourceH);
    Save:= SelectObject(MemDC, MemBmp);

    StretchBlt(MemDC, 0, 0, SourceW, SourceH, MaskDC, MaskX, MaskY, SourceW, SourceH, SRCCOPY);
    StretchBlt(MemDC, 0, 0, SourceW, SourceH, SourceDC, SourceX, SourceY, SourceW, SourceH, SRCERASE);

    crText:= SetTextColor(DestDC, $00000000);
    crBack:= SetBkColor(DestDC, $00FFFFFF);

    StretchBlt(DestDC, DestX, DestY, DestW, DestH, MaskDC, MaskX, MaskY, SourceW, SourceH, SRCAND);
    StretchBlt(DestDC, DestX, DestY, DestW, DestH, MemDC, 0, 0, SourceW, SourceH, SRCINVERT);

    SetTextColor(DestDC, crText);
    SetTextColor(DestDC, crBack);
    SelectObject(MemDC, Save);
    DeleteObject(MemBmp);
    DeleteDC(MemDC);
  end;
  result:= true;
end;

//----------------------------------------------------------------------------
procedure TransparentBitBlt (hSourceDC, hDestDC : THandle; ARect : TRect;
                             AOrigin : TPoint; ATransparentColor : Dword);
//----------------------------------------------------------------------------

var
  MonoBitmap : THandle;
  MonoDC : THandle;
  OldBitmap : THandle;
  OldBkColor : integer;
  Width : integer;
  Height : integer;

begin
  Width:= ARect.Right - ARect.Left;
  Height:= ARect.Bottom - ARect.Top;
  MonoDC:= CreateCompatibleDC(hSourceDC);
  MonoBitmap:= CreateCompatibleBitmap(MonoDC, Width, Height);
  OldBitmap:= SelectObject(MonoDC, MonoBitmap);
  try
    OldBkColor:= SetBkColor(hSourceDC, ATransparentColor);
    BitBlt(MonoDC, 0, 0, Width, Height, hSourceDC, 0, 0, SRCCOPY);
    SetBkColor(hSourceDC, OldBkColor);
    TransparentStretchBlt(hDestDC, AOrigin.x, AOrigin.y, Width, Height,
                          hSourceDC, ARect.Left, ARect.Top, Width, Height, MonoDC, 0, 0);
  finally
    SelectObject(MonoDC, OldBitmap);
    DeleteDC(MonoDC);
    DeleteObject(MonoBitmap);
  end;
end;

//----------------------------------------------------------------------------
procedure TransparentPaintXY (AControl : TWinControl; hBackDC : THandle;
                              var fTransparent : boolean; X, Y : integer);
//----------------------------------------------------------------------------

var
  DC : THandle;
  MemDC : THandle;
  FormDC : THandle;
  OldMemObject : THandle;
  OldFormObject : THandle;
  Bitmap : THandle;
  FormBitmap : THandle;

begin
  with AControl do
  begin
    if (Parent <> nil) and
       ([csReading, csLoading] * ComponentState = []) and
       ([csReading, csLoading] * Parent.ComponentState = []) and
       HandleAllocated and
       Visible then
    begin
      DC:= GetDC(Handle);
      MemDC:= CreateCompatibleDC(DC);
      FormDC:= CreateCompatibleDC(DC);

      FormBitmap:= CreateCompatibleBitmap(DC, Width, Height);
      OldFormObject:= SelectObject(formdc, FormBitmap);
      Bitmap:= CreateCompatibleBitmap(DC, Width, Height);
      OldMemObject:= SelectObject(MemDC, Bitmap);
{ 1 }
      BitBlt(FormDC, 0, 0, Width, Height, hBackDC, X, Y, SRCCOPY);
      fTransparent:= false;
{ 2 }
      PaintTo(MemDC, 0, 0);
      fTransparent:= true;
{ 3 }
      TransparentBitBlt(MemDC, FormDC, Rect(0, 0, Width, Height),
                        Point(0, 0), GetPixel(MemDC, 0, 0));
{ 4 }
      BitBlt(DC, 0, 0, Width, Height, FormDC, 0, 0, SRCCOPY);

      SelectObject(FormDC, OldFormObject);
      DeleteObject(FormBitmap);
      SelectObject(MemDC, OldMemObject);
      DeleteObject(Bitmap);
      ReleaseDC(Handle, DC);
      DeleteDC(MemDC);
      DeleteDC(FormDC);
    end;
  end;
end;

//----------------------------------------------------------------------------
constructor TTransparentPaint.Create (AControl : TWinControl);
//----------------------------------------------------------------------------

begin
  inherited Create;
  fControl:= AControl;
  fTempDC:= 0;
  fTransparent:= false;
end;

//----------------------------------------------------------------------------
destructor TTransparentPaint.Destroy;
//----------------------------------------------------------------------------

begin
  DeleteTempDC;
  inherited;
end;

//----------------------------------------------------------------------------
procedure TTransparentPaint.DeleteTempDC;
//----------------------------------------------------------------------------

begin
  if (fTempDC <> 0) then
  begin
    SelectObject(fTempDC, fOldBitmap);
    DeleteObject(fTempBitmap);
    DeleteDC(fTempDC);
    fTempDC:= 0;
  end;
end;

//----------------------------------------------------------------------------
procedure TTransparentPaint.InternalPaint;
//----------------------------------------------------------------------------

var
  ParentControl : TWinControl;
  P : TPoint;

begin
  if fTransparent and not (iDrawing > 0) then
  begin
    ParentControl:= fControl.Parent;

    while (ParentControl <> nil) and
          IsTransparentControl(ParentControl.Parent) do
      ParentControl:= ParentControl.Parent;

    if (ParentControl <> nil) and IsTransparentControl(ParentControl) then
    begin
      P.X:= 0;
      P.Y:= 0;
      P:= fControl.ClientToScreen(P);
      P:= ParentControl.ScreenToClient(P);
      TransparentPaintXY(fControl, GetParentDC(ParentControl), fTransparent, P.X, P.Y);
    end else
    begin
      SaveBackground(fControl, fTempDC, fTempBitmap, fOldBitmap);
      TransparentPaintXY(fControl, fTempDC, fTransparent, 0, 0);
    end;
  end;
end;

//----------------------------------------------------------------------------
procedure TTransparentPaint.SetTransparent (Value : boolean);
//----------------------------------------------------------------------------

begin
  fTransparent:= Value;
end;

//============================================================================
// class TXPGroupBox
//============================================================================

//----------------------------------------------------------------------------
constructor TXPGroupBox.Create (AOwner : TComponent);
//----------------------------------------------------------------------------

begin
  inherited;
  fObject:= TTransparentPaint.Create(Self);
end;

//----------------------------------------------------------------------------
destructor TXPGroupBox.Destroy;
//----------------------------------------------------------------------------

begin
  fObject.Free;
  inherited;
end;

//----------------------------------------------------------------------------
procedure TXPGroupBox.CMEnabledChanged (var Message : TMessage);
//----------------------------------------------------------------------------

var
  i : integer;

begin
  inherited;
  if fObject.Transparent then
    fObject.InternalPaint;
  Invalidate;

  i:= 0;
  while (i < ControlCount) do
  begin
    if (Controls[i] is TWinControl) then
    begin
      if not Enabled then
      begin
        if TWinControl(Controls[i]).Enabled then
          TWinControl(Controls[i]).Tag:= TWinControl(Controls[i]).Tag or tagDISABLED;

        TWinControl(Controls[i]).Enabled:= false;
      end else
      begin
        if (TWinControl(Controls[i]).Tag and tagDISABLED = tagDISABLED) then
          TWinControl(Controls[i]).Enabled:= true;
        TWinControl(Controls[i]).Tag:= TWinControl(Controls[i]).Tag and not tagDISABLED;
      end;
    end;
    inc(i,1);
  end;
end;

//----------------------------------------------------------------------------
procedure TXPGroupBox.WMEraseBkgnd (var Message: TWMEraseBkgnd);
//----------------------------------------------------------------------------

begin
  if fObject.Transparent then
    Message.Result:= 1
  else
    inherited;
end;

//----------------------------------------------------------------------------
procedure TXPGroupBox.WMMove (var Message : TMessage);
//----------------------------------------------------------------------------

var
  i : integer;

begin
  inherited;
  if fObject.Transparent then
    fObject.InternalPaint;
  Invalidate;

  i:= 0;
  while (i < ControlCount) do
  begin
    if (Controls[i] is TWinControl) and
       (IsTransparentControl(TWinControl(Controls[i]))) then
      PostMessage(TWinControl(Controls[i]).Handle, WM_MOVE, 0, 0);
    inc(i, 1);
  end;
end;

//----------------------------------------------------------------------------
procedure TXPGroupBox.WMPaint (var Message : TWMPaint);
//----------------------------------------------------------------------------

var
  DrawRect : TRect;
  PaintStruct : TPaintStruct;

begin
  with fObject do
  begin
    if Transparent then
    begin
      if not (iDrawing > 0) then
      begin
        GetUpdateRect(fControl.Handle, DrawRect, false);
        if not IsRectEmpty(DrawRect) then
        begin
          BeginPaint(fControl.Handle, PaintStruct);
          InternalPaint;
          EndPaint(fControl.Handle, PaintStruct);
          Message.Result:= 0;
        end;
      end;
    end else
      inherited;
  end;
end;

//----------------------------------------------------------------------------
procedure TXPGroupBox.WMSize (var Message : TMessage);
//----------------------------------------------------------------------------

begin
  inherited;
  with fObject do
  begin
    if Transparent then
    begin
      DeleteTempDC;
      InternalPaint;
    end;
  end;
  Invalidate;
end;

//----------------------------------------------------------------------------
procedure TXPGroupBox.Paint;
//----------------------------------------------------------------------------

const
  CAPTIONOFFSET = 7;
  CAPTIONMARGIN = 1;

var
  hhTheme : HTHEME;
  H : integer;
  iDrawState, iMargin : integer;
  DrawRect, CaptionRect : TRect;
  DummyRect : TRect;
  Flags : Dword;
  s : WideString;

begin
  hhTheme:= 0;

  with Canvas do
  begin
    Font:= Self.Font;
    H:= TextHeight('0');
    DrawRect:= Rect(0, H div 2, Width, Height);
    Flags:= DrawTextBiDiModeFlags(DT_SINGLELINE);
    if (Text = '') then
      iMargin:= 0
    else
      iMargin:= CAPTIONMARGIN;

    if UseRightToLeftAlignment then
      CaptionRect:= Rect(DrawRect.Right - TextWidth(Text) - CAPTIONOFFSET, 0,
                         DrawRect.Right - CAPTIONOFFSET - 2*iMargin, H)
    else
      CaptionRect:= Rect(DrawRect.Left + CAPTIONOFFSET, 0,
                         DrawRect.Left + TextWidth(Text) + CAPTIONOFFSET + 2*iMargin, H);

    if UseThemes and not (csDesigning in ComponentState) then
      hhTheme:= OpenThemeData(0,'Button');

    if (hhTheme <> 0) then
    try
      if Enabled then
        iDrawState:= GBS_NORMAL
      else
        iDrawState:= GBS_DISABLED;

      s:= WideString(Text);
      GetThemeTextExtent(hhTheme, Handle, BP_GROUPBOX, iDrawState,
                         PWideChar(s), -1, Flags, nil, DummyRect);

      with CaptionRect do
        ExcludeClipRect(Handle, Left, Top, Right, Bottom);

      DrawThemeBackground(hhTheme, Handle, BP_GROUPBOX, iDrawState, @DrawRect, nil);
      SelectClipRgn(Handle, 0);

      DrawThemeText(hhTheme, Handle, BP_GROUPBOX, iDrawState,
                    PWideChar(s), -1, Flags, 0, @CaptionRect);
    finally
      CloseThemeData(hhTheme);
    end else

    begin
      with CaptionRect do
        ExcludeClipRect(Handle, Left, Top, Right, Bottom);
      if Ctl3D then
      begin
        inc(DrawRect.Left, 1);
        inc(DrawRect.Top, 1);
        Brush.Color:= clBtnHighlight;
        FrameRect(DrawRect);
        OffsetRect(DrawRect, -1, -1);
        Brush.Color:= clBtnShadow;
      end else
        Brush.Color:= clWindowFrame;
      FrameRect(DrawRect);
      SelectClipRgn(Handle, 0);

      if (Text <> '') then
      begin
        inc(CaptionRect.Left, iMargin);
        DrawText(Handle, PChar(Text), Length(Text), CaptionRect, Flags or DT_CALCRECT);
        SetBKMode(Handle, Windows.TRANSPARENT);

        if not Enabled then
        begin
          OffsetRect(CaptionRect, 1, 1);
          Font.Color:= clBtnHighlight;
          DrawText(Handle, PChar(Text), Length(Text), CaptionRect, Flags);
          OffsetRect(CaptionRect, -1, -1);
          Font.Color:= clBtnShadow;
        end;
        DrawText(Handle, PChar(Text), Length(Text), CaptionRect, Flags);
      end;
    end;
  end;
end;

//----------------------------------------------------------------------------
procedure TXPGroupBox.SetTransparent (Value : boolean);
//----------------------------------------------------------------------------

var
  i : integer;

begin
  fTransparent:= Value;

  Value:= Value or (UseThemes and not (Parent is TForm));
  if (Value <> fObject.Transparent) then
  begin
    fObject.Transparent:= Value;
    fObject.InternalPaint;
    Invalidate;
  end;

  i:= 0;
  while (i < ControlCount) do
  begin
    if (Controls[i] is TXPGroupBox) then
      TXPGroupBox(Controls[i]).Transparent:= Value
    else if (Controls[i] is TXPCheckBox) then
      TXPCheckBox(Controls[i]).Transparent:= Value
    else if (Controls[i] is TXPRadiobutton) then
      TXPRadioButton(Controls[i]).Transparent:= Value
    else if (Controls[i] is TXPTrackBar) then
      TXPTrackBar(Controls[i]).Transparent:= Value;
    inc(i,1);
  end;
end;

//============================================================================
// class TXPCheckBox
//============================================================================

//----------------------------------------------------------------------------
constructor TXPCheckBox.Create (AOwner : TComponent);
//----------------------------------------------------------------------------

begin
  inherited;
  fObject:= TTransparentPaint.Create(Self);
end;

//----------------------------------------------------------------------------
destructor TXPCheckBox.Destroy;
//----------------------------------------------------------------------------

begin
  fObject.Free;
  inherited;
end;

//----------------------------------------------------------------------------
procedure TXPCheckBox.CMEnabledChanged (var Message : TMessage);
//----------------------------------------------------------------------------

begin
  inherited;
  if fObject.Transparent then
    fObject.InternalPaint;
end;

//----------------------------------------------------------------------------
procedure TXPCheckBox.WMEraseBkgnd (var Message : TWMEraseBkgnd);
//----------------------------------------------------------------------------

begin
  if fObject.Transparent then
    Message.Result:= 1
  else
    inherited;
end;

//----------------------------------------------------------------------------
procedure TXPCheckBox.WMMove (var Message : TMessage);
//----------------------------------------------------------------------------

begin
  inherited;
  if fObject.Transparent then
    fObject.InternalPaint;
end;

//----------------------------------------------------------------------------
procedure TXPCheckBox.WMPaint (var Message : TWMPaint);
//----------------------------------------------------------------------------

var
  PaintStruct : TPaintStruct;
  DrawRect : TRect;

begin
  with fObject do
  begin
    if Transparent then
    begin
      if not (iDrawing > 0) then
      begin
        GetUpdateRect(fControl.Handle, DrawRect, false);
        if not IsRectEmpty(DrawRect) then
        begin
          BeginPaint(fControl.Handle, PaintStruct);
          InternalPaint;
          EndPaint(fControl.Handle, PaintStruct);
          Message.Result:= 0;
        end;
      end;
    end else
      inherited;
  end;
end;

//----------------------------------------------------------------------------
procedure TXPCheckBox.WMSize (var Message : TMessage);
//----------------------------------------------------------------------------

begin
  inherited;
  with fObject do
  begin
    if Transparent then
    begin
      DeleteTempDC;
      InternalPaint;
    end;
  end;
end;

//----------------------------------------------------------------------------
procedure TXPCheckBox.SetTransparent (Value : boolean);
//----------------------------------------------------------------------------

begin
  fTransparent:= Value;
  Value:= Value or
          (UseThemes and not ((Parent is TForm) or (Parent is TXPGroupBox)));
  if (Value <> fObject.Transparent) then
  begin
    fObject.Transparent:= Value;
    Invalidate;
  end;
end;

//============================================================================
// class TXPRadioButton
//============================================================================

//----------------------------------------------------------------------------
constructor TXPRadioButton.Create (AOwner : TComponent);
//----------------------------------------------------------------------------

begin
  inherited;
  fObject:= TTransparentPaint.Create(Self);
end;

//----------------------------------------------------------------------------
destructor TXPRadioButton.Destroy;
//----------------------------------------------------------------------------

begin
  fObject.Free;
  inherited;
end;

//----------------------------------------------------------------------------
procedure TXPRadioButton.CMEnabledChanged (var Message : TMessage);
//----------------------------------------------------------------------------

begin
  inherited;
  if fObject.Transparent then
    fObject.InternalPaint;
end;

//----------------------------------------------------------------------------
procedure TXPRadioButton.WMEraseBkgnd (var Message : TWMEraseBkgnd);
//----------------------------------------------------------------------------

begin
  if fObject.Transparent then
    Message.Result:= 1
  else
    inherited;
end;

//----------------------------------------------------------------------------
procedure TXPRadioButton.WMMove (var Message : TMessage);
//----------------------------------------------------------------------------

begin
  inherited;
  if fObject.Transparent then
    fObject.InternalPaint;
end;

//----------------------------------------------------------------------------
procedure TXPRadioButton.WMPaint (var Message : TWMPaint);
//----------------------------------------------------------------------------

var
  PaintStruct : TPaintStruct;
  DrawRect : TRect;

begin
  with fObject do
  begin
    if Transparent then
    begin
      if not (iDrawing > 0) then
      begin
        GetUpdateRect(FControl.Handle, DrawRect, false);
        if not IsRectEmpty(DrawRect) then
        begin
          BeginPaint(FControl.Handle, PaintStruct);
          InternalPaint;
          EndPaint(FControl.Handle, PaintStruct);
          Message.Result:= 0;
        end;
      end;
    end else
      inherited;
  end;
end;

//----------------------------------------------------------------------------
procedure TXPRadioButton.WMSize (var Message : TMessage);
//----------------------------------------------------------------------------

begin
  inherited;
  with fObject do
  begin
    if Transparent then
    begin
      DeleteTempDC;
      InternalPaint;
    end;
  end;
end;

//----------------------------------------------------------------------------
procedure TXPRadioButton.SetTransparent (Value : boolean);
//----------------------------------------------------------------------------

begin
  fTransparent:= Value;
  Value:= Value or
          (UseThemes and not ((Parent is TForm) or (Parent is TXPGroupBox)));
  if (Value <> fObject.Transparent) then
  begin
    fObject.Transparent:= Value;
    Invalidate;
  end;
end;

//============================================================================
// class TXPTrackBar
//============================================================================

//----------------------------------------------------------------------------
constructor TXPTrackBar.Create (AOwner : TComponent);
//----------------------------------------------------------------------------

begin
  inherited;
  fObject:= TTransparentPaint.Create(Self);
end;

//----------------------------------------------------------------------------
destructor TXPTrackBar.Destroy;
//----------------------------------------------------------------------------

begin
  fObject.Free;
  inherited;
end;

//----------------------------------------------------------------------------
procedure TXPTrackBar.CMEnabledChanged (var Message : TMessage);
//----------------------------------------------------------------------------

begin
  inherited;
  if fObject.Transparent then
    fObject.InternalPaint;
end;

//----------------------------------------------------------------------------
procedure TXPTrackBar.WMEraseBkgnd (var Message : TWMEraseBkgnd);
//----------------------------------------------------------------------------

begin
  if fObject.Transparent then
    Message.Result:= 1
  else
    inherited;
end;

//----------------------------------------------------------------------------
procedure TXPTrackBar.WMMove (var Message : TMessage);
//----------------------------------------------------------------------------

begin
  inherited;
  if fObject.Transparent then
    fObject.InternalPaint;
end;

//----------------------------------------------------------------------------
procedure TXPTrackBar.WMPaint (var Message : TWMPaint);
//----------------------------------------------------------------------------

var
  PaintStruct : TPaintStruct;
  DrawRect : TRect;

begin
  with fObject do
  begin
    if Transparent then
    begin
      if not (iDrawing > 0) then
      begin
        GetUpdateRect(FControl.Handle, DrawRect, false);
        if not IsRectEmpty(DrawRect) then
        begin
          BeginPaint(FControl.Handle, PaintStruct);
          InternalPaint;
          EndPaint(FControl.Handle, PaintStruct);
          Message.Result:= 0;
        end;
      end;
    end else
      inherited;
  end;
end;

//----------------------------------------------------------------------------
procedure TXPTrackBar.WMSize (var Message : TMessage);
//----------------------------------------------------------------------------

begin
  inherited;
  with fObject do
  begin
    if Transparent then
    begin
      DeleteTempDC;
      InternalPaint;
    end;
  end;
end;

//----------------------------------------------------------------------------
procedure TXPTrackBar.SetTransparent (Value : boolean);
//----------------------------------------------------------------------------

begin
  fTransparent:= Value;
  Value:= Value or
          (UseThemes and not ((Parent is TForm) or (Parent is TXPGroupBox)));
  if (Value <> fObject.Transparent) then
  begin
    fObject.Transparent:= Value;
    Invalidate;
  end;
end;

//============================================================================
// class TXPPageControlEditor
//============================================================================

resourcestring
  sDEF_TABSHEET = 'XPTabSheet';

{$IFDEF DEF_LANGUAGE}
  sNEW_PAGE = 'Neue Seite';
  sDEL_PAGE = 'Seite lschen';
  sNEXT_PAGE = 'Nchste Seite';
  sPREV_PAGE = 'Vorige Seite';
{$ELSE}
  sNEW_PAGE = 'New Page';
  sDEL_PAGE = 'Delete Page';
  sNEXT_PAGE = 'Next Page';
  sPREV_PAGE = 'Previous Page';
{$ENDIF}

//----------------------------------------------------------------------------
procedure TXPPageControlEditor.ExecuteVerb (Index : integer);
//----------------------------------------------------------------------------

var
  NewPage : TXPTabSheet;
  PControl : TXPPageControl;

begin
  if Component is TXPPageControl then
    PControl:= TXPPageControl(Component)
  else
    PControl:= TXPPageControl(TXPTabSheet(Component).PageControl);

  case Index of
    0:  begin  // New Page
          NewPage:= TXPTabSheet.Create(Designer.GetRoot);
          with NewPage do
          begin
            Parent:= PControl;
            PageControl:= PControl;
            Caption:= sDEF_TABSHEET + IntToStr(PControl.PageCount);
            Name:= Caption;
          end;
        end;
    1:  begin  // Delete Page
          with PControl do
          begin
            NewPage:= TXPTabSheet(ActivePage);
            NewPage.PageControl := nil;
            NewPage.Free;
          end;
        end;
    2:  begin  // Next Page
          PControl.FindNextPage(PControl.ActivePage,True,False);
        end;
    3:  begin  // Previous Page
          PControl.FindNextPage(PControl.ActivePage,False,False);
        end;
  end;
  if (Designer <> nil) then
    Designer.Modified;
end;

//----------------------------------------------------------------------------
function TXPPageControlEditor.GetVerb (Index : integer) : string;
//----------------------------------------------------------------------------

begin
  case Index of
    0 : result:= sNEW_PAGE;
    1 : result:= sDEL_PAGE;
    2 : result:= sNEXT_PAGE;
    3 : result:= sPREV_PAGE;
  end;
end;

//----------------------------------------------------------------------------
function TXPPageControlEditor.GetVerbCount : integer;
//----------------------------------------------------------------------------

begin
  result:= 4;
end;

//============================================================================

initialization

//============================================================================

begin
  iDrawing:= 0;
  TransparentClasses:= TList.Create;
  TransparentClasses.Add(TXPCheckBox);
  TransparentClasses.Add(TXPGroupBox);
  TransparentClasses.Add(TXPRadioButton);
  TransparentClasses.Add(TXPTrackBar);
end;

//============================================================================

finalization

//============================================================================

begin
  TransparentClasses.Free;
end;

//============================================================================

end.


