unit Ado25Editors;

interface  

{$I Ado25.inc}

uses
  Classes, SysUtils, ActiveX, Dialogs, Forms, Controls, DsgnIntf, StrEdit,
  AdoConnection, AdoRecordset, Ado21Int, AdoUtils, AdoConsts;

type
  TOleEnumItem = record
    Value: TOleEnum;
    ValueStr: string;
  end;

  TWideStringProperty = class(TCaptionProperty)
    function GetAttributes: TPropertyAttributes; override;
    procedure Edit; override;
  end;

  TSourceProperty = class(TWideStringProperty)
    function  GetAttributes: TPropertyAttributes; override;
    procedure GetValues(Proc: TGetStrProc); override;
  end;

  TFieldNameProperty = class(TStringProperty)
    function  GetAttributes: TPropertyAttributes; override;
    procedure GetValues(Proc: TGetStrProc); override;
  end;

  TProviderProperty = class(TStringProperty)
    function  GetAttributes: TPropertyAttributes; override;
    procedure GetValues(Proc: TGetStrProc); override;
  end;

  TConnectionStringProperty = class(TStringProperty)
    function  GetAttributes: TPropertyAttributes; override;
    procedure Edit; override;
  end;

  TOleEnumArray = array of TOleEnumItem;

  TOleEnumProperty = class(TEnumProperty)
  private
    FOleEnumArray: TOleEnumArray;
    function OleEnumItemsCount: integer;
    function GetOleEnumItems(index: integer): TOleEnumItem;
    property OleEnumItems[index: integer]: TOleEnumItem read GetOleEnumItems;
  protected
    procedure SetOleEnumArray(Value: array of TOleEnumItem);
  public
    constructor Create(const ADesigner: IFormDesigner; APropCount: Integer); override;
    function GetValue: string; override;
    procedure GetValues(Proc: TGetStrProc); override;
    procedure SetValue(const Value: string); override;
  end;

  TOleEnumSetElementProperty = class(TNestedProperty)
  private
    FElement: TOleEnumItem;
  protected
    constructor Create(Parent: TPropertyEditor; AElement: TOleEnumItem); reintroduce;
    property Element: TOleEnumItem read FElement;
  public
    function AllEqual: Boolean; override;
    function GetAttributes: TPropertyAttributes; override;
    function GetName: string; override;
    function GetValue: string; override;
    procedure GetValues(Proc: TGetStrProc); override;
    procedure SetValue(const Value: string); override;
   end;

  TOleEnumSetProperty = class(TOleEnumProperty)
  public
    function GetAttributes: TPropertyAttributes; override;
    procedure GetProperties(Proc: TGetPropEditProc); override;
    function GetValue: string; override;
  end;

  TConnectOptionProperty = class(TOleEnumProperty)
    constructor Create(const ADesigner: IFormDesigner; APropCount: Integer); override;
  end;

  TStateProperty = class(TOleEnumProperty)
    constructor Create(const ADesigner: IFormDesigner; APropCount: Integer); override;
  end;

  TIsolationLevelProperty = class(TOleEnumProperty)
    constructor Create(const ADesigner: IFormDesigner; APropCount: Integer); override;
  end;

  TCursorLocationProperty = class(TOleEnumProperty)
    constructor Create(const ADesigner: IFormDesigner; APropCount: Integer); override;
  end;

  TModeProperty = class(TOleEnumProperty)
    constructor Create(const ADesigner: IFormDesigner; APropCount: Integer); override;
  end;

  TAttributesProperty = class(TOleEnumProperty)
    constructor Create(const ADesigner: IFormDesigner; APropCount: Integer); override;
  end;

  TMarshalOptionsProperty = class(TOleEnumProperty)
    constructor Create(const ADesigner: IFormDesigner; APropCount: Integer); override;
  end;  

  TPositionProperty = class(TOleEnumProperty)
    constructor Create(const ADesigner: IFormDesigner; APropCount: Integer); override;
  end;

  TCursorTypeProperty = class(TOleEnumProperty)
    constructor Create(const ADesigner: IFormDesigner; APropCount: Integer); override;
  end;

  TLockTypeProperty = class(TOleEnumProperty)
    constructor Create(const ADesigner: IFormDesigner; APropCount: Integer); override;
  end;

  TCommandTypeProperty = class(TOleEnumProperty)
    constructor Create(const ADesigner: IFormDesigner; APropCount: Integer); override;
  end;   

  TDataTypeProperty = class(TOleEnumProperty)
    constructor Create(const ADesigner: IFormDesigner; APropCount: Integer); override;
  end;

  {TFieldStatusProperty = class(TOleEnumProperty)
    constructor Create(const ADesigner: IFormDesigner; APropCount: Integer); override;
  end; }

  TFieldAttributesProperty = class(TOleEnumProperty)
    constructor Create(const ADesigner: IFormDesigner; APropCount: Integer); override;
  end;

  TParameterAttributesProperty = class(TOleEnumProperty)
    constructor Create(const ADesigner: IFormDesigner; APropCount: Integer); override;
  end;  

  TParameterDirectionProperty = class(TOleEnumProperty)
    constructor Create(const ADesigner: IFormDesigner; APropCount: Integer); override;
  end;


implementation 

{$IFDEF WIN32}
 {$D-}
{$ENDIF}

const
  BooleanIdents: array [Boolean] of string = ('False', 'True');

var
  ConnectionStringEditorOpen: Boolean;
  ProviderNames: TStringList;
//------------------------------------------------------------------------------
function StrToBoolean(Value: string): boolean;
begin
  result:= SameText(Value, BooleanIdents[True]);
end;
//------------------------------------------------------------------------------
function BooleanToStr(Value: boolean): string;
begin
  result:= BooleanIdents[Value];
end;            
//==============================================================================
{ TWideStringProperty }
//------------------------------------------------------------------------------
procedure TWideStringProperty.Edit;
var Temp: string;
    Comp: TPersistent;
begin
  with TStrEditDlg.Create(Application) do
  try
    CodeWndBtn.Enabled:= false;
    Comp := GetComponent(0);
    if Comp is TComponent then
      Caption := TComponent(Comp).Name + '.' + GetName
    else Caption := GetName;
    Temp := GetStrValue;
    Memo.Lines.Text:= Temp;
    UpdateStatus(nil);
    if ShowModal = mrOk then begin
      Temp := Memo.Text;
      while (Length(Temp) > 0) and (Temp[Length(Temp)] < ' ') do
        System.Delete(Temp, Length(Temp), 1);
      SetStrValue(Temp);
    end;
  finally
    Free;
  end;
end;
//------------------------------------------------------------------------------
function TWideStringProperty.GetAttributes: TPropertyAttributes;
begin
  Result := inherited GetAttributes + [paDialog];
end;
//==============================================================================
{ TConnectionStringProperty }
//------------------------------------------------------------------------------
procedure TConnectionStringProperty.Edit;
var
  C: TConnection;
  S: String;
begin
  if ConnectionStringEditorOpen then EXIT;
  C:= TConnection(GetComponent(0));
  S:= C.ConnectionString;
  ConnectionStringEditorOpen:= True;
  try
    C.ConnectionString:= ConnectionStringDlg(S);
  finally
    ConnectionStringEditorOpen:= False;
  end;
  if S <> C.ConnectionString then C.Active:= False;
  Modified;
end;
//------------------------------------------------------------------------------
function TConnectionStringProperty.GetAttributes: TPropertyAttributes;
begin
  Result:= [paDialog];
end;
//==============================================================================
{ TProviderProperty }
//------------------------------------------------------------------------------
function TProviderProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paMultiSelect, paValueList, paSortList];
end;
//------------------------------------------------------------------------------
procedure TProviderProperty.GetValues(Proc: TGetStrProc);
var i: integer;
begin
  for i:= 0 to ProviderNames.Count-1 do
    Proc(ProviderNames[i]);
end;
//==============================================================================
{ TSourceProperty }
//------------------------------------------------------------------------------
function TSourceProperty.GetAttributes: TPropertyAttributes;
var Rst: TRecordset;
begin
  Rst:= TRecordset(GetComponent(0));
  case rst.CommandType of
    0: result := [paValueList, paSortList];
    TOleEnum(adCmdUnspecified)  : result:= [];
    adCmdUnknown                : result:= [];
    adCmdText                   : result:= [paDialog];
    adCmdTable                  : result:= [paValueList, paSortList];
    adCmdTableDirect            : result:= [paValueList, paSortList];
    adCmdStoredProc             : result:= [];
    adCmdFile                   : result:= [];
    else                        result:= [];
  end;
end;
//------------------------------------------------------------------------------
procedure TSourceProperty.GetValues(Proc: TGetStrProc);
var Rst: TRecordset;
    Tables: TStringList;
    i: integer;
begin
  Rst:= TRecordset(GetComponent(0));
  if Rst.Connection = nil then
    DoAdoError(emNoConnection);
  Tables:= TStringList.Create;
  try
    Rst.Connection.GetTableNames(Tables);
    for i:= 0 to Tables.Count-1 do
      Proc(Tables[i]);
  finally
    Tables.Free;
  end;
end;
//==============================================================================
{ TFieldNameProperty }
//------------------------------------------------------------------------------
function TFieldNameProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paMultiSelect, paValueList, paSortList];
end;
//------------------------------------------------------------------------------
procedure TFieldNameProperty.GetValues(Proc: TGetStrProc);
var Rst: TRecordset;
    RstFields: TStringList;
    i: integer;
begin
  Rst:= TRecordset(GetComponent(0));
  if Rst.Connection = nil then
    DoAdoError(emNoConnection);
  if Rst.Source = '' then
    DoAdoError(emNoSource);
  RstFields:= TStringList.Create;
  try
    Rst.Connection.GetFieldNames(Rst.Source, RstFields);
    for i:= 0 to RstFields.Count-1 do
      Proc(RstFields[i]);
  finally
    RstFields.Free;
  end;
  Modified;
end;
//==============================================================================
{ TOleEnumProperty }
//------------------------------------------------------------------------------
constructor TOleEnumProperty.Create(const ADesigner: IFormDesigner;
  APropCount: Integer);
begin
  inherited;
end;
//------------------------------------------------------------------------------
procedure TOleEnumProperty.GetValues(Proc: TGetStrProc);
var i: integer;
begin
  for i:= 0 to OleEnumItemsCount-1 do
    Proc(OleEnumItems[i].ValueStr);
end;
//------------------------------------------------------------------------------
function TOleEnumProperty.GetValue: string;
var i: integer;
begin
  for i:= 0 to OleEnumItemsCount-1 do
    if Integer(OleEnumItems[i].Value) = GetOrdValue then begin
       result:= OleEnumItems[i].ValueStr;
       exit;
    end;
  result:= IntToStr(GetOrdValue);
end;
//------------------------------------------------------------------------------
procedure TOleEnumProperty.SetValue(const Value: string);
var i: integer;
begin
  for i:= 0 to OleEnumItemsCount-1 do
    if SameText(Value, OleEnumItems[i].ValueStr) then begin
      SetOrdValue(integer(OleEnumItems[i].Value));
      exit;
    end;
  SetOrdValue(StrToInt(Value));
end;
//------------------------------------------------------------------------------
procedure TOleEnumProperty.SetOleEnumArray(Value: array of TOleEnumItem);
var i: integer;
begin
  SetLength(FOleEnumArray, high(Value) - low(Value) +1);
  for i:= 0 to OleEnumItemsCount-1 do
    FOleEnumArray[i]:= Value[i];
end;
//------------------------------------------------------------------------------
function TOleEnumProperty.GetOleEnumItems(index: integer): TOleEnumItem;
begin
  if (index >= 0) and (index < OleEnumItemsCount) then
    result:= FOleEnumArray[index];
end;
//------------------------------------------------------------------------------
function TOleEnumProperty.OleEnumItemsCount: integer;
begin
  result:= length(FOleEnumArray);
end;
//==============================================================================
{ TOleEnumSetElementProperty }
//------------------------------------------------------------------------------
constructor TOleEnumSetElementProperty.Create(Parent: TPropertyEditor;
  AElement: TOleEnumItem);
begin
  inherited Create(Parent);
  FElement := AElement;
end;
//------------------------------------------------------------------------------
function TOleEnumSetElementProperty.AllEqual: Boolean;
var
  I: Integer;
  S: TOleEnum;
  V: Boolean;
begin
  Result := False;
  if PropCount > 1 then
  begin
    Integer(S) := GetOrdValue;
    V := OleEnumInSet(FElement.Value, S);
    for I := 1 to PropCount - 1 do
    begin
      Integer(S) := GetOrdValueAt(I);
      if OleEnumInSet(FElement.Value, S) <> V then Exit;
    end;
  end;
  Result := True;
end;
//------------------------------------------------------------------------------
function TOleEnumSetElementProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paMultiSelect, paValueList, paSortList];
end;
//------------------------------------------------------------------------------
function TOleEnumSetElementProperty.GetName: string;
begin
  result:= FElement.ValueStr;
end;
//------------------------------------------------------------------------------
function TOleEnumSetElementProperty.GetValue: string;
var
  S: TOleEnum;
begin
  Integer(S) := GetOrdValue;
  Result := BooleanToStr(OleEnumInSet(FElement.Value, S));
end;
//------------------------------------------------------------------------------
procedure TOleEnumSetElementProperty.GetValues(Proc: TGetStrProc);
begin
  Proc(BooleanToStr(False));
  Proc(BooleanToStr(True));
end;
//------------------------------------------------------------------------------
procedure TOleEnumSetElementProperty.SetValue(const Value: string);
var
  S: TOleEnum;
begin
  Integer(S) := GetOrdValue;
  if StrToBoolean(Value) then
    OleEnumInclude(FElement.Value, S)
  else
    OleEnumExclude(FElement.Value, S);
  SetOrdValue(Integer(S));
end;
//==============================================================================
{ TOleEnumSetProperty }
//------------------------------------------------------------------------------
function TOleEnumSetProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paMultiSelect, paSubProperties, paReadOnly, paRevertable];
end;
//------------------------------------------------------------------------------
procedure TOleEnumSetProperty.GetProperties(Proc: TGetPropEditProc);
var
  I: Integer;
begin
    for I := 0 to OleEnumItemsCount-1 do
      Proc(TOleEnumSetElementProperty.Create(Self, OleEnumItems[i]));
end;
//------------------------------------------------------------------------------
function TOleEnumSetProperty.GetValue: string;
var
  S: TOleEnum;
  I: Integer;
begin
  Integer(S) := GetOrdValue;
  Result := '[';
  for I := 0 to OleEnumItemsCount-1 do
    if OleEnumInSet(OleEnumItems[I].Value, S) then
    begin
      if Length(Result) <> 1 then Result := Result + ',';
      Result := Result + OleEnumItems[I].ValueStr;
    end;
  Result := Result + ']';
end;

//==============================================================================
//==============================================================================
{ TConnectOptionProperty }
//------------------------------------------------------------------------------
const
  ConnectOptionArray: array[0..1] of TOleEnumItem =
    ((Value: TOleEnum(adConnectUnspecified);    ValueStr: 'adConnectUnspecified'),
     (Value: adAsyncConnect;                    ValueStr: 'adAsyncConnect'));

constructor TConnectOptionProperty.Create(const ADesigner: IFormDesigner; APropCount: Integer);
begin
  inherited;
  SetOleEnumArray(ConnectOptionArray);
end;
//==============================================================================
{ TStateEditor }
//------------------------------------------------------------------------------
const
  StateArray: array[0..4] of TOleEnumItem =
    ((Value: adStateClosed;     ValueStr: 'adStateClosed'),
     (Value: adStateOpen;       ValueStr: 'adStateOpen'),
     (Value: adStateConnecting; ValueStr: 'adStateConnecting'),
     (Value: adStateExecuting;  ValueStr: 'adStateExecuting'),
     (Value: adStateFetching;   ValueStr: 'adStateFetching'));

constructor TStateProperty.Create(const ADesigner: IFormDesigner;
  APropCount: Integer);
begin
  inherited;
  SetOleEnumArray(StateArray);
end;
//==============================================================================
{ TIsolationLevelPropety }
//------------------------------------------------------------------------------
const
  IsolationLevelArray: array[0..8] of TOleEnumItem =
    ((Value: TOleEnum(adXactUnspecified);       ValueStr: 'adXactUnspecified'),
     (Value: adXactChaos;                       ValueStr: 'adXactChaos'),
     (Value: adXactReadUncommitted;             ValueStr: 'adXactReadUncommitted'),
     (Value: adXactBrowse;                      ValueStr: 'adXactBrowse'),
     (Value: adXactCursorStability;             ValueStr: 'adXactCursorStability'),
     (Value: adXactReadCommitted;               ValueStr: 'adXactReadCommitted'),
     (Value: adXactRepeatableRead;              ValueStr: 'adXactRepeatableRead'),
     (Value: adXactSerializable;                ValueStr: 'adXactSerializable'),
     (Value: adXactIsolated;                    ValueStr: 'adXactIsolated'));

constructor TIsolationLevelProperty.Create(const ADesigner: IFormDesigner;
  APropCount: Integer);
begin
  inherited;
  SetOleEnumArray(IsolationLevelArray);
end;
//==============================================================================
{ TCursorLocationPropety }
//------------------------------------------------------------------------------
const
  CursorLocationArray: array[0..3] of TOleEnumItem =
    ((Value: adUseNone;     ValueStr: 'adUseNone'),
     (Value: adUseServer;       ValueStr: 'adUseServer'),
     (Value: adUseClient; ValueStr: 'adUseClient'),
     (Value: adUseClientBatch;  ValueStr: 'adUseClientBatch'));

constructor TCursorLocationProperty.Create(const ADesigner: IFormDesigner;
  APropCount: Integer);
begin
  inherited;
  SetOleEnumArray(CursorLocationArray)
end;
//==============================================================================
{ TModeProperty }
//------------------------------------------------------------------------------
const
  ModeArray: array[0..7] of TOleEnumItem =
    ((Value: adModeUnknown;     ValueStr: 'adModeUnknown'),
     (Value: adModeRead;       ValueStr: 'adModeRead'),
     (Value: adModeWrite; ValueStr: 'adModeWrite'),
     (Value: adModeReadWrite;  ValueStr: 'adModeReadWrite'),
     (Value: adModeShareDenyRead;  ValueStr: 'adModeShareDenyRead'),
     (Value: adModeShareDenyWrite;  ValueStr: 'adModeShareDenyWrite'),
     (Value: adModeShareExclusive;  ValueStr: 'adModeShareExclusive'),
     (Value: adModeShareDenyNone;  ValueStr: 'adModeShareDenyNone'));

constructor TModeProperty.Create(const ADesigner: IFormDesigner;
  APropCount: Integer);
begin
  inherited;
  SetOleEnumArray(ModeArray);
end;
//==============================================================================
{ TAttributesProperty }
//------------------------------------------------------------------------------
const
  AttributesArray: array[0..3] of TOleEnumItem =
    ((Value: adXactCommitRetaining;     ValueStr: 'adXactCommitRetaining'),
     (Value: adXactAbortRetaining;       ValueStr: 'adXactAbortRetaining'),
     (Value: adXactAsyncPhaseOne; ValueStr: 'adXactAsyncPhaseOne'),
     (Value: adXactSyncPhaseOne;  ValueStr: 'adXactSyncPhaseOne'));

constructor TAttributesProperty.Create(const ADesigner: IFormDesigner;
  APropCount: Integer);
begin
  inherited;
  SetOleEnumArray(AttributesArray);
end;
//==============================================================================
{ TMarshalOptionsProperty }
//------------------------------------------------------------------------------
const
  MarshalOptionsArray: array[0..1] of TOleEnumItem =
    ((Value: adMarshalAll;          ValueStr: 'adMarshalAll'),
     (Value: adMarshalModifiedOnly; ValueStr: 'adMarshalModifiedOnly'));

constructor TMarshalOptionsProperty.Create(const ADesigner: IFormDesigner;
  APropCount: Integer);
begin
  inherited;
  SetOleEnumArray(MarshalOptionsArray);
end;
//==============================================================================
{ TPositionProperty }
//------------------------------------------------------------------------------
const
  PositionArray: array[0..2] of TOleEnumItem =
    ((Value: TOleEnum(adPosBOF);        ValueStr: 'adPosBOF'),
     (Value: TOleEnum(adPosEOF);        ValueStr: 'adPosEOF'),
     (Value: TOleEnum(adPosUnknown);    ValueStr: 'adPosUnknown'));

constructor TPositionProperty.Create(const ADesigner: IFormDesigner;
  APropCount: Integer);
begin
  inherited;
  SetOleEnumArray(PositionArray);
end;
//==============================================================================
{ TCursorTypeProperty }
//------------------------------------------------------------------------------
const
  CursorTypeArray: array[0..3] of TOleEnumItem =
    ((Value: adOpenForwardOnly;  ValueStr: 'adOpenForwardOnly'),
     (Value: adOpenKeyset;       ValueStr: 'adOpenKeyset'),
     (Value: adOpenDynamic;      ValueStr: 'adOpenDynamic'),
     (Value: adOpenStatic;       ValueStr: 'adOpenStatic'));

constructor TCursorTypeProperty.Create(const ADesigner: IFormDesigner;
  APropCount: Integer);
begin
  inherited;
  SetOleEnumArray(CursorTypeArray);
end;
//==============================================================================
{ TLockTypeProperty }
//------------------------------------------------------------------------------
const
  LockTypeArray: array[0..4] of TOleEnumItem =
    ((Value: TOleEnum(adLockUnspecified);       ValueStr: 'adLockUnspecified'),
     (Value: adLockReadOnly;                    ValueStr: 'adLockReadOnly'),
     (Value: adLockPessimistic;                 ValueStr: 'adLockPessimistic'),
     (Value: adLockOptimistic;                  ValueStr: 'adLockOptimistic'),
     (Value: adLockBatchOptimistic;             ValueStr: 'adLockBatchOptimistic'));

constructor TLockTypeProperty.Create(const ADesigner: IFormDesigner;
  APropCount: Integer);
begin
  inherited;
  SetOleEnumArray(LockTypeArray);
end;
//==============================================================================
{ TCommandTypeProperty }
//------------------------------------------------------------------------------
const
  CommandTypeArray: array[0..6] of TOleEnumItem =
    ((Value: TOleEnum(adCmdUnspecified);        ValueStr: 'adCmdUnspecified'),
     (Value: adCmdUnknown;                      ValueStr: 'adCmdUnknown'),
     (Value: adCmdText;                         ValueStr: 'adCmdText'),
     (Value: adCmdTable;                        ValueStr: 'adCmdTable'),
     (Value: adCmdStoredProc;                   ValueStr: 'adCmdStoredProc'),
     (Value: adCmdFile;                         ValueStr: 'adCmdFile'),
     (Value: adCmdTableDirect;                  ValueStr: 'adCmdTableDirect'));

constructor TCommandTypeProperty.Create(const ADesigner: IFormDesigner;
  APropCount: Integer);
begin
  inherited;
  SetOleEnumArray(CommandTypeArray);
end;
//==============================================================================
{ TDataTypeProperty }
//------------------------------------------------------------------------------
const
  DataTypeArray: array[0..38] of TOleEnumItem =
    ((Value: adEmpty;     ValueStr: 'adEmpty'),
     (Value: adTinyInt;        ValueStr: 'adTinyInt'),
     (Value: adSmallInt;     ValueStr: 'adSmallInt'),
     (Value: adInteger;     ValueStr: 'adInteger'),
     (Value: adBigInt;     ValueStr: 'adBigInt'),
     (Value: adUnsignedTinyInt;     ValueStr: 'adUnsignedTinyInt'),
     (Value: adUnsignedSmallInt;     ValueStr: 'adUnsignedSmallInt'),
     (Value: adUnsignedInt;     ValueStr: 'adUnsignedInt'),
     (Value: adUnsignedBigInt;     ValueStr: 'adUnsignedBigInt'),
     (Value: adSingle;     ValueStr: 'adSingle'),
     (Value: adDouble;     ValueStr: 'adDouble'),
     (Value: adCurrency;     ValueStr: 'adCurrency'),
     (Value: adDecimal;     ValueStr: 'adDecimal'),
     (Value: adNumeric;     ValueStr: 'adNumeric'),
     (Value: adBoolean;     ValueStr: 'adBoolean'),
     (Value: adError;     ValueStr: 'adError'),
     (Value: adUserDefined;     ValueStr: 'adUserDefined'),
     (Value: adVariant;     ValueStr: 'adVariant'),
     (Value: adIDispatch;      ValueStr: 'adIDispatch'),
     (Value: adIUnknown;      ValueStr: 'adIUnknown'),
     (Value: adGUID;      ValueStr: 'adGUID'),
     (Value: adDate;      ValueStr: 'adDate'),
     (Value: adDBDate;      ValueStr: 'adDBDate'),
     (Value: adDBTime;      ValueStr: 'adDBTime'),
     (Value: adDBTimeStamp;      ValueStr: 'adDBTimeStamp'),
     (Value: adBSTR;      ValueStr: 'adBSTR'),
     (Value: adChar;      ValueStr: 'adChar'),
     (Value: adVarChar;      ValueStr: 'adVarChar'),
     (Value: adLongVarChar;      ValueStr: 'adLongVarChar'),
     (Value: adWChar;      ValueStr: 'adWChar'),
     (Value: adVarWChar;      ValueStr: 'adVarWChar'),
     (Value: adLongVarWChar;      ValueStr: 'adLongVarWChar'),
     (Value: adBinary;      ValueStr: 'adBinary'),
     (Value: adVarBinary;      ValueStr: 'adVarBinary'),
     (Value: adLongVarBinary;      ValueStr: 'adLongVarBinary'),
     (Value: adChapter;      ValueStr: 'adChapter'),
     (Value: adFileTime;      ValueStr: 'adFileTime'),
     (Value: adPropVariant;      ValueStr: 'adPropVariant'),
     (Value: adVarNumeric;      ValueStr: 'adVarNumeric'));

constructor TDataTypeProperty.Create(const ADesigner: IFormDesigner;
  APropCount: Integer);
begin
  inherited;
  SetOleEnumArray(DataTypeArray);
end;
//==============================================================================
{ TFieldStatusProperty }
//------------------------------------------------------------------------------
{const
  FieldStatusArray: array[0..28] of TOleEnumItem =
    ((Value: adFieldOK;                  ValueStr: 'adFieldOK'),
     (Value: adFieldCantConvertValue;    ValueStr: 'adFieldCantConvertValue'),
     (Value: adFieldIsNull;              ValueStr: 'adFieldIsNull'),
     (Value: adFieldTruncated;           ValueStr: 'adFieldTruncated'),
     (Value: adFieldSignMismatch;        ValueStr: 'adFieldSignMismatch'),
     (Value: adFieldDataOverflow;        ValueStr: 'adFieldDataOverflow'),
     (Value: adFieldCantCreate;          ValueStr: 'adFieldCantCreate'),
     (Value: adFieldUnavailable;         ValueStr: 'adFieldUnavailable'),
     (Value: adFieldPermissionDenied;    ValueStr: 'adFieldPermissionDenied'),
     (Value: adFieldIntegrityViolation;  ValueStr: 'adFieldIntegrityViolation'),
     (Value: adFieldSchemaViolation;     ValueStr: 'adFieldSchemaViolation'),
     (Value: adFieldBadStatus;           ValueStr: 'adFieldBadStatus'),
     (Value: adFieldDefault;             ValueStr: 'adFieldDefault'),
     (Value: adFieldIgnore;              ValueStr: 'adFieldIgnore'),
     (Value: adFieldDoesNotExist;        ValueStr: 'adFieldDoesNotExist'),
     (Value: adFieldInvalidURL;          ValueStr: 'adFieldInvalidURL'),
     (Value: adFieldResourceLocked;      ValueStr: 'adFieldResourceLocked'),
     (Value: adFieldResourceExists;      ValueStr: 'adFieldResourceExists'),
     (Value: adFieldCannotComplete;      ValueStr: 'adFieldCannotComplete'),
     (Value: adFieldVolumeNotFound;      ValueStr: 'adFieldVolumeNotFound'),
     (Value: adFieldOutOfSpace;          ValueStr: 'adFieldOutOfSpace'),
     (Value: adFieldCannotDeleteSource;  ValueStr: 'adFieldCannotDeleteSource'),
     (Value: adFieldResourceOutOfScope;  ValueStr: 'adFieldResourceOutOfScope'),
     (Value: adFieldAlreadyExists;       ValueStr: 'adFieldAlreadyExists'),
     (Value: adFieldPendingInsert;       ValueStr: 'adFieldPendingInsert'),
     (Value: adFieldPendingDelete;       ValueStr: 'adFieldPendingDelete'),
     (Value: adFieldPendingChange;       ValueStr: 'adFieldPendingChange'),
     (Value: adFieldPendingUnknown;      ValueStr: 'adFieldPendingUnknown'),
     (Value: adFieldPendingUnknownDelete;  ValueStr: 'adFieldPendingUnknownDelete'));

constructor TFieldStatusProperty.Create(const ADesigner: IFormDesigner;
  APropCount: Integer);
begin
  inherited;
  SetOleEnumArray(FieldStatusArray);
end; }
//==============================================================================
{ TFieldAttributesProperty }
//------------------------------------------------------------------------------
const
  FieldAttributesArray: array[0..12] of TOleEnumItem =
    ((Value: TOleEnum(adFldUnspecified);        ValueStr: 'adFldUnspecified'),
     (Value: adFldMayDefer;                     ValueStr: 'adFldMayDefer'),
     (Value: adFldUpdatable;                    ValueStr: 'adFldUpdatable'),
     (Value: adFldUnknownUpdatable;             ValueStr: 'adFldUnknownUpdatable'),
     (Value: adFldFixed;                        ValueStr: 'adFldFixed'),
     (Value: adFldIsNullable;                   ValueStr: 'adFldIsNullable'),
     (Value: adFldMayBeNull;                    ValueStr: 'adFldMayBeNull'),
     (Value: adFldLong;                         ValueStr: 'adFldLong'),
     (Value: adFldRowID;                        ValueStr: 'adFldRowID'),
     (Value: adFldRowVersion;                   ValueStr: 'adFldRowVersion'),
     (Value: adFldCacheDeferred;                ValueStr: 'adFldCacheDeferred'),
     (Value: adFldNegativeScale;                ValueStr: 'adFldNegativeScale'),
     (Value: adFldKeyColumn;                    ValueStr: 'adFldKeyColumn'));

constructor TFieldAttributesProperty.Create(const ADesigner: IFormDesigner;
  APropCount: Integer);
begin
  inherited;
  SetOleEnumArray(FieldAttributesArray);
end;
//==============================================================================
{ TParameterAttributesProperty }
//------------------------------------------------------------------------------
const
  ParameterAttributesArray: array[0..2] of TOleEnumItem =
    ((Value: adParamSigned;     ValueStr: 'adParamSigned'),
     (Value: adParamNullable;   ValueStr: 'adParamNullable'),
     (Value: adParamLong;       ValueStr: 'adParamLong'));

constructor TParameterAttributesProperty.Create(const ADesigner: IFormDesigner;
  APropCount: Integer);
begin
  inherited;
  SetOleEnumArray(ParameterAttributesArray);
end;
//==============================================================================
{ TParameterDirectionProperty }
//------------------------------------------------------------------------------
const
  ParameterDirectionArray: array[0..4] of TOleEnumItem =
    ((Value: adParamUnknown;     ValueStr: 'adParamUnknown'),
     (Value: adParamInput;       ValueStr: 'adParamInput'),
     (Value: adParamOutput;      ValueStr: 'adParamOutput'),
     (Value: adParamInputOutput; ValueStr: 'adParamInputOutput'),
     (Value: adParamReturnValue; ValueStr: 'adParamReturnValue'));

constructor TParameterDirectionProperty.Create(const ADesigner: IFormDesigner;
  APropCount: Integer);
begin
  inherited;
  SetOleEnumArray(ParameterDirectionArray);
end;

initialization
  ProviderNames:= TStringList.Create;
  ProviderNames.Sorted:= true;
  ProviderNames.Duplicates:= dupIgnore;
  GetProviderNames(ProviderNames);

//------------------------------------------------------------------------------
finalization
  FreeAndNil(ProviderNames);

end.
