unit AdoRecordset;

interface

{$I Ado25.inc}

uses
  ActiveX, Classes, Graphics, OleServer, StdVCL,  Windows, Forms, Dialogs,
  SysUtils,
  Ado21Int, AdoConnection, AdoConsts, AdoUtils, AdoCommand;

type
  TRecordsetWillChangeField = procedure(Sender: TObject; cFields: Integer;
                                                         Fields: OleVariant;
                                                         var adStatus: EventStatusEnum;
                                                         const pRecordset: IRecordset) of object;
  TRecordsetFieldChangeComplete = procedure(Sender: TObject; cFields: Integer;
                                                             Fields: OleVariant;
                                                             const pError: IError;
                                                             var adStatus: EventStatusEnum;
                                                             const pRecordset: IRecordset) of object;
  TRecordsetWillChangeRecord = procedure(Sender: TObject; adReason: EventReasonEnum;
                                                          cRecords: Integer;
                                                          var adStatus: EventStatusEnum;
                                                          const pRecordset: IRecordset) of object;
  TRecordsetRecordChangeComplete = procedure(Sender: TObject; adReason: EventReasonEnum;
                                                              cRecords: Integer;
                                                              const pError: IError;
                                                              var adStatus: EventStatusEnum;
                                                              const pRecordset: IRecordset) of object;
  TRecordsetWillChangeRecordset = procedure(Sender: TObject; adReason: EventReasonEnum;
                                                             var adStatus: EventStatusEnum;
                                                             const pRecordset: IRecordset) of object;
  TRecordsetRecordsetChangeComplete = procedure(Sender: TObject; adReason: EventReasonEnum;
                                                                 const pError: IError;
                                                                 var adStatus: EventStatusEnum;
                                                                 const pRecordset: IRecordset) of object;
  TRecordsetWillMove = procedure(Sender: TObject; adReason: EventReasonEnum; 
                                                  var adStatus: EventStatusEnum;
                                                  const pRecordset: IRecordset) of object;
  TRecordsetMoveComplete = procedure(Sender: TObject; adReason: EventReasonEnum;
                                                      const pError: IError;
                                                      var adStatus: EventStatusEnum;
                                                      const pRecordset: IRecordset) of object;
  TRecordsetEndOfRecordset = procedure(Sender: TObject; var fMoreData: WordBool;
                                                        var adStatus: EventStatusEnum;
                                                        const pRecordset: IRecordset) of object;
  TRecordsetFetchProgress = procedure(Sender: TObject; Progress: Integer;
                                                       MaxProgress: Integer;
                                                       var adStatus: EventStatusEnum;
                                                       const pRecordset: IRecordset) of object;
  TRecordsetFetchComplete = procedure(Sender: TObject; const pError: IError;
                                                       var adStatus: EventStatusEnum;
                                                       const pRecordset: IRecordset) of object;


  TFieldItems = class;

  TRecordset = class(TOleServer)
  private
    FIntf:        IRecordset;
    FConnection: TConnection;
    FFieldItems: TFieldItems;
    FCommandType: CommandTypeEnum;
    FSource: WideString;
    FAbsolutePage: PositionEnum;
    FAbsolutePosition: PositionEnum;
    FMarshalOptions: MarshalOptionsEnum;
    FFilter: OleVariant;
    FDetailRecordsets: TList;
    FRecordLocked: boolean;

    FOnWillChangeField: TRecordsetWillChangeField;
    FOnFieldChangeComplete: TRecordsetFieldChangeComplete;
    FOnWillChangeRecord: TRecordsetWillChangeRecord;
    FOnRecordChangeComplete: TRecordsetRecordChangeComplete;
    FOnWillChangeRecordset: TRecordsetWillChangeRecordset;
    FOnRecordsetChangeComplete: TRecordsetRecordsetChangeComplete;
    FOnWillMove: TRecordsetWillMove;
    FOnMoveComplete: TRecordsetMoveComplete;
    FOnEndOfRecordset: TRecordsetEndOfRecordset;
    FOnFetchProgress: TRecordsetFetchProgress;
    FOnFetchComplete: TRecordsetFetchComplete;
    FCommand: TCommand;
    function GetFieldValue(FieldName: string): OleVariant;
    procedure SetFieldValue(FieldName: string; Value: OleVariant);
    function GetRecordset: IRecordset;
    procedure SetConnection(const Value: TConnection);
    function GetActive: boolean;
    procedure SetActive(const Value: boolean);
    procedure WaitUntilNotReady;
    procedure SetSource(const Value: WideString);
    procedure SetCommandType(const Value: CommandTypeEnum);
    function GetFieldItems: TFieldItems;
    function GetFieldAsStr(FieldName: string): string;
    procedure SetFieldAsStr(FieldName: string; const Value: string);
    function GetFieldAsBool(FieldName: string): boolean;
    function GetFieldAsInt(FieldName: string): integer;
    procedure SetFieldAsBool(FieldName: string; const Value: boolean);
    procedure SetFieldAsInt(FieldName: string; const Value: integer);
    function GetFieldAsDbl(FieldName: string): double;
    procedure SetFieldAsDbl(FieldName: string; const Value: double);
    function GetFiltered: boolean;
    procedure SetCommand(const Value: TCommand);
    function GetSource: WideString;
  protected
    procedure InitServerData; override;
    procedure InvokeEvent(DispID: TDispID; var Params: TVariantArray); override;
    function  Get_AbsolutePosition: PositionEnum;
    procedure Set_AbsolutePosition(pl: PositionEnum);
    procedure _Set_ActiveConnection(const pvar: IDispatch);
    procedure Set_ActiveConnection(pvar: OleVariant);
    function  Get_ActiveConnection: OleVariant;
    function  Get_BOF: WordBool;
    function  Get_Bookmark: OleVariant;
    procedure Set_Bookmark(pvBookmark: OleVariant);
    function  Get_CacheSize: Integer;
    procedure Set_CacheSize(pl: Integer);
    function  Get_CursorType: CursorTypeEnum;
    procedure Set_CursorType(plCursorType: CursorTypeEnum);
    function  Get_EOF: WordBool;
    function  Get_Fields: IFields;
    function  Get_LockType: LockTypeEnum;
    procedure Set_LockType(plLockType: LockTypeEnum);
    function  Get_MaxRecords: Integer;
    procedure Set_MaxRecords(plMaxRecords: Integer);
    function  Get_RecordCount: Integer;
    procedure _Set_Source(const pvSource: IDispatch);
    procedure Set_Source(const pvSource: WideString);
    function  Get_Source: OleVariant;
    function  Get_AbsolutePage: PositionEnum;
    procedure Set_AbsolutePage(pl: PositionEnum);
    function  Get_EditMode: EditModeEnum;
    function  Get_Filter: OleVariant;
    procedure Set_Filter(Criteria: OleVariant);
    function  Get_PageCount: Integer;
    function  Get_PageSize: Integer;
    procedure Set_PageSize(pl: Integer);
    function  Get_Sort: WideString;
    procedure Set_Sort(const Criteria: WideString);
    function  Get_Status: Integer;
    function  Get_State: Integer;
    function  Get_CursorLocation: CursorLocationEnum;
    procedure Set_CursorLocation(plCursorLoc: CursorLocationEnum);
    function  Get_Collect(Index: OleVariant): OleVariant;
    procedure Set_Collect(Index: OleVariant; pvar: OleVariant);
    function  Get_MarshalOptions: MarshalOptionsEnum;
    procedure Set_MarshalOptions(peMarshal: MarshalOptionsEnum);
    procedure CheckConnection;
    procedure CheckActive;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor  Destroy; override;
    procedure Connect; override;
    procedure ConnectTo(svrIntf: IRecordset);
    procedure Disconnect; override;
    procedure LockRecord;
    procedure UnlockRecord;
    procedure AddNew; overload;
    procedure AddNew(FieldList: OleVariant); overload;
    procedure AddNew(FieldList: OleVariant; Values: OleVariant); overload;
    procedure CancelUpdate;
    procedure Close;
    procedure Delete(AffectRecords: AffectEnum = adAffectCurrent);
    function  GetRows(Rows: Integer): OleVariant; overload;
    function  GetRows(Rows: Integer; Start: OleVariant): OleVariant; overload;
    function  GetRows(Rows: Integer; Start: OleVariant; Fields: OleVariant): OleVariant; overload;
    procedure MoveNext;
    procedure MovePrevious;
    procedure MoveFirst;
    procedure MoveLast;
    procedure Move(NumRecords: Integer); overload;
    procedure Move(NumRecords: Integer; Start: OleVariant); overload;
    procedure Open; overload;
    procedure Open(Source: OleVariant; ActiveConnection: OleVariant; CursorType: CursorTypeEnum;
                   LockType: LockTypeEnum; Options: Integer); overload;
    procedure Requery(Options: Integer); overload;
    procedure Requery; overload;
    procedure _xResync(AffectRecords: AffectEnum);
    procedure Update; overload;
    procedure Update(Fields: OleVariant); overload;
    procedure Update(Fields: OleVariant; Values: OleVariant); overload;
    function  _xClone: IRecordset;
    procedure UpdateBatch(AffectRecords: AffectEnum);
    procedure CancelBatch(AffectRecords: AffectEnum);
    function  NextRecordset: IRecordset; overload;
    function  NextRecordset(out RecordsAffected: OleVariant): IRecordset; overload;
    function  Supports(CursorOptions: CursorOptionEnum): WordBool;
    function Find(const Criteria: WideString; SkipRecords: Integer;
                    SearchDirection: SearchDirectionEnum): boolean; overload;
    function Find(const Criteria: WideString; SkipRecords: Integer;
                   SearchDirection: SearchDirectionEnum; Start: OleVariant): boolean; overload;
    function FindNext(const Criteria: WideString): boolean; overload;    
    function FindPrevious(const Criteria: WideString): boolean; overload;
    function FindFirst(const Criteria: WideString): boolean; overload;
    function FindLast(const Criteria: WideString): boolean; overload;
    property  Recordset: IRecordset read GetRecordset;
    property EditMode: EditModeEnum read Get_EditMode;
    property BOF: WordBool read Get_BOF;
    property Bookmark: OleVariant read Get_Bookmark write Set_Bookmark;
    property EOF: WordBool read Get_EOF;
    property Fields: IFields read Get_Fields;
    property FieldAsInt[FieldName: string]: integer read GetFieldAsInt write SetFieldAsInt;
    property FieldAsBool[FieldName: string]: boolean read GetFieldAsBool write SetFieldAsBool;
    property FieldAsStr[FieldName: string]: string read GetFieldAsStr write SetFieldAsStr;
    property FieldAsDbl[FieldName: string]: double read GetFieldAsDbl write SetFieldAsDbl;
    property PageCount: Integer read Get_PageCount;
    property Status: Integer read Get_Status;     
    property Filter: OleVariant read Get_Filter write Set_Filter;
    property Collect[Index: OleVariant]: OleVariant read Get_Collect write Set_Collect;
    property Filtered: boolean read GetFiltered;
  published
    property State: Integer read Get_State;
    property AbsolutePosition: PositionEnum read Get_AbsolutePosition write Set_AbsolutePosition;
    property CacheSize: Integer read Get_CacheSize write Set_CacheSize;
    property CursorType: CursorTypeEnum read Get_CursorType write Set_CursorType;
    property LockType: LockTypeEnum read Get_LockType write Set_LockType;
    property MaxRecords: Integer read Get_MaxRecords write Set_MaxRecords;
    property AbsolutePage: PositionEnum read Get_AbsolutePage write Set_AbsolutePage;
    property PageSize: Integer read Get_PageSize write Set_PageSize;
    property Sort: WideString read Get_Sort write Set_Sort;
    property CursorLocation: CursorLocationEnum read Get_CursorLocation write Set_CursorLocation;
    property MarshalOptions: MarshalOptionsEnum read Get_MarshalOptions write Set_MarshalOptions;
    //new properties
    property Connection: TConnection read FConnection write SetConnection;
    property Source: WideString read GetSource write SetSource;
    property CommandType: CommandTypeEnum read FCommandType write SetCommandType;    
    property Active: boolean read GetActive write SetActive default false;
    property FieldItems: TFieldItems read GetFieldItems;
    property RecordCount: Integer read Get_RecordCount;
    //property Command: TCommand read FCommand write SetCommand;

    property OnWillChangeField: TRecordsetWillChangeField read FOnWillChangeField write FOnWillChangeField;
    property OnFieldChangeComplete: TRecordsetFieldChangeComplete read FOnFieldChangeComplete write FOnFieldChangeComplete;
    property OnWillChangeRecord: TRecordsetWillChangeRecord read FOnWillChangeRecord write FOnWillChangeRecord;
    property OnRecordChangeComplete: TRecordsetRecordChangeComplete read FOnRecordChangeComplete write FOnRecordChangeComplete;
    property OnWillChangeRecordset: TRecordsetWillChangeRecordset read FOnWillChangeRecordset write FOnWillChangeRecordset;
    property OnRecordsetChangeComplete: TRecordsetRecordsetChangeComplete read FOnRecordsetChangeComplete write FOnRecordsetChangeComplete;
    property OnWillMove: TRecordsetWillMove read FOnWillMove write FOnWillMove;
    property OnMoveComplete: TRecordsetMoveComplete read FOnMoveComplete write FOnMoveComplete;
    property OnEndOfRecordset: TRecordsetEndOfRecordset read FOnEndOfRecordset write FOnEndOfRecordset;
    property OnFetchProgress: TRecordsetFetchProgress read FOnFetchProgress write FOnFetchProgress;
    property OnFetchComplete: TRecordsetFetchComplete read FOnFetchComplete write FOnFetchComplete;
  end;

  TFieldItem = class(TCollectionItem)
  private
    FField: IField;
    procedure CheckField;
    function GetActualSize: integer;
    function GetAttributes: FieldAttributeEnum;
    function GetDefinedSize: integer;
    function GetName: WideString;
    function GetNumericScale: byte;
    function GetOriginalValue: OleVariant;
    function GetPrecision: byte;
    function GetProperties: IProperties;
    function GetType: DataTypeEnum;
    function GetUnderlyingValue: OleVariant;
    function GetValue: OleVariant;
    procedure SetField(const Value: IField);
    procedure SetValue(const Value: OleVariant);
  protected
    function GetDisplayName: string; override;
  public
    constructor Create(Collection: TCollection); override;
    procedure AppendChunk(Data: OleVariant);
    function GetChunk(Length: Integer): OleVariant;
    procedure Assign(Source: TPersistent); override;

    property Field: IField read FField write SetField;
    property Properties: IProperties read GetProperties;
    property OriginalValue: OleVariant read GetOriginalValue;
    property UnderlyingValue: OleVariant read GetUnderlyingValue;
    property Value: OleVariant read GetValue write SetValue;
  published
    property ActualSize: integer read GetActualSize;
    property Attributes: FieldAttributeEnum read GetAttributes;
    property DefinedSize: integer read GetDefinedSize;
    property Name: WideString read GetName;
    property NumericScale: byte read GetNumericScale;
    property Precision: byte read GetPrecision;
    property Type_: DataTypeEnum read GetType;
  end;

  TFieldItems = class(TCollection)
  private
    FFields: IFields;
    procedure SetFields(const Value: IFields);
    function InternalAdd: TFieldItem;
  protected
    ///procedure Added(var Item: TCollectionItem); override;
    //procedure Deleting(Item: TCollectionItem); override;
  public
    constructor Create;
    function Add: TFieldItem;
    function Insert(Index: Integer): TFieldItem;
    function GetItem(Index: Integer): TFieldItem;
    procedure SetItem(Index: Integer; Value: TFieldItem);
    property Items[Index: Integer]: TFieldItem read GetItem write SetItem;
    property Fields: IFields read FFields write SetFields;
  end;

  TRecordsetFields = class(TComponent)
  private
    FFieldItems: TFieldItems;
    FRecordset: TRecordset;
    procedure SetRecordset(const Value: TRecordset);
    function GetFieldItems: TFieldItems;
    procedure CheckRecordset;
    function GetFields: IFields;
  protected
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
  public
    constructor Create(AOwner: TComponent); override;
    procedure Update;
    procedure CancelUpdate;
    property Fields: IFields read GetFields;
  published
    property Recordset: TRecordset read FRecordset write SetRecordset;
    property FieldItems: TFieldItems read GetFieldItems;
  end;

implementation

uses ComObj;

procedure TRecordset.InitServerData;
const
  CServerData: TServerData = (
    ClassID:   '{00000535-0000-0010-8000-00AA006D2EA4}';
    IntfIID:   '{0000050E-0000-0010-8000-00AA006D2EA4}';
    EventIID:  '{00000266-0000-0010-8000-00AA006D2EA4}';
    LicenseKey: nil;
    Version: 500);
begin
  ServerData := @CServerData;
end;

procedure TRecordset.Connect;
var
  punk: IUnknown;
begin
  if FIntf = nil then
  begin
    punk := GetServer;
    ConnectEvents(punk);
    Fintf:= punk as IRecordset;
  end;
end;

procedure TRecordset.ConnectTo(svrIntf: IRecordset);
begin
  Disconnect;
  FIntf := svrIntf;
  ConnectEvents(FIntf);
end;

procedure TRecordset.DisConnect;
begin
  if Fintf <> nil then
  begin
    DisconnectEvents(FIntf);
    FIntf := nil;
  end;
end;

function TRecordset.GetRecordset: IRecordset;
begin
  if FIntf = nil then
    Connect;
  Assert(FIntf <> nil, 'DefaultInterface is NULL. Component is not connected to Server. You must call ''Connect'' or ''ConnectTo'' before this operation');
  Result := FIntf;
end;

constructor TRecordset.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FFieldItems:= TFieldItems.Create;
  FFilter:= adFilterNone;
  FMarshalOptions:= adMarshalAll;
  FDetailRecordsets:= TList.Create;
end;

destructor TRecordset.Destroy;
begin
  FreeAndNil(FFieldItems);
  FreeAndNil(FDetailRecordsets);
  inherited Destroy;
end;

procedure TRecordset.InvokeEvent(DispID: TDispID; var Params: TVariantArray);
type PEventStatusEnum = ^EventStatusEnum;
type PWordBool = ^WordBool;

   function VarToIDispatch(Value: OleVariant): IDispatch;
   begin
     result:= IDispatch(TVarData(Value).VDispatch);
   end;

   function VarToIError(Value: OleVariant): IError;
   begin
       result:= VarToIDispatch(Value) as IError;
   end;      

   function VarToIRecordset(Value: OleVariant): IRecordset;
   begin
       result:= VarToIDispatch(Value) as IRecordset;
   end;

   function VarToStatusRef(Value: OleVariant): PEventStatusEnum;
   begin
      result:= @EventStatusEnum(TVarData(Value).VInteger);
   end;   

   function VarToBooleanRef(Value: OleVariant): PWordBool;
   begin
      result:= @WordBool(TVarData(Value).VBoolean);
   end;

begin
  case DispID of
   -1: Exit;  // DISPID_UNKNOWN
   9: if Assigned(FOnWillChangeField) then
            FOnWillChangeField(Self, Params[0], Params[1],
                               VarToStatusRef(Params[2])^,
                               VarToIRecordset(Params[3]));
   10: if Assigned(FOnFieldChangeComplete) then
            FOnFieldChangeComplete(Self, Params[0], Params[1],
                                   VarToIError(Params[2]),
                                   VarToStatusRef(Params[3])^,
                                   VarToIRecordset(Params[4]));
   11: if Assigned(FOnWillChangeRecord) then
            FOnWillChangeRecord(Self, Params[0], Params[1],
                                VarToStatusRef(Params[2])^,
                                VarToIRecordset(Params[3]));
   12: if Assigned(FOnRecordChangeComplete) then
            FOnRecordChangeComplete(Self,Params[0],Params[1],
                                    VarToIError(Params[2]),
                                    VarToStatusRef(Params[3])^,
                                    VarToIRecordset(Params[4]));
   13: if Assigned(FOnWillChangeRecordset) then
            FOnWillChangeRecordset(Self, Params[0], VarToStatusRef(Params[1])^,
                                   VarToIRecordset(Params[2]));
   14: if Assigned(FOnRecordsetChangeComplete) then
            FOnRecordsetChangeComplete(Self, Params[0], VarToIError(Params[1]),
                                       VarToStatusRef(Params[2])^,
                                       VarToIRecordset(Params[3]));
   15: if Assigned(FOnWillMove) then
            FOnWillMove(Self, Params[0], VarToStatusRef(Params[1])^,
                        VarToIRecordset(Params[2]));
   16: if Assigned(FOnMoveComplete) then
            FOnMoveComplete(Self, Params[0], VarToIError(Params[1]),
                            VarToStatusRef(Params[2])^,
                            VarToIRecordset(Params[3]));
   17: if Assigned(FOnEndOfRecordset) then
            FOnEndOfRecordset(Self, VarToBooleanRef(Params[0])^,
                              VarToStatusRef(Params[1])^,
                              VarToIRecordset(Params[2]));
   18: if Assigned(FOnFetchProgress) then
            FOnFetchProgress(Self, Params[0], Params[1],
                             VarToStatusRef(Params[2])^,
                             VarToIRecordset(Params[3]));
   19: if Assigned(FOnFetchComplete) then
            FOnFetchComplete(Self, VarToIError(Params[0]),
                             VarToStatusRef(Params[1])^,
                             VarToIRecordset(Params[2]));
  end; {case DispID}
end;

function  TRecordset.Get_AbsolutePosition: PositionEnum;
begin
  if Active then
    FAbsolutePosition:= Recordset.AbsolutePosition;
  Result := FAbsolutePosition;
end;

procedure TRecordset.Set_AbsolutePosition(pl: PositionEnum);
begin    
  FAbsolutePosition:= pl;
  if Active then begin
    Recordset.AbsolutePosition := FAbsolutePosition;
  end;
end;

function  TRecordset.Get_AbsolutePage: PositionEnum;
begin
  if Active then
    FAbsolutePage:= Recordset.AbsolutePage;
  Result := FAbsolutePage;
end;

procedure TRecordset.Set_AbsolutePage(pl: PositionEnum);
begin
  FAbsolutePage:= pl;
  if Active then begin
    Recordset.AbsolutePage := FAbsolutePage;
  end;
end;

procedure TRecordset._Set_ActiveConnection(const pvar: IDispatch);
  { Warning: The property ActiveConnection has a setter and a getter whose
  types do not match. Delphi was unable to generate a property of
  this sort and so is using a Variant to set the property instead. }
var
  InterfaceVariant: OleVariant;
begin
  InterfaceVariant := Recordset;
  InterfaceVariant.ActiveConnection := pvar;
end;

procedure TRecordset.Set_ActiveConnection(pvar: OleVariant);
  { Warning: The property ActiveConnection has a setter and a getter whose
  types do not match. Delphi was unable to generate a property of
  this sort and so is using a Variant to set the property instead. }
var
  InterfaceVariant: OleVariant;
begin
  InterfaceVariant := Recordset;
  InterfaceVariant.ActiveConnection := pvar;
end;

function  TRecordset.Get_ActiveConnection: OleVariant;
var
  InterfaceVariant : OleVariant;
begin
  InterfaceVariant := Recordset;
  Result := InterfaceVariant.ActiveConnection;
end;

function  TRecordset.Get_BOF: WordBool;
begin
  Result := Recordset.BOF;
end;

function  TRecordset.Get_Bookmark: OleVariant;
begin
  if not Active or BOF or EOF then
    result:= EmptyParam
  else
    result:= Recordset.Bookmark;
end;

procedure TRecordset.Set_Bookmark(pvBookmark: OleVariant);
begin        
  if Active and (pvBookmark<>EmptyParam) then begin
    Recordset.Bookmark := pvBookmark;
  end;
end;

function  TRecordset.Get_CacheSize: Integer;
begin
  Result := Recordset.CacheSize;
end;

procedure TRecordset.Set_CacheSize(pl: Integer);
begin
  Recordset.CacheSize := pl;
end;

function  TRecordset.Get_CursorType: CursorTypeEnum;
begin
  Result := Recordset.CursorType;
end;

procedure TRecordset.Set_CursorType(plCursorType: CursorTypeEnum);
begin
  if CursorType<>plCursorType then begin
    Close;
    Recordset.CursorType := plCursorType;
  end;
end;

function  TRecordset.Get_EOF: WordBool;
begin
  Result := Recordset.EOF;
end;

function  TRecordset.Get_Fields: IFields;
begin
  Result := Recordset.Fields as IFields;
end;
//------------------------------------------------------------------------------
function TRecordset.GetFieldItems: TFieldItems;
begin
  result:= nil;
  if not Active then
    exit;
    //DoAdoError(emNoRecordsetActive); hlkhkl
  //if Recordset.BOF and Recordset.EOF then
  //  exit;
  FFieldItems.Fields:= Fields;
  result:= FFieldItems;
end;
//------------------------------------------------------------------------------
function TRecordset.GetFieldValue(FieldName: string): OleVariant;
begin
  try
    result:= Fields[FieldName].Value;
  except     
    on E: Exception do
      DoAdoError(emGetFieldValue, [FieldName, E.Message])
  end;
end;
//------------------------------------------------------------------------------
procedure TRecordset.SetFieldValue(FieldName: string; Value: OleVariant);
begin
  try
    Fields[FieldName].Value:= Value;
  except
    on E: Exception do
      DoAdoError(emAssignFieldValue, [FieldName, E.Message])
  end;
end;
//------------------------------------------------------------------------------
function TRecordset.GetFieldAsStr(FieldName: string): string;
begin
  result:= VarToStr(GetFieldValue(FieldName));
end;
//------------------------------------------------------------------------------
procedure TRecordset.SetFieldAsStr(FieldName: string; const Value: string);
begin
  SetFieldValue(FieldName, Value);
end;
//------------------------------------------------------------------------------
function TRecordset.GetFieldAsInt(FieldName: string): integer;
var resVar: oleVAriant;
begin
  resVar:= GetFieldValue(FieldName) ;
  if VarIsNull(resVar) then
    result:= 0
  else
    result:= resVar;
end;
//------------------------------------------------------------------------------
procedure TRecordset.SetFieldAsInt(FieldName: string;
  const Value: integer);
begin
  SetFieldValue(FieldName, Value);
end;
//------------------------------------------------------------------------------
function TRecordset.GetFieldAsBool(FieldName: string): boolean;
var resVar: oleVAriant;
begin
  resVar:= GetFieldValue(FieldName) ;
  if VarIsNull(resVar) then
    result:= false
  else
    result:= resVar;
end;
//------------------------------------------------------------------------------
procedure TRecordset.SetFieldAsBool(FieldName: string;
  const Value: boolean);
begin
  SetFieldValue(FieldName, Value);
end;        
//------------------------------------------------------------------------------
function TRecordset.GetFieldAsDbl(FieldName: string): double;
var resVar: oleVAriant;
begin
  resVar:= GetFieldValue(FieldName) ;
  if VarIsNull(resVar) then
    result:= 0.0
  else
    result:= resVar;
end;
//------------------------------------------------------------------------------
procedure TRecordset.SetFieldAsDbl(FieldName: string; const Value: double);
begin
  SetFieldValue(FieldName, Value);
end;

function  TRecordset.Get_LockType: LockTypeEnum;
begin
  Result := Recordset.LockType;
end;

procedure TRecordset.Set_LockType(plLockType: LockTypeEnum);
begin
  if LockType<>plLockType then begin
    Close;
    Recordset.LockType := plLockType;
  end;
end;

function  TRecordset.Get_MaxRecords: Integer;
begin
  Result := Recordset.MaxRecords;
end;

procedure TRecordset.Set_MaxRecords(plMaxRecords: Integer);
begin
  Recordset.MaxRecords := plMaxRecords;
end;

function  TRecordset.Get_RecordCount: Integer;
begin
  if not Active then
    result:= -1
  else
    Result := Recordset.RecordCount;
  {if EOF and BOF then
    result:= 0
  else  }
end;

procedure TRecordset._Set_Source(const pvSource: IDispatch);
  { Warning: The property Source has a setter and a getter whose
  types do not match. Delphi was unable to generate a property of
  this sort and so is using a Variant to set the property instead. }
var
  InterfaceVariant: OleVariant;
begin
  InterfaceVariant := Recordset;
  InterfaceVariant.Source := pvSource;
end;

procedure TRecordset.Set_Source(const pvSource: WideString);
  { Warning: The property Source has a setter and a getter whose
  types do not match. Delphi was unable to generate a property of
  this sort and so is using a Variant to set the property instead. }
var
  InterfaceVariant: OleVariant;
begin
  InterfaceVariant := Recordset;
  InterfaceVariant.Source := pvSource;
end;

function  TRecordset.Get_Source: OleVariant;
var
  InterfaceVariant : OleVariant;
begin
  InterfaceVariant := Recordset;
  Result := InterfaceVariant.Source;
end;

function  TRecordset.Get_EditMode: EditModeEnum;
begin
  if EOF or BOF then
    result:= adEditNone
  else
    Result := Recordset.EditMode;
end;

function  TRecordset.Get_PageCount: Integer;
begin
  Result := Recordset.PageCount;
end;

function  TRecordset.Get_PageSize: Integer;
begin
  Result := Recordset.PageSize;
end;

procedure TRecordset.Set_PageSize(pl: Integer);
begin
  Recordset.PageSize := pl;
end;

function  TRecordset.Get_Sort: WideString;
begin
  Result := Recordset.Sort;
end;

procedure TRecordset.Set_Sort(const Criteria: WideString);
  { Warning: The property Sort has a setter and a getter whose
  types do not match. Delphi was unable to generate a property of
  this sort and so is using a Variant to set the property instead. }
var
  InterfaceVariant: OleVariant;
begin
  if (Criteria<>'') then
    CursorLocation:= adUseClient;
  InterfaceVariant := Recordset;
  InterfaceVariant.Sort := Criteria;
end;

function  TRecordset.Get_Status: Integer;
begin
  Result := Recordset.Status;
end;

function  TRecordset.Get_State: Integer;
begin
  Result := Recordset.State;
end;

function  TRecordset.Get_CursorLocation: CursorLocationEnum;
begin
  Result := Recordset.CursorLocation;
end;

procedure TRecordset.Set_CursorLocation(plCursorLoc: CursorLocationEnum);
begin
  if CursorLocation<>plCursorLoc then begin
    Close;
    Recordset.CursorLocation := plCursorLoc;
    if CursorLocation <> adUseClient then
      Sort:= ''; 
  end;
end;

function  TRecordset.Get_Collect(Index: OleVariant): OleVariant;
var
  InterfaceVariant : OleVariant;
begin
  InterfaceVariant := Recordset;
  Result := InterfaceVariant.Collect[Index];
end;

procedure TRecordset.Set_Collect(Index: OleVariant; pvar: OleVariant);
begin
  Recordset.Collect[Index] := pvar;
end;

function  TRecordset.Get_MarshalOptions: MarshalOptionsEnum;
begin
  if Active then
    FMarshalOptions:= Recordset.MarshalOptions;
  Result := FMarshalOptions;
end;

procedure TRecordset.Set_MarshalOptions(peMarshal: MarshalOptionsEnum);
begin
  FMarshalOptions:= peMarshal;
  if Active then
    Recordset.MarshalOptions := FMarshalOptions;
end;   

procedure TRecordset.UnlockRecord;
begin
  FRecordLocked:= false;
end;

procedure TRecordset.LockRecord;
var i: integer;
begin
  if not Active then
    exit;
  if LockType = adLockPessimistic then begin
    // Before assign value check that field can be updated
    for i:= 0 to Fields.Count - 1 do
      if (Fields[I].Attributes and adFldUpdatable) <> 0 then begin
        Fields[I].Value := Fields[I].Value;
        Break;
        FRecordLocked:= true;
      end;
  end;
end;

procedure TRecordset.AddNew;
begin
  AddNew(EmptyParam, EmptyParam);
end;

procedure TRecordset.AddNew(FieldList: OleVariant);
begin
  AddNew(FieldList, EmptyParam);
end;

procedure TRecordset.AddNew(FieldList: OleVariant; Values: OleVariant);
begin
  CheckActive;
  Recordset.AddNew(FieldList, Values);
end;

procedure TRecordset.CancelUpdate;
begin          
  if not Active then exit;
  if EditMode <> adEditNone then begin
    Recordset.CancelUpdate;
    if FRecordLocked then
      LockRecord;
  end;
end;

procedure TRecordset.Close;
begin
  if Active then begin
    Recordset.Close;
    FRecordLocked:= false;
  end;
end;

procedure TRecordset.Delete(AffectRecords: AffectEnum);
begin
  CheckActive;
  Recordset.Delete(AffectRecords);
end;

function  TRecordset.GetRows(Rows: Integer): OleVariant;
begin
  Result := GetRows(Rows, EmptyParam, EmptyParam);
end;

function  TRecordset.GetRows(Rows: Integer; Start: OleVariant): OleVariant;
begin
  Result := GetRows(Rows, Start, EmptyParam);
end;

function  TRecordset.GetRows(Rows: Integer; Start: OleVariant; Fields: OleVariant): OleVariant;
begin
  CheckActive;
  Result := Recordset.GetRows(Rows, Start, Fields);
end;

procedure TRecordset.Move(NumRecords: Integer);
begin
  Move(NumRecords, EmptyParam);
end;

procedure TRecordset.Move(NumRecords: Integer; Start: OleVariant);
begin
  CheckActive;
  Recordset.Move(NumRecords, Start);
end;

procedure TRecordset.MoveNext;
begin
  CheckActive;
  Recordset.MoveNext;
end;

procedure TRecordset.MovePrevious;
begin
  CheckActive;
  Recordset.MovePrevious;
end;

procedure TRecordset.MoveFirst;
begin
  CheckActive;
  if not Recordset.BOF then begin
    Recordset.MoveFirst;
  end;
end;

procedure TRecordset.MoveLast;
begin
  CheckActive;
  if not Recordset.EOF then begin
    Recordset.MoveLast;
  end;
end;

procedure TRecordset.Open(Source: OleVariant; ActiveConnection: OleVariant;
                          CursorType: CursorTypeEnum; LockType: LockTypeEnum; Options: Integer);
begin
  Close;
  Recordset.Open(Source, ActiveConnection, CursorType, LockType, Options);
  FCommandType:= Options;
  if VarType(Source) = varOleStr then
    FSource:= Source
  else
    FSource:= '';
end;
//------------------------------------------------------------------------------
procedure TRecordset.Open;
var NewRst: IRecordset;
begin
  if Active then
    exit;       
  CheckConnection;
  if Source <> '' then begin
    Open(Source, Connection.Connection, CursorType, LockType, CommandType);
  end
  else begin
    //if not Assigned(FCommand) then
      DoAdoError(emNoSource);
    //NewRst:= FCommand.Execute;
    //ConnectTo(NewRst);
  end;
end;
//------------------------------------------------------------------------------
procedure TRecordset.Requery;
begin
  Requery(Integer(adOptionUnspecified));
end;
//------------------------------------------------------------------------------
procedure TRecordset.Requery(Options: Integer);
begin
  if Active then begin
    Recordset.Requery(Options);
  end;
end;
//------------------------------------------------------------------------------
procedure TRecordset._xResync(AffectRecords: AffectEnum);
begin
  Recordset._xResync(AffectRecords);
end;
//------------------------------------------------------------------------------
procedure TRecordset.Update;
begin
  Update(EmptyParam, EmptyParam);
end;
//------------------------------------------------------------------------------
procedure TRecordset.Update(Fields: OleVariant);
begin
  Update(Fields, EmptyParam);
end;
//------------------------------------------------------------------------------
procedure TRecordset.Update(Fields: OleVariant; Values: OleVariant);
begin
  if not Active then exit;
  if EditMode <> adEditNone then begin
    Recordset.Update(Fields, Values);
    if FRecordLocked then
      LockRecord;
  end;
end;
//------------------------------------------------------------------------------
function  TRecordset._xClone: IRecordset;
begin
  Result := Recordset._xClone as IRecordset;
end;
//------------------------------------------------------------------------------
procedure TRecordset.UpdateBatch(AffectRecords: AffectEnum);
begin
  Recordset.UpdateBatch(AffectRecords);
end;
//------------------------------------------------------------------------------
procedure TRecordset.CancelBatch(AffectRecords: AffectEnum);
begin
  Recordset.CancelBatch(AffectRecords);
end;
//------------------------------------------------------------------------------
function  TRecordset.NextRecordset: IRecordset;
begin
  Result := NextRecordset(EmptyParam);
end;
//------------------------------------------------------------------------------
function  TRecordset.NextRecordset(out RecordsAffected: OleVariant): IRecordset;
begin
  Result := Recordset.NextRecordset(RecordsAffected) as IRecordset;
end;
//------------------------------------------------------------------------------
function  TRecordset.Supports(CursorOptions: CursorOptionEnum): WordBool;
begin
  Result := Recordset.Supports(CursorOptions);
end;        
//------------------------------------------------------------------------------
function TRecordset.FindFirst(const Criteria: WideString): boolean;
begin
  MoveFirst;
  result:= Find(Criteria, 0, adSearchForward);
end;
//------------------------------------------------------------------------------
function TRecordset.FindLast(const Criteria: WideString): boolean;
begin
  MoveLast;
  result:= Find(Criteria, 0, TOleEnum(adSearchBackward));
end;
//------------------------------------------------------------------------------
function TRecordset.FindNext(const Criteria: WideString): boolean;
begin
  result:= Find(Criteria, 0, adSearchForward);
end;
//------------------------------------------------------------------------------
function TRecordset.FindPrevious(const Criteria: WideString): boolean;
begin
  result:= Find(Criteria, 0, TOleEnum(adSearchBackward));
end;
//------------------------------------------------------------------------------
function TRecordset.Find(const Criteria: WideString; SkipRecords: Integer;
                          SearchDirection: SearchDirectionEnum): boolean;
begin
  result:= Find(Criteria, SkipRecords, SearchDirection, EmptyParam);
end;
//------------------------------------------------------------------------------
function TRecordset.Find(const Criteria: WideString; SkipRecords: Integer;
                          SearchDirection: SearchDirectionEnum; Start: OleVariant): boolean;
var bmrk: OleVariant;
begin
  CheckActive;
  if Supports(adBookmark) then
    bmrk:= Bookmark;
  Recordset.Find(Criteria, SkipRecords, SearchDirection, Start);
  result:= ((SearchDirection=TOleEnum(adSearchBackward)) and not BOF)
           or ((SearchDirection=adSearchForward) and not EOF);
  
  if Supports(adBookmark) and not result then
    Bookmark:= bmrk;
end;
//------------------------------------------------------------------------------
procedure TRecordset.WaitUntilNotReady;
begin
  if Assigned(Recordset) then
    while (Recordset.State = adStateConnecting) do
      Application.ProcessMessages;
end;
//------------------------------------------------------------------------------
function TRecordset.GetActive: boolean;
begin
  WaitUntilNotReady;
  Result := Assigned(Recordset)
    and ((adStateOpen and Recordset.State) <> 0);
end;
//------------------------------------------------------------------------------
procedure TRecordset.SetActive(const Value: boolean);
begin
  if Value then begin
    if not(csLoading in ComponentState) then
      Open;
  end
  else
    Close;
end;
//------------------------------------------------------------------------------
function TRecordset.GetFiltered: boolean;
begin
  result:= false;
  if VarType(Filter) = varOleStr then
    result:= VarToStr(Filter) <> '';
  result:= result or VarIsArray(Filter);
  result:= result or ((VarType(Filter) = varInteger) and (Filter <> adFilterNone));
end;
//------------------------------------------------------------------------------
function  TRecordset.Get_Filter: OleVariant;
begin
  Result:= FFilter;
end;
//------------------------------------------------------------------------------
procedure TRecordset.Set_Filter(Criteria: OleVariant);
begin
  //if VarIsClear(Criteria) then
  //  FFilter:= adFilterNone
  //else
  //  if (VarIsStr(Criteria) or VarIsArray(Criteria) or VarIsOrdinal(Criteria)) then
      FFilter:= Criteria
  //  else
  //    DoAdoError(emInvalidFilter);
end;
//------------------------------------------------------------------------------
procedure TRecordset.SetConnection(const Value: TConnection);
begin
  if Value<>Connection then begin
    Close;
    if Assigned(Value) then
      Value.FreeNotification(self);
    FConnection := Value;
  end;
end;     
//------------------------------------------------------------------------------
procedure TRecordset.SetCommand(const Value: TCommand);
begin
  if Value<>FCommand then begin
    Close;
    if Assigned(Value) then begin
      Value.FreeNotification(self);
      FSource:= '';//Value.CommandText;
      // ctiveConnection:= nil;
    end;
    FCommand := Value;
  end;
end;
//------------------------------------------------------------------------------
procedure TRecordset.SetSource(const Value: WideString);
begin
  if Source<>Value then begin
    close;
    FSource := Value;
    if Value<>'' then
      FCommand:= nil;
  end;
end;       
//------------------------------------------------------------------------------
function  TRecordset.GetSource: WideString;
begin
  {if Assigned(Command) then
    FSource:= Command.CommandText
  else}
    result:= FSource;
end;
//------------------------------------------------------------------------------
procedure TRecordset.SetCommandType(const Value: CommandTypeEnum);
begin
  if CommandType<>Value then begin
    Close;
    FCommandType := Value;
  end;
end;
//------------------------------------------------------------------------------
procedure TRecordset.CheckConnection;
begin
  if Connection = nil then
    DoAdoError(emNoConnection);
  if not Connection.Active then
    DoAdoError(emNoConnectionActive);
end;
//------------------------------------------------------------------------------
procedure TRecordset.CheckActive;
begin
  if not Active then
    DoAdoError(emNoRecordsetActive);
end;
//------------------------------------------------------------------------------
procedure TRecordset.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited;
  if (Operation = opRemove)  then begin
    if (AComponent = FConnection) then
      FConnection:= nil;
    if (AComponent = FCommand) then
      FCommand:= nil;
  end;
end;
//==============================================================================
{ TFieldItem }
//------------------------------------------------------------------------------
constructor TFieldItem.Create(Collection: TCollection);
begin
  inherited;
end;
//------------------------------------------------------------------------------
procedure TFieldItem.CheckField;
begin
  if not Assigned(FField) then begin
    DoAdoError(emNoField);
  end;
end;
//------------------------------------------------------------------------------
procedure TFieldItem.SetField(const Value: IField);
begin
  FField := Value;
end;
//------------------------------------------------------------------------------
function TFieldItem.GetDisplayName: string;
begin
  if Assigned(Field) then begin
    result:= Name;
  end;
  if result='' then
    result:= 'Field not set';
end;
//------------------------------------------------------------------------------
procedure TFieldItem.AppendChunk(Data: OleVariant);
begin
  CheckField;
  Field.AppendChunk(Data);
end;
//------------------------------------------------------------------------------
function TFieldItem.GetChunk(Length: Integer): OleVariant;
begin
  CheckField;
  result:= Field.GetChunk(Length);
end;
//------------------------------------------------------------------------------
procedure TFieldItem.Assign(Source: TPersistent);
begin
  inherited Assign(Source);
  if Source is TFieldItem then begin
    FField:= TFieldItem(Source).FField;
  end
end;
//------------------------------------------------------------------------------
function TFieldItem.GetActualSize: integer;
begin
  try
    result:= Field.ActualSize;
  except
    result:= -1;
  end;
end;
//------------------------------------------------------------------------------
function TFieldItem.GetAttributes: FieldAttributeEnum;
begin
  if Assigned(Field) then
    result:= Field.Attributes
  else
    result:= TOleEnum(adFldUnspecified);
end;
//------------------------------------------------------------------------------
function TFieldItem.GetDefinedSize: integer;
begin
  if Assigned(Field) then
    result:= Field.DefinedSize
  else
    result:= 0;
end;
//------------------------------------------------------------------------------
function TFieldItem.GetName: WideString;
begin   
  if Assigned(Field) then
    result:= Field.Name
  else
    result:= '';
end;
//------------------------------------------------------------------------------
function TFieldItem.GetNumericScale: byte;
begin
  if Assigned(Field) then
    result:= Field.NumericScale
  else
    result:= 0;
end;
//------------------------------------------------------------------------------
function TFieldItem.GetOriginalValue: OleVariant;
begin
  if Assigned(Field) then
    result:= Field.OriginalValue
  else
    result:= VarEmpty;
end;
//------------------------------------------------------------------------------
function TFieldItem.GetPrecision: byte;
begin
  if Assigned(Field) then
    result:= Field.Precision
  else
    result:= 0;
end;
//------------------------------------------------------------------------------
function TFieldItem.GetProperties: IProperties;
begin
  if Assigned(Field) then
    result:= Field.Properties
  else
    result:= nil;
end;
//------------------------------------------------------------------------------
function TFieldItem.GetType: DataTypeEnum;
begin
  if Assigned(Field) then
    result:= Field.Type_
  else
    result:= adEmpty;
end;
//------------------------------------------------------------------------------
function TFieldItem.GetUnderlyingValue: OleVariant;
begin
  CheckField;
  result:= Field.UnderlyingValue;
end;
//------------------------------------------------------------------------------
function TFieldItem.GetValue: OleVariant;
begin
  CheckField;
  result:= Field.Value;
end;
//------------------------------------------------------------------------------
procedure TFieldItem.SetValue(const Value: OleVariant);
begin
  CheckField;
  Field.Value:= Value;
end;
//==============================================================================
{ TFieldItems }
//------------------------------------------------------------------------------
constructor TFieldItems.Create;
begin
  inherited Create(TFieldItem);
end;
//------------------------------------------------------------------------------
function TFieldItems.InternalAdd: TFieldItem;
begin
  result:= inherited Add as TFieldItem;
end;
//------------------------------------------------------------------------------
function TFieldItems.Add: TFieldItem;
begin
  result:= nil; 
  DoAdoError(emAddField);
end;
//------------------------------------------------------------------------------
function TFieldItems.Insert(Index: Integer): TFieldItem;
begin
  result:= nil;
  DoAdoError(emInsertField);
end;
//------------------------------------------------------------------------------
function TFieldItems.GetItem(Index: Integer): TFieldItem;
begin
  result:= GetItem(Index) as TFieldItem;
end;
//------------------------------------------------------------------------------
procedure TFieldItems.SetItem(Index: Integer; Value: TFieldItem);
begin
  inherited SetItem(index, Value);
end;
//------------------------------------------------------------------------------
procedure TFieldItems.SetFields(const Value: IFields);
var i: integer;
begin
  FFields := Value;
  Clear;
  if assigned(Fields) then
    for i:= 0 to Fields.Count-1 do
      with InternalAdd do begin
        Field:= Fields[i] as IField;
      end;
end;
//==============================================================================
{ TRecordsetFields }
//------------------------------------------------------------------------------
constructor TRecordsetFields.Create(AOwner: TComponent);
begin
  inherited;
  FFieldItems:= TFieldItems.Create;
end;
//------------------------------------------------------------------------------
procedure TRecordsetFields.CheckRecordset;
begin
  if not Assigned(Recordset) then
    DoAdoError(emNoRecordset);
  if not Recordset.Active then
    DoAdoError(emNoRecordsetActive);
end;
//------------------------------------------------------------------------------
function TRecordsetFields.GetFieldItems: TFieldItems;
begin
  result:= nil;
  if not Assigned(Recordset) then
    exit;
  CheckRecordset;
  FFieldItems.Fields:= Recordset.Fields;
  result:= FFieldItems;
end;
//------------------------------------------------------------------------------
procedure TRecordsetFields.SetRecordset(const Value: TRecordset);
begin
  FRecordset := Value;
end;
//------------------------------------------------------------------------------
procedure TRecordsetFields.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited;
  if (AComponent = FRecordset) and (Operation = opRemove) then
    FRecordset:= nil;
end;
//------------------------------------------------------------------------------
procedure TRecordsetFields.CancelUpdate;
begin
  CheckRecordset;
  Recordset.CancelUpdate;
end;
//------------------------------------------------------------------------------
procedure TRecordsetFields.Update;
begin
  CheckRecordset;
  Recordset.Update;
end;
//------------------------------------------------------------------------------
function TRecordsetFields.GetFields: IFields;
begin
  CheckRecordset;
  result:= Recordset.Fields;
end;

end.
